@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Tue, 22 Jul 2003
-/// @version $Id: GUIDetectorBuilder.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: GUIDetectorBuilder.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Builds detectors for guisim
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -79,9 +79,10 @@ public:
      * @param[in] lane The lane the detector is placed at
      * @param[in] pos The position on the lane the detector is placed at
      * @param[in] splitByType Whether additional information split by vehicle classes shall be generated
+     * @param[in] show Whether to show the detector in the gui if available
      */
     virtual MSDetectorFileOutput* createInductLoop(const std::string& id,
-            MSLane* lane, SUMOReal pos, bool splitByType);
+            MSLane* lane, SUMOReal pos, bool splitByType, bool show = true);
 
 
     /** @brief Creates an instance of an e1 detector using the given values
@@ -97,21 +98,6 @@ public:
             MSLane* lane, SUMOReal pos, const std::string& od);
 
 
-#ifdef HAVE_INTERNAL
-    /** @brief Creates an instance of a mesoscopic e1 detector using the given values
-     *
-     * Simply calls the MEInductLoop constructor
-     *
-     * @param[in] id The id the detector shall have
-     * @param[in] s The segment the detector is placed at
-     * @param[in] pos ?
-     * @todo Position is not used, herein!?
-     */
-    virtual MEInductLoop* createMEInductLoop(const std::string& id,
-            MESegment* s, SUMOReal pos);
-#endif
-
-
     /** @brief Creates an instance of an e2-detector (areal detector) using the given values
      *
      * Simply calls the GUI_E2_ZS_Collector constructor
@@ -124,7 +110,7 @@ public:
      * @param[in] haltingSpeedThreshold Detector parameter: the speed a vehicle's speed must be below to be assigned as jammed
      * @param[in] jamDistThreshold Detector parameter: the distance between two vehicles in order to not count them to one jam
      */
-    virtual MSDetectorFileOutput* createSingleLaneE2Detector(const std::string& id,
+    virtual MSE2Collector* createSingleLaneE2Detector(const std::string& id,
             DetectorUsage usage, MSLane* lane, SUMOReal pos, SUMOReal length,
             SUMOTime haltingTimeThreshold,
             SUMOReal haltingSpeedThreshold,
