@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: GUIEdge.cpp 18467 2015-05-29 03:50:41Z behrisch $
+/// @version $Id: GUIEdge.cpp 20901 2016-06-07 11:50:11Z namdre $
 ///
 // A road/street connecting two junctions (gui-version)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -52,6 +52,7 @@
 #include <microsim/MSGlobals.h>
 #include <microsim/logging/CastingFunctionBinding.h>
 #include <microsim/logging/FunctionBinding.h>
+#include "GUITriggeredRerouter.h"
 #include "GUIEdge.h"
 #include "GUIVehicle.h"
 #include "GUINet.h"
@@ -59,12 +60,11 @@
 #include "GUIPerson.h"
 #include "GUIContainer.h"
 
-#ifdef HAVE_INTERNAL
 #include <mesogui/GUIMEVehicleControl.h>
+#include <mesogui/GUIMEVehicle.h>
 #include <mesosim/MESegment.h>
 #include <mesosim/MELoop.h>
 #include <mesosim/MEVehicle.h>
-#endif
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -128,8 +128,25 @@ GUIEdge::getTotalLength(bool includeInternal, bool eachLane) {
 Boundary
 GUIEdge::getBoundary() const {
     Boundary ret;
-    for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-        ret.add((*i)->getShape().getBoxBoundary());
+    if (getPurpose() != MSEdge::EDGEFUNCTION_DISTRICT) {
+        for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
+            ret.add((*i)->getShape().getBoxBoundary());
+        }
+    } else {
+        // take the starting coordinates of all follower edges and the endpoints
+        // of all successor edges
+        for (MSEdgeVector::const_iterator it = mySuccessors.begin(); it != mySuccessors.end(); ++it) {
+            const std::vector<MSLane*>& lanes = (*it)->getLanes();
+            for (std::vector<MSLane*>::const_iterator it_lane = lanes.begin(); it_lane != lanes.end(); ++it_lane) {
+                ret.add((*it_lane)->getShape().front());
+            }
+        }
+        for (MSEdgeVector::const_iterator it = myPredecessors.begin(); it != myPredecessors.end(); ++it) {
+            const std::vector<MSLane*>& lanes = (*it)->getLanes();
+            for (std::vector<MSLane*>::const_iterator it_lane = lanes.begin(); it_lane != lanes.end(); ++it_lane) {
+                ret.add((*it_lane)->getShape().back());
+            }
+        }
     }
     ret.grow(10);
     return ret;
@@ -170,23 +187,23 @@ GUIParameterTableWindow*
 GUIEdge::getParameterWindow(GUIMainWindow& app,
                             GUISUMOAbstractView& parent) {
     GUIParameterTableWindow* ret = 0;
-#ifdef HAVE_INTERNAL
     ret = new GUIParameterTableWindow(app, *this, 16);
     // add edge items
     ret->mkItem("length [m]", false, (*myLanes)[0]->getLength());
     ret->mkItem("allowed speed [m/s]", false, getAllowedSpeed());
-    ret->mkItem("occupancy [%]", true, new FunctionBinding<GUIEdge, SUMOReal>(this, &GUIEdge::getBruttoOccupancy, 100.));
+    ret->mkItem("brutto occupancy [%]", true, new FunctionBinding<GUIEdge, SUMOReal>(this, &GUIEdge::getBruttoOccupancy, 100.));
     ret->mkItem("mean vehicle speed [m/s]", true, new FunctionBinding<GUIEdge, SUMOReal>(this, &GUIEdge::getMeanSpeed));
     ret->mkItem("flow [veh/h/lane]", true, new FunctionBinding<GUIEdge, SUMOReal>(this, &GUIEdge::getFlow));
+    ret->mkItem("routing speed [m/s]", true, new FunctionBinding<MSEdge, SUMOReal>(this, &MSEdge::getRoutingSpeed));
     ret->mkItem("#vehicles", true, new CastingFunctionBinding<GUIEdge, SUMOReal, unsigned int>(this, &GUIEdge::getVehicleNo));
     ret->mkItem("vehicle ids", false, getVehicleIDs());
     // add segment items
     MESegment* segment = getSegmentAtPosition(parent.getPositionInformation());
     ret->mkItem("segment index", false, segment->getIndex());
     ret->mkItem("segment length [m]", false, segment->getLength());
-    ret->mkItem("segment allowed speed [m/s]", false, segment->getMaxSpeed());
-    ret->mkItem("segment jam threshold [%]", false, segment->getRelativeJamThreshold());
-    ret->mkItem("segment occupancy [%]", true, new FunctionBinding<MESegment, SUMOReal>(segment, &MESegment::getRelativeOccupancy));
+    ret->mkItem("segment allowed speed [m/s]", false, segment->getEdge().getSpeedLimit());
+    ret->mkItem("segment jam threshold [%]", false, segment->getRelativeJamThreshold() * 100);
+    ret->mkItem("segment brutto occupancy [%]", true, new FunctionBinding<MESegment, SUMOReal>(segment, &MESegment::getRelativeOccupancy, 100));
     ret->mkItem("segment mean vehicle speed [m/s]", true, new FunctionBinding<MESegment, SUMOReal>(segment, &MESegment::getMeanSpeed));
     ret->mkItem("segment flow [veh/h/lane]", true, new FunctionBinding<MESegment, SUMOReal>(segment, &MESegment::getFlow));
     ret->mkItem("segment #vehicles", true, new CastingFunctionBinding<MESegment, SUMOReal, size_t>(segment, &MESegment::getCarNumber));
@@ -194,10 +211,6 @@ GUIEdge::getParameterWindow(GUIMainWindow& app,
 
     // close building
     ret->closeBuilding();
-#else
-    UNUSED_PARAMETER(app);
-    UNUSED_PARAMETER(parent);
-#endif
     return ret;
 }
 
@@ -205,7 +218,9 @@ GUIEdge::getParameterWindow(GUIMainWindow& app,
 Boundary
 GUIEdge::getCenteringBoundary() const {
     Boundary b = getBoundary();
-    b.grow(20);
+    // ensure that vehicles and persons on the side are drawn even if the edge
+    // is outside the view
+    b.grow(10);
     return b;
 }
 
@@ -218,23 +233,19 @@ GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
     glPushName(getGlID());
     // draw the lanes
     for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-#ifdef HAVE_INTERNAL
         if (MSGlobals::gUseMesoSim) {
             setColor(s);
         }
-#endif
         GUILane* l = dynamic_cast<GUILane*>(*i);
         if (l != 0) {
             l->drawGL(s);
         }
     }
-#ifdef HAVE_INTERNAL
     if (MSGlobals::gUseMesoSim) {
         if (s.scale * s.vehicleSize.getExaggeration(s) > s.vehicleSize.minSize) {
             drawMesoVehicles(s);
         }
     }
-#endif
     glPopName();
     // (optionally) draw the name and/or the street name
     const bool drawEdgeName = s.edgeName.show && myFunction == EDGEFUNCTION_NORMAL;
@@ -287,15 +298,13 @@ GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
 }
 
 
-#ifdef HAVE_INTERNAL
 void
 GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
-    const GUIVisualizationTextSettings& nameSettings = s.vehicleName;
-    const SUMOReal exaggeration = s.vehicleSize.getExaggeration(s);
     GUIMEVehicleControl* vehicleControl = GUINet::getGUIInstance()->getGUIMEVehicleControl();
     if (vehicleControl != 0) {
         // draw the meso vehicles
         vehicleControl->secureVehicles();
+        AbstractMutex::ScopedLocker locker(myLock);
         size_t laneIndex = 0;
         MESegment::Queue queue;
         for (std::vector<MSLane*>::const_iterator msl = myLanes->begin(); msl != myLanes->end(); ++msl, ++laneIndex) {
@@ -304,20 +313,17 @@ GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
             SUMOReal segmentOffset = 0; // offset at start of current segment
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
                     segment != 0; segment = segment->getNextSegment()) {
-                const SUMOReal length = segment->getLength() * segment->getLengthGeometryFactor();
+                const SUMOReal length = segment->getLength();
                 if (laneIndex < segment->numQueues()) {
                     // make a copy so we don't have to worry about synchronization
                     queue = segment->getQueue(laneIndex);
-                    const SUMOReal avgCarSize = segment->getBruttoOccupancy() / segment->getCarNumber();
-                    const SUMOReal avgCarHalfSize = 0.5 * avgCarSize;
                     const size_t queueSize = queue.size();
                     SUMOReal vehiclePosition = segmentOffset + length;
                     // draw vehicles beginning with the leader at the end of the segment
                     SUMOReal xOff = 0;
                     for (size_t i = 0; i < queueSize; ++i) {
-                        MSBaseVehicle* veh = queue[queueSize - i - 1];
+                        GUIMEVehicle* veh = static_cast<GUIMEVehicle*>(queue[queueSize - i - 1]);
                         const SUMOReal vehLength = veh->getVehicleType().getLengthWithGap();
-                        setVehicleColor(s, veh);
                         while (vehiclePosition < segmentOffset) {
                             // if there is only a single queue for a
                             // multi-lane edge shift vehicles and start
@@ -326,28 +332,8 @@ GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
                             xOff += 2;
                         }
                         const Position p = l->geometryPositionAtOffset(vehiclePosition);
-                        const SUMOReal angle = -l->getShape().rotationDegreeAtOffset(l->interpolateLanePosToGeometryPos(vehiclePosition));
-                        glPushMatrix();
-                        glTranslated(p.x(), p.y(), 0);
-                        glRotated(angle, 0, 0, 1);
-                        glTranslated(xOff, 0, GLO_VEHICLE);
-                        glScaled(exaggeration, vehLength * exaggeration, 1);
-                        glBegin(GL_TRIANGLES);
-                        glVertex2d(0, 0);
-                        glVertex2d(0 - 1.25, 1);
-                        glVertex2d(0 + 1.25, 1);
-                        glEnd();
-                        glPopMatrix();
-                        if (nameSettings.show) {
-                            glPushMatrix();
-                            glRotated(angle, 0, 0, 1);
-                            glTranslated(xOff, 0, 0);
-                            glRotated(-angle, 0, 0, 1);
-                            GLHelper::drawText(veh->getID(),
-                                               l->geometryPositionAtOffset(vehiclePosition - 0.5 * vehLength),
-                                               GLO_MAX, nameSettings.size / s.scale, nameSettings.color);
-                            glPopMatrix();
-                        }
+                        const SUMOReal angle = l->getShape().rotationAtOffset(l->interpolateLanePosToGeometryPos(vehiclePosition));
+                        veh->drawOnPos(s, p, angle);
                         vehiclePosition -= vehLength;
                     }
                 }
@@ -406,22 +392,6 @@ GUIEdge::getBruttoOccupancy() const {
 }
 
 
-SUMOReal
-GUIEdge::getMeanSpeed() const {
-    SUMOReal v = 0;
-    SUMOReal no = 0;
-    for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this); segment != 0; segment = segment->getNextSegment()) {
-        SUMOReal vehNo = (SUMOReal) segment->getCarNumber();
-        v += vehNo * segment->getMeanSpeed();
-        no += vehNo;
-    }
-    if (no == 0) {
-        return getSpeedLimit();
-    }
-    return v / no;
-}
-
-
 SUMOReal
 GUIEdge::getAllowedSpeed() const {
     return (*myLanes)[0]->getSpeedLimit();
@@ -436,7 +406,73 @@ GUIEdge::getRelativeSpeed() const {
 
 void
 GUIEdge::setColor(const GUIVisualizationSettings& s) const {
-    GLHelper::setColor(s.edgeColorer.getScheme().getColor(getColorValue(s.edgeColorer.getActive())));
+    const GUIColorer& c = s.edgeColorer;
+    if (!setFunctionalColor(c.getActive()) && !setMultiColor(c)) {
+        GLHelper::setColor(c.getScheme().getColor(getColorValue(c.getActive())));
+    }
+}
+
+
+bool
+GUIEdge::setFunctionalColor(size_t activeScheme) const {
+    switch (activeScheme) {
+        case 9: {
+            const PositionVector& shape = getLanes()[0]->getShape();
+            SUMOReal hue = GeomHelper::naviDegree(shape.beginEndAngle()); // [0-360]
+            GLHelper::setColor(RGBColor::fromHSV(hue, 1., 1.));
+            return true;
+        }
+        default:
+            return false;
+    }
+}
+
+
+bool
+GUIEdge::setMultiColor(const GUIColorer& c) const {
+    const size_t activeScheme = c.getActive();
+    mySegmentColors.clear();
+    switch (activeScheme) {
+        case 10: // alternating segments
+            for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
+                    segment != 0; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(c.getScheme().getColor(segment->getIndex() % 2));
+            }
+            //std::cout << getID() << " scheme=" << c.getScheme().getName() << " schemeCols=" << c.getScheme().getColors().size() << " thresh=" << toString(c.getScheme().getThresholds()) << " segmentColors=" << mySegmentColors.size() << " [0]=" << mySegmentColors[0] << " [1]=" << mySegmentColors[1] <<  "\n";
+            return true;
+        case 11: // by segment jammed state
+            for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
+                    segment != 0; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(c.getScheme().getColor(segment->free() ? 0 : 1));
+            }
+            return true;
+        case 12: // by segment occupancy
+            for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
+                    segment != 0; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(c.getScheme().getColor(segment->getRelativeOccupancy()));
+            }
+            return true;
+        case 13: // by segment speed
+            for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
+                    segment != 0; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(c.getScheme().getColor(segment->getMeanSpeed()));
+            }
+            return true;
+        case 14: // by segment flow
+            for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
+                    segment != 0; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(c.getScheme().getColor(3600 * segment->getCarNumber() * segment->getMeanSpeed() / segment->getLength()));
+            }
+            return true;
+        case 15: // by segment relative speed
+            for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
+                    segment != 0; segment = segment->getNextSegment()) {
+                mySegmentColors.push_back(c.getScheme().getColor(segment->getMeanSpeed() / getAllowedSpeed()));
+            }
+            return true;
+        default:
+            return false;
+    }
 }
 
 
@@ -457,6 +493,8 @@ GUIEdge::getColorValue(size_t activeScheme) const {
             return getFlow();
         case 7:
             return getRelativeSpeed();
+        case 8:
+            return getRoutingSpeed();
     }
     return 0;
 }
@@ -490,65 +528,46 @@ GUIEdge::getSegmentAtPosition(const Position& pos) {
 }
 
 
+
 void
-GUIEdge::setVehicleColor(const GUIVisualizationSettings& s, MSBaseVehicle* veh) const {
-    const GUIColorer& c = s.vehicleColorer;
-    if (!GUIVehicle::setFunctionalColor(c.getActive(), veh)) {
-        GLHelper::setColor(c.getScheme().getColor(getVehicleColorValue(c.getActive(), veh)));
+GUIEdge::closeTraffic(const GUILane* lane) {
+    const std::vector<MSLane*>& lanes = getLanes();
+    const bool isClosed = lane->isClosed();
+    for (std::vector<MSLane*>::const_iterator i = lanes.begin(); i != lanes.end(); ++i) {
+        GUILane* l = dynamic_cast<GUILane*>(*i);
+        if (l->isClosed() == isClosed) {
+            l->closeTraffic(false);
+        }
     }
+    rebuildAllowedLanes();
 }
 
 
-SUMOReal
-GUIEdge::getVehicleColorValue(size_t activeScheme, MSBaseVehicle* veh) const {
-    switch (activeScheme) {
-        case 8:
-            return veh->getSpeed();
-        case 9:
-            return STEPS2TIME(veh->getWaitingTime());
-        case 10:
-            return 0; // invalid getLastLaneChangeOffset();
-        case 11:
-            return getVehicleMaxSpeed(veh);
-        case 12:
-            return 0; // invalid getCO2Emissions();
-        case 13:
-            return 0; // invalid getCOEmissions();
-        case 14:
-            return 0; // invalid getPMxEmissions();
-        case 15:
-            return 0; // invalid  getNOxEmissions();
-        case 16:
-            return 0; // invalid getHCEmissions();
-        case 17:
-            return 0; // invalid getFuelConsumption();
-        case 18:
-            return 0; // invalid getHarmonoise_NoiseEmissions();
-        case 19: // !!! unused!?
-            if (veh->getNumberReroutes() == 0) {
-                return -1;
-            }
-            return veh->getNumberReroutes();
-        case 20:
-            return 0; // invalid gSelected.isSelected(GLO_VEHICLE, getGlID());
-        case 21:
-            return 0; // invalid getBestLaneOffset();
-        case 22:
-            return 0; // invalid getAcceleration();
-        case 23:
-            return 0; // invalid getTimeGap();
+void
+GUIEdge::addRerouter() {
+    MSEdgeVector edges;
+    edges.push_back(this);
+    GUITriggeredRerouter* rr = new GUITriggeredRerouter(getID() + "_dynamic_rerouter", edges, 1, "", false,
+            GUINet::getGUIInstance()->getVisualisationSpeedUp());
+
+    MSTriggeredRerouter::RerouteInterval ri;
+    ri.begin = MSNet::getInstance()->getCurrentTimeStep();
+    ri.end = SUMOTime_MAX;
+    ri.edgeProbs.add(1, &MSTriggeredRerouter::mySpecialDest_keepDestination);
+    rr->myIntervals.push_back(ri);
+
+    // trigger rerouting for vehicles already on this edge
+    const std::vector<MSLane*>& lanes = getLanes();
+    for (std::vector<MSLane*>::const_iterator i = lanes.begin(); i != lanes.end(); ++i) {
+        const MSLane::VehCont& vehicles = (*i)->getVehiclesSecure();
+        for (MSLane::VehCont::const_iterator v = vehicles.begin(); v != vehicles.end(); ++v) {
+            if ((*v)->getLane() == (*i)) {
+                rr->notifyEnter(**v, MSMoveReminder::NOTIFICATION_JUNCTION);
+            } // else: this is the shadow during a continuous lane change
+        }
+        (*i)->releaseVehicles();
     }
-    return 0;
 }
 
-
-
-
-
-
-
-
-#endif
-
 /****************************************************************************/
 
