@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUILane.h 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: GUILane.h 20863 2016-06-03 09:27:22Z behrisch $
 ///
 // Representation of a lane in the micro simulation (gui-version)
 /****************************************************************************/
@@ -152,6 +152,20 @@ public:
     MSVehicle* removeVehicle(MSVehicle* remVehicle, MSMoveReminder::Notification notification, bool notify);
 
 
+    /** @brief Sets the information about a vehicle lapping into this lane
+     *
+     * This vehicle is added to myVehicles and may be distinguished from regular
+     * vehicles by the disparity between this lane and v->getLane()
+     * @param[in] v The vehicle which laps into this lane
+     * @return This lane's length
+     */
+    SUMOReal setPartialOccupation(MSVehicle* v);
+
+    /** @brief Removes the information about a vehicle lapping into this lane
+     * @param[in] v The vehicle which laps into this lane
+     */
+    void resetPartialOccupation(MSVehicle* v);
+
 
     /// @name inherited from GUIGlObject
     //@{
@@ -247,11 +261,12 @@ protected:
      * @param[in] veh The vehicle to be incorporated
      * @param[in] pos The position of the vehicle
      * @param[in] speed The speed of the vehicle
+     * @param[in] posLat The lateral position of the vehicle
      * @param[in] at
      * @param[in] notification The cause of insertion (i.e. departure, teleport, parking) defaults to departure
      * @see MSLane::incorporateVehicle
      */
-    virtual void incorporateVehicle(MSVehicle* veh, SUMOReal pos, SUMOReal speed,
+    virtual void incorporateVehicle(MSVehicle* veh, SUMOReal pos, SUMOReal speed, SUMOReal posLat,
                                     const MSLane::VehCont::iterator& at,
                                     MSMoveReminder::Notification notification = MSMoveReminder::NOTIFICATION_DEPARTED);
 
@@ -266,6 +281,8 @@ private:
     void drawLane2LaneConnections() const;
 
 
+    /// @brief add intermediate points at segment borders
+    PositionVector splitAtSegments(const PositionVector& shape);
 
 private:
     /// @brief gets the color value according to the current scheme index
@@ -298,6 +315,9 @@ private:
     /// The color of the shape parts (cached)
     mutable std::vector<RGBColor> myShapeColors;
 
+    /// @brief the meso segment index for each geometry segment
+    std::vector<int> myShapeSegments;
+
     /// @brief Half of lane width, for speed-up
     SUMOReal myHalfLaneWidth;
 
