@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Okt 2003
-/// @version $Id: GUI_E2_ZS_Collector.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: GUI_E2_ZS_Collector.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // The gui-version of the MS_E2_ZS_Collector
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -33,11 +33,6 @@
 #endif
 
 #include <microsim/output/MSE2Collector.h>
-#include <microsim/MSNet.h>
-#include <utils/geom/Position.h>
-#include <utils/geom/PositionVector.h>
-#include <utils/common/VectorHelper.h>
-#include <utils/common/ValueSource.h>
 #include "GUIDetectorWrapper.h"
 
 
