@@ -8,12 +8,12 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Tue, 06 Mar 2001
-/// @version $Id: MSEdge.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: MSEdge.cpp 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // A road/street connecting two junctions
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -40,20 +40,22 @@
 #include <utils/common/StringTokenizer.h>
 #include <utils/options/OptionsCont.h>
 #include "MSEdge.h"
+#include "MSInsertionControl.h"
 #include "MSJunction.h"
 #include "MSLane.h"
 #include "MSLaneChanger.h"
+#include "MSLaneChangerSublane.h"
 #include "MSGlobals.h"
+#include "MSNet.h"
 #include "MSVehicle.h"
+#include "MSLeaderInfo.h"
 #include "MSContainer.h"
 #include "MSEdgeWeightsStorage.h"
 #include <microsim/devices/MSDevice_Routing.h>
 
-#ifdef HAVE_INTERNAL
 #include <mesosim/MELoop.h>
 #include <mesosim/MESegment.h>
 #include <mesosim/MEVehicle.h>
-#endif
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -82,6 +84,7 @@ MSEdge::MSEdge(const std::string& id, int numericalID,
     myStreetName(streetName),
     myEdgeType(edgeType),
     myPriority(priority),
+    myWidth(0),
     myLength(-1.),
     myEmptyTraveltime(-1.),
     myAmDelayed(false),
@@ -109,19 +112,53 @@ MSEdge::initialize(const std::vector<MSLane*>* lanes) {
     myLanes = lanes;
     if (!lanes->empty()) {
         recalcCache();
-        if (myLanes->size() > 1) {
-            myLaneChanger = new MSLaneChanger(myLanes, OptionsCont::getOptions().getBool("lanechange.allow-swap"));
-        }
     }
     if (myFunction == EDGEFUNCTION_DISTRICT) {
         myCombinedPermissions = SVCAll;
     }
+    for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
+        myWidth += (*i)->getWidth();
+    }
+    if (MSGlobals::gLateralResolution > 0 || MSGlobals::gLaneChangeDuration > 0) {
+        SUMOReal widthBefore = 0;
+        for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
+            (*i)->setRightSideOnEdge(widthBefore, (int)mySublaneSides.size());
+            MSLeaderInfo ahead(*i);
+            for (int j = 0; j < ahead.numSublanes(); ++j) {
+                mySublaneSides.push_back(widthBefore + j * MSGlobals::gLateralResolution);
+            }
+            widthBefore += (*i)->getWidth();
+        }
+    }
 }
 
 
 void MSEdge::recalcCache() {
+    if (myLanes->empty()) {
+        return;
+    }
     myLength = myLanes->front()->getLength();
     myEmptyTraveltime = myLength / MAX2(getSpeedLimit(), NUMERICAL_EPS);
+
+    if (MSGlobals::gMesoTLSPenalty > 0) {
+        // add tls penalties to the minimum travel time
+        SUMOTime minPenalty = -1;
+        for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
+            MSLane* l = *i;
+            const MSLinkCont& lc = l->getLinkCont();
+            for (MSLinkCont::const_iterator j = lc.begin(); j != lc.end(); ++j) {
+                MSLink* link = *j;
+                if (minPenalty == -1) {
+                    minPenalty = link->getMesoTLSPenalty();
+                } else {
+                    minPenalty = MIN2(minPenalty, link->getMesoTLSPenalty());
+                }
+            }
+        }
+        if (minPenalty > 0) {
+            myEmptyTraveltime += STEPS2TIME(minPenalty);
+        }
+    }
 }
 
 
@@ -132,6 +169,7 @@ MSEdge::closeBuilding() {
         myAllowed[0]->push_back(*i);
         const MSLinkCont& lc = (*i)->getLinkCont();
         for (MSLinkCont::const_iterator j = lc.begin(); j != lc.end(); ++j) {
+            (*j)->initParallelLinks();
             MSLane* toL = (*j)->getLane();
             if (toL != 0) {
                 MSEdge& to = toL->getEdge();
@@ -161,6 +199,52 @@ MSEdge::closeBuilding() {
     }
     std::sort(mySuccessors.begin(), mySuccessors.end(), by_id_sorter());
     rebuildAllowedLanes();
+    // segment building depends on the finished list of successors (for multi-queue)
+    if (MSGlobals::gUseMesoSim && !myLanes->empty()) {
+        MSGlobals::gMesoNet->buildSegmentsFor(*this, OptionsCont::getOptions());
+    }
+}
+
+
+void
+MSEdge::buildLaneChanger() {
+    if (!myLanes->empty()) {
+        const bool allowSwap = OptionsCont::getOptions().getBool("lanechange.allow-swap");
+        const bool allowChanging = allowsLaneChanging();
+        if (MSGlobals::gLateralResolution > 0) {
+            // may always initiate sublane-change
+            myLaneChanger = new MSLaneChangerSublane(myLanes, allowChanging, allowSwap);
+        } else {
+            if (MSGlobals::gLaneChangeDuration > 0) {
+                myLaneChanger = new MSLaneChanger(myLanes, allowChanging, allowSwap);
+            } else if (myLanes->size() > 1 || canChangeToOpposite()) {
+                myLaneChanger = new MSLaneChanger(myLanes, allowChanging, allowSwap);
+            }
+        }
+    }
+}
+
+
+bool
+MSEdge::allowsLaneChanging() {
+    if (myFunction == EDGEFUNCTION_INTERNAL) {
+        // allow changing only if all links leading to this internal lane have priority
+        // or they are controlled by a traffic light
+        for (std::vector<MSLane*>::const_iterator it = myLanes->begin(); it != myLanes->end(); ++it) {
+            MSLane* pred = (*it)->getLogicalPredecessorLane();
+            MSLink* link = MSLinkContHelper::getConnectingLink(*pred, **it);
+            assert(link != 0);
+            LinkState state = link->getState();
+            if (state == LINKSTATE_MINOR
+                    || state == LINKSTATE_EQUAL
+                    || state == LINKSTATE_STOP
+                    || state == LINKSTATE_ALLWAY_STOP
+                    || state == LINKSTATE_DEADEND) {
+                return false;
+            }
+        }
+    }
+    return true;
 }
 
 
@@ -402,7 +486,21 @@ MSEdge::insertVehicle(SUMOVehicle& v, SUMOTime time, const bool checkOnly) const
                                "' is too high for the departure edge '" + getID() + "'.");
         }
     }
-#ifdef HAVE_INTERNAL
+    if (checkOnly && v.getEdge()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT) {
+        return true;
+    }
+    if (!checkOnly) {
+        std::string msg;
+        if (!v.hasValidRoute(msg)) {
+            if (MSGlobals::gCheckRoutes) {
+                throw ProcessError("Vehicle '" + v.getID() + "' has no valid route. " + msg);
+            } else if (v.getEdge()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT) {
+                WRITE_WARNING("Removing vehicle '" + pars.id + "' which has no valid route.");
+                MSNet::getInstance()->getInsertionControl().descheduleDeparture(&v);
+                return false;
+            }
+        }
+    }
     if (MSGlobals::gUseMesoSim) {
         SUMOReal pos = 0.0;
         switch (pars.departPosProcedure) {
@@ -446,13 +544,7 @@ MSEdge::insertVehicle(SUMOVehicle& v, SUMOTime time, const bool checkOnly) const
         }
         return result;
     }
-#else
-    UNUSED_PARAMETER(time);
-#endif
     if (checkOnly) {
-        if (v.getEdge()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT) {
-            return true;
-        }
         switch (v.getParameter().departLaneProcedure) {
             case DEPART_LANE_GIVEN:
             case DEPART_LANE_DEFAULT:
@@ -482,11 +574,18 @@ MSEdge::changeLanes(SUMOTime t) {
     }
     if (myFunction == EDGEFUNCTION_INTERNAL) {
         // allow changing only if all links leading to this internal lane have priority
+        // or they are controlled by a traffic light
         for (std::vector<MSLane*>::const_iterator it = myLanes->begin(); it != myLanes->end(); ++it) {
             MSLane* pred = (*it)->getLogicalPredecessorLane();
             MSLink* link = MSLinkContHelper::getConnectingLink(*pred, **it);
             assert(link != 0);
-            if (!link->havePriority()) {
+            LinkState state = link->getState();
+            if (state == LINKSTATE_MINOR
+                    || state == LINKSTATE_EQUAL
+                    || state == LINKSTATE_STOP
+                    || state == LINKSTATE_ALLWAY_STOP
+                    || state == LINKSTATE_ZIPPER
+                    || state == LINKSTATE_DEADEND) {
                 return;
             }
         }
@@ -498,7 +597,7 @@ MSEdge::changeLanes(SUMOTime t) {
 
 #ifdef HAVE_INTERNAL_LANES
 const MSEdge*
-MSEdge::getInternalFollowingEdge(MSEdge* followerAfterInternal) const {
+MSEdge::getInternalFollowingEdge(const MSEdge* followerAfterInternal) const {
     //@todo to be optimized
     for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
         MSLane* l = *i;
@@ -519,36 +618,50 @@ MSEdge::getInternalFollowingEdge(MSEdge* followerAfterInternal) const {
 #endif
 
 
+SUMOReal
+MSEdge::getMeanSpeed() const {
+    SUMOReal v = 0;
+    SUMOReal no = 0;
+    if (MSGlobals::gUseMesoSim) {
+        for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this); segment != 0; segment = segment->getNextSegment()) {
+            const SUMOReal vehNo = (SUMOReal) segment->getCarNumber();
+            v += vehNo * segment->getMeanSpeed();
+            no += vehNo;
+        }
+        if (no == 0) {
+            return getLength() / myEmptyTraveltime; // may include tls-penalty
+        }
+    } else {
+        for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
+            const SUMOReal vehNo = (SUMOReal)(*i)->getVehicleNumber();
+            v += vehNo * (*i)->getMeanSpeed();
+            no += vehNo;
+        }
+        if (no == 0) {
+            return getSpeedLimit();
+        }
+    }
+    return v / no;
+}
+
+
 SUMOReal
 MSEdge::getCurrentTravelTime(SUMOReal minSpeed) const {
     assert(minSpeed > 0);
     if (!myAmDelayed) {
         return myEmptyTraveltime;
     }
-    SUMOReal v = 0;
-#ifdef HAVE_INTERNAL
-    if (MSGlobals::gUseMesoSim) {
-        MESegment* first = MSGlobals::gMesoNet->getSegmentForEdge(*this);
-        unsigned segments = 0;
-        do {
-            v += first->getMeanSpeed();
-            first = first->getNextSegment();
-            segments++;
-        } while (first != 0);
-        v /= (SUMOReal) segments;
-    } else {
-#endif
-        for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-            v += (*i)->getMeanSpeed();
-        }
-        v /= (SUMOReal) myLanes->size();
-#ifdef HAVE_INTERNAL
-    }
-#endif
-    return getLength() / MAX2(minSpeed, v);
+    return getLength() / MAX2(minSpeed, getMeanSpeed());
 }
 
 
+SUMOReal
+MSEdge::getRoutingSpeed() const {
+    return MSDevice_Routing::getAssumedSpeed(this);
+}
+
+
+
 bool
 MSEdge::dictionary(const std::string& id, MSEdge* ptr) {
     DictType::iterator it = myDict.find(id);
@@ -576,22 +689,15 @@ MSEdge::dictionary(const std::string& id) {
 }
 
 
-MSEdge*
-MSEdge::dictionary(size_t id) {
-    assert(myEdges.size() > id);
-    return myEdges[id];
-}
-
-
 size_t
 MSEdge::dictSize() {
     return myDict.size();
 }
 
 
-size_t
-MSEdge::numericalDictSize() {
-    return myEdges.size();
+const MSEdgeVector&
+MSEdge::getAllEdges() {
+    return myEdges;
 }
 
 
@@ -666,6 +772,17 @@ MSEdge::getVehicleMaxSpeed(const SUMOVehicle* const veh) const {
 }
 
 
+void
+MSEdge::setMaxSpeed(SUMOReal val) const {
+    if (myLanes != 0) {
+        for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
+            (*i)->setMaxSpeed(val);
+        }
+    }
+}
+
+
+
 std::vector<MSTransportable*>
 MSEdge::getSortedPersons(SUMOTime timestep) const {
     std::vector<MSTransportable*> result(myPersons.begin(), myPersons.end());
@@ -694,7 +811,7 @@ MSEdge::transportable_by_position_sorter::operator()(const MSTransportable* cons
 
 const MSEdgeVector&
 MSEdge::getSuccessors(SUMOVehicleClass vClass) const {
-    if (vClass == SVC_IGNORING || !MSNet::getInstance()->hasPermissions()) {
+    if (vClass == SVC_IGNORING || !MSNet::getInstance()->hasPermissions() || myFunction == EDGEFUNCTION_DISTRICT) {
         return mySuccessors;
     }
 #ifdef HAVE_FOX
@@ -702,32 +819,37 @@ MSEdge::getSuccessors(SUMOVehicleClass vClass) const {
         MSDevice_Routing::lock();
     }
 #endif
-    std::map<SUMOVehicleClass, MSEdgeVector>::const_iterator i = myClassesSuccessorMap.find(vClass);
-    if (i != myClassesSuccessorMap.end()) {
-        // can use cached value
-#ifdef HAVE_FOX
-        if (MSDevice_Routing::isParallel()) {
-            MSDevice_Routing::unlock();
-        }
-#endif
-        return i->second;
-    } else {
+    std::map<SUMOVehicleClass, MSEdgeVector>::iterator i = myClassesSuccessorMap.find(vClass);
+    if (i == myClassesSuccessorMap.end()) {
+        // instantiate vector
+        myClassesSuccessorMap[vClass];
+        i = myClassesSuccessorMap.find(vClass);
         // this vClass is requested for the first time. rebuild all successors
         for (MSEdgeVector::const_iterator it = mySuccessors.begin(); it != mySuccessors.end(); ++it) {
-            const std::vector<MSLane*>* allowed = allowedLanes(*it, vClass);
-            if (allowed != 0 && allowed->size() > 0) {
-                myClassesSuccessorMap[vClass].push_back(*it);
+            if ((*it)->getPurpose() == EDGEFUNCTION_DISTRICT) {
+                i->second.push_back(*it);
+            } else {
+                const std::vector<MSLane*>* allowed = allowedLanes(*it, vClass);
+                if (allowed != 0 && allowed->size() > 0) {
+                    i->second.push_back(*it);
+                }
             }
         }
-#ifdef HAVE_FOX
-        if (MSDevice_Routing::isParallel()) {
-            MSDevice_Routing::unlock();
-        }
-#endif
-        return myClassesSuccessorMap[vClass];
     }
+    // can use cached value
+#ifdef HAVE_FOX
+    if (MSDevice_Routing::isParallel()) {
+        MSDevice_Routing::unlock();
+    }
+#endif
+    return i->second;
 }
 
 
+bool
+MSEdge::canChangeToOpposite() {
+    return !myLanes->empty() && myLanes->back()->getOpposite() != 0;
+}
+
 /****************************************************************************/
 
