@@ -6,7 +6,7 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSEdge.h 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: MSEdge.h 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // A road/street connecting two junctions
 /****************************************************************************/
@@ -147,6 +147,13 @@ public:
     /// @todo Has to be called after all edges were built and all connections were set...; Still, is not very nice
     void closeBuilding();
 
+    /// Has to be called after all sucessors and predecessors have been set (after closeBuilding())
+    void buildLaneChanger();
+
+    /* @brief returns whether initizliaing a lane change is permitted on this edge
+     * @note Has to be called after all sucessors and predecessors have been set (after closeBuilding())
+     */
+    bool allowsLaneChanging();
 
     /// @name Access to the edge's lanes
     /// @{
@@ -454,6 +461,10 @@ public:
         return MSNet::getInstance()->getTravelTime(edge, veh, time);
     }
 
+    /** @brief Returns the averaged speed used by the routing device
+     */
+    SUMOReal getRoutingSpeed() const;
+
 
     /// @name Methods releated to vehicle insertion
     /// @{
@@ -546,6 +557,18 @@ public:
         return myCombinedPermissions;
     }
 
+    /** @brief Returns the edges's width (sum over all lanes)
+     * @return This edges's width
+     */
+    SUMOReal getWidth() const {
+        return myWidth;
+    }
+
+    /// @brief Returns the right side offsets of this edge's sublanes
+    const std::vector<SUMOReal> getSubLaneSides() const {
+        return mySublaneSides;
+    }
+
     void rebuildAllowedLanes();
 
 
@@ -618,8 +641,15 @@ public:
         myAmDelayed = true;
     }
 
-    /// @brief get the mean speed for mesoscopic simulation
-    SUMOReal getMesoMeanSpeed() const;
+    bool hasLaneChanger() const {
+        return myLaneChanger != 0;
+    }
+
+    /// @brief whether this edge allows changing to the opposite direction edge
+    bool canChangeToOpposite();
+
+    /// @brief get the mean speed
+    SUMOReal getMeanSpeed() const;
 
     /// @brief grant exclusive access to the mesoscopic state
     virtual void lock() const {}
@@ -721,6 +751,7 @@ protected:
     /// @brief lookup in map and return 0 if not found
     const std::vector<MSLane*>* getAllowedLanesWithDefault(const AllowedLanesCont& c, const MSEdge* dest) const;
 
+
 protected:
     /// @brief This edge's numerical id
     const int myNumericalID;
@@ -784,6 +815,9 @@ protected:
     /// @brief the priority of the edge (used during network creation)
     const int myPriority;
 
+    /// Edge width [m]
+    SUMOReal myWidth;
+
     /// @brief the length of the edge (cached value for speedup)
     SUMOReal myLength;
 
@@ -796,6 +830,9 @@ protected:
     /// @brief whether this edge belongs to a roundabout
     bool myAmRoundabout;
 
+    /// @brief the right side for each sublane on this edge
+    std::vector<SUMOReal> mySublaneSides;
+
     /// @name Static edge container
     /// @{
 
