@@ -4,12 +4,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSVehicleContainer.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSVehicleContainer.h 20626 2016-05-03 12:49:04Z namdre $
 ///
 // vehicles sorted by their departures
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -34,7 +34,6 @@
 
 #include <vector>
 #include <iostream>
-#include "MSNet.h"
 
 
 // ===========================================================================
@@ -71,6 +70,9 @@ public:
     /// Adds a single vehicle
     void add(SUMOVehicle* veh);
 
+    /// Removes a single vehicle
+    void remove(SUMOVehicle* veh);
+
     /// Adds a container with vehicles departing at the given time
     void add(SUMOTime time, const VehicleVector& cont);
 
