@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 10. Dec 2003
-/// @version $Id: MSVehicleControl.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: MSVehicleControl.h 20593 2016-04-29 06:10:53Z namdre $
 ///
 // The class responsible for building and deletion of vehicles
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -160,17 +160,19 @@ public:
 
 
     /** @brief Returns the begin of the internal vehicle map
-     *
      * @return The begin of the internal vehicle map
      */
-    constVehIt loadedVehBegin() const;
+    constVehIt loadedVehBegin() const {
+        return myVehicleDict.begin();
+    }
 
 
     /** @brief Returns the end of the internal vehicle map
-     *
      * @return The end of the internal vehicle map
      */
-    constVehIt loadedVehEnd() const;
+    constVehIt loadedVehEnd() const {
+        return myVehicleDict.end();
+    }
     /// @}
 
 
@@ -234,12 +236,12 @@ public:
     unsigned int getQuota(SUMOReal frac = -1) const;
 
 
-    /** @brief Returns the number of build vehicles that have not been removed
-     * or need to wait for a passenger
+    /** @brief Returns the number of build vehicles that have not been removed or
+     * need to wait for a passenger or a container
      * @return Number of active vehicles
      */
     int getActiveVehicleCount() const {
-        return myLoadedVehNo - (myWaitingForPerson + myEndedVehNo);
+        return myLoadedVehNo - (myWaitingForPerson + myWaitingForContainer + myEndedVehNo);
     }
 
 
@@ -264,9 +266,7 @@ public:
     }
 
     /// @brief return the number of teleports (including collisions)
-    unsigned int getTeleportCount() const {
-        return myCollisions + myTeleportsJam + myTeleportsYield + myTeleportsWrongLane;
-    }
+    unsigned int getTeleportCount() const;
 
     /// @brief return the number of emergency stops
     unsigned int getEmergencyStops() const {
@@ -349,11 +349,19 @@ public:
     void insertVTypeIDs(std::vector<std::string>& into) const;
     /// @}
 
+    /// @brief Adds a vehicle to the list of waiting vehiclse to a given edge
     void addWaiting(const MSEdge* const edge, SUMOVehicle* vehicle);
 
+    /// @brief Removes a vehicle from the list of waiting vehicles to a given edge
     void removeWaiting(const MSEdge* const edge, SUMOVehicle* vehicle);
 
-    SUMOVehicle* getWaitingVehicle(const MSEdge* const edge, const std::set<std::string>& lines);
+    /* @brief returns a vehicle of the given lines that is waiting for a for a person or a container at this edge at the given positions
+     * @param[in] edge The edge at which the vehicle is positioned.
+     * @param[in] lines The set of lines from which at least one must correspond to the line of the vehicle
+     * @param[in] position The vehicle shall be positioned in the interval [position - t, position + t], where t is some tolerance
+     * @param[in] ridingID The id of the person or container that wants to ride
+     */
+    SUMOVehicle* getWaitingVehicle(const MSEdge* const edge, const std::set<std::string>& lines, const SUMOReal position, const std::string ridingID);
 
     /** @brief increases the count of vehicles waiting for a person to allow recogniztion of person related deadlocks
      */
@@ -367,6 +375,18 @@ public:
         myWaitingForPerson--;
     }
 
+    /** @brief increases the count of vehicles waiting for a container to allow recogniztion of container related deadlocks
+     */
+    void registerOneWaitingForContainer() {
+        myWaitingForContainer++;
+    }
+
+    /** @brief decreases the count of vehicles waiting for a container to allow recogniztion of container related deadlocks
+     */
+    void unregisterOneWaitingForContainer() {
+        myWaitingForContainer--;
+    }
+
     /// @brief registers one collision-related teleport
     void registerCollision() {
         myCollisions++;
@@ -392,26 +412,41 @@ public:
         myEmergencyStops++;
     }
 
-    /// @name State I/O (mesosim only)
+    /// @name State I/O
     /// @{
 
     /** @brief Sets the current state variables as loaded from the stream
      */
-    void setState(int runningVehNo, int endedVehNo, SUMOReal totalDepartureDelay, SUMOReal totalTravelTime);
+    void setState(int runningVehNo, int loadedVehNo, int endedVehNo, SUMOReal totalDepartureDelay, SUMOReal totalTravelTime);
 
     /** @brief Saves the current state into the given stream
      */
     void saveState(OutputDevice& out);
     /// @}
 
+    /// @brief avoid counting a vehicle twice if it was loaded from state and route input
+    void discountStateLoaded() {
+        myLoadedVehNo--;
+    }
 
-    /** @brief removes any vehicles that are still waiting
+
+    /** @brief informes about all waiting vehicles (deletion in destructor)
      */
     void abortWaiting();
 
     /// @brief compute (optional) random offset to the departure time
     SUMOTime computeRandomDepartOffset() const;
 
+    /// @brief return the maximum speed factor for all vehicles that ever entered the network
+    SUMOReal getMaxSpeedFactor() const {
+        return myMaxSpeedFactor;
+    }
+
+    /// @brief return the minimum deceleration capability for all vehicles that ever entered the network
+    SUMOReal getMinDeceleration() const {
+        return myMinDeceleration;
+    }
+
 private:
     /** @brief Checks whether the vehicle type (distribution) may be added
      *
@@ -492,18 +527,29 @@ protected:
     /// @brief Whether no vehicle type was loaded
     bool myDefaultVTypeMayBeDeleted;
 
-    /// the lists of waiting vehicles
+    /// @brief Whether no pedestrian type was loaded
+    bool myDefaultPedTypeMayBeDeleted;
+
+    /// the lists of waiting vehicles to a given edge
     std::map<const MSEdge* const, std::vector<SUMOVehicle*> > myWaiting;
 
-    /// the number of vehicles contained in myWaiting which can only continue by being triggered
+    /// the number of vehicles wainting for persons contained in myWaiting which can only continue by being triggered
     unsigned int myWaitingForPerson;
 
+    /// the number of vehicles wainting for containers contained in myWaiting which can only continue by being triggered
+    unsigned int myWaitingForContainer;
+
     /// @brief The scaling factor (especially for inc-dua)
     SUMOReal myScale;
 
     /// @brief The maximum random offset to be added to vehicles departure times (non-negative)
     SUMOTime myMaxRandomDepartOffset;
 
+    /// @brief The maximum speed factor for all vehicles in the network
+    SUMOReal myMaxSpeedFactor;
+
+    /// @brief The minimum deceleration capability for all vehicles in the network
+    SUMOReal myMinDeceleration;
 
 private:
     /// @brief invalidated copy constructor
