@@ -8,7 +8,7 @@
 /// @author  Robbin Blokpoel
 /// @author  Jakob Erdmann
 /// @date    Mon Feb 03 2014 10:13 CET
-/// @version $Id: MSE2Collector.cpp 20284 2016-03-22 21:07:03Z namdre $
+/// @version $Id: MSE2Collector.cpp 20687 2016-05-10 11:27:00Z behrisch $
 ///
 // An areal (along a single lane) detector
 /****************************************************************************/
@@ -134,7 +134,7 @@ MSE2Collector::notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason
         // vehicle is teleporting over the edge
         return false;
     }
-    if (veh.getPositionOnLane() - veh.getVehicleType().getLength() >= myEndPos) {
+    if (veh.getBackPositionOnLane(myLane) >= myEndPos) {
         // vehicle is beyond the detector
         return false;
     }
@@ -213,7 +213,7 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
 
         SUMOReal length = veh->getVehicleType().getLength();
         if (veh->getLane() == getLane()) {
-            if (veh->getPositionOnLane() - veh->getVehicleType().getLength() < myStartPos) {
+            if (veh->getBackPositionOnLane(myLane) < myStartPos) {
                 // vehicle entered detector partially
                 length -= (veh->getVehicleType().getLength() - (veh->getPositionOnLane() - myStartPos));
             }
@@ -224,8 +224,7 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
         } else {
             // ok, the vehicle is only partially still on the detector, has already moved to the
             //  next lane; still, we do not know how far away it is
-            assert(veh == myLane->getPartialOccupator());
-            length = myEndPos - myLane->getPartialOccupatorEnd();
+            length = myEndPos - veh->getBackPositionOnLane(myLane);
         }
         assert(length >= 0);
 
@@ -316,11 +315,11 @@ MSE2Collector::detectorUpdate(const SUMOTime /* step */) {
             (*(*i)->firstStandingVehicle)->getPositionOnLane()
             - (*(*i)->lastStandingVehicle)->getPositionOnLane()
             + (*(*i)->lastStandingVehicle)->getVehicleType().getLengthWithGap();
-        const MSVehicle* const occ = myLane->getPartialOccupator();
-        if (occ && occ == *(*i)->firstStandingVehicle && occ != *(*i)->lastStandingVehicle) {
-            jamLengthInMeters = myLane->getPartialOccupatorEnd() + occ->getVehicleType().getLengthWithGap()
-                                - (*(*i)->lastStandingVehicle)->getPositionOnLane()
-                                + (*(*i)->lastStandingVehicle)->getVehicleType().getLengthWithGap();
+        if ((*(*i)->firstStandingVehicle)->getLane() != myLane) {
+            // vehicle is partial occupator, discount the length that is not on
+            // this lane
+            jamLengthInMeters -= ((*(*i)->firstStandingVehicle)->getVehicleType().getLengthWithGap() -
+                                  (myLane->getLength() - (*(*i)->firstStandingVehicle)->getBackPositionOnLane(myLane)));
         }
         unsigned jamLengthInVehicles = (unsigned) distance((*i)->firstStandingVehicle, (*i)->lastStandingVehicle) + 1;
         // apply them to the statistics
@@ -570,11 +569,10 @@ MSE2Collector::getCurrentStartedHalts() const {
 
 int
 MSE2Collector::by_vehicle_position_sorter::operator()(const SUMOVehicle* v1, const SUMOVehicle* v2) {
-    const MSVehicle* const occ = myLane->getPartialOccupator();
-    if (v1 == occ) {
+    if (!v1->isFrontOnLane(myLane)) {
         return true;
     }
-    if (v2 == occ) {
+    if (!v2->isFrontOnLane(myLane)) {
         return false;
     }
     return v1->getPositionOnLane() > v2->getPositionOnLane();
