@@ -6,7 +6,7 @@
 /// @author  Bjoern Hendriks
 /// @author  Michael Behrisch
 /// @date    Mon, 10.05.2004
-/// @version $Id: MSXMLRawOut.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: MSXMLRawOut.cpp 20550 2016-04-26 10:57:45Z namdre $
 ///
 // Realises dumping the complete network state
 /****************************************************************************/
@@ -152,6 +152,10 @@ MSXMLRawOut::writeVehicle(OutputDevice& of, const MSBaseVehicle& veh) {
         if (!MSGlobals::gUseMesoSim) {
             const MSVehicle& microVeh = static_cast<const MSVehicle&>(veh);
             // microsim-specific stuff
+            if (MSGlobals::gLateralResolution > 0) {
+                const SUMOReal posLat = microVeh.getLateralPositionOnLane();
+                of.writeAttr(SUMO_ATTR_POSITION_LAT, posLat);
+            }
             const unsigned int personNumber = microVeh.getPersonNumber();
             if (personNumber > 0) {
                 of.writeAttr(SUMO_ATTR_PERSON_NUMBER, personNumber);
