@@ -8,12 +8,12 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Sept 2002
-/// @version $Id: MSTLLogicControl.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSTLLogicControl.cpp 20997 2016-06-20 07:00:52Z namdre $
 ///
 // A class that stores and controls tls and switching of their programs
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -227,7 +227,7 @@ MSTLLogicControl::TLSLogicVariants::executeOnSwitchActions() const {
 
 
 void
-MSTLLogicControl::TLSLogicVariants::addLink(MSLink* link, MSLane* lane, unsigned int pos) {
+MSTLLogicControl::TLSLogicVariants::addLink(MSLink* link, MSLane* lane, int pos) {
     for (std::map<std::string, MSTrafficLightLogic*>::iterator i = myVariants.begin(); i != myVariants.end(); ++i) {
         (*i).second->addLink(link, lane, pos);
     }
@@ -348,7 +348,7 @@ MSTLLogicControl::WAUTSwitchProcedure_GSP::adaptLogic(SUMOTime step) {
     } else {
         deltaToStretch = (cycleTimeTo - currentPosTo + gspTo);
     }
-    unsigned int newdur = (unsigned int) myTo->getPhase(stepTo).duration - diff + deltaToStretch;
+    const SUMOTime newdur = myTo->getPhase(stepTo).duration - diff + deltaToStretch;
     myTo->changeStepAndDuration(myControl, step, stepTo, newdur);
 }
 
@@ -424,8 +424,8 @@ MSTLLogicControl::WAUTSwitchProcedure_Stretch::cutLogic(SUMOTime step, SUMOTime
     SUMOTime toCut = 0;
     for (int i = 0; i < areasNo; i++) {
         StretchBereichDef def = getStretchBereichDef(myTo, i + 1);
-        SUMOTime begin = TIME2STEPS(def.begin);
-        unsigned int end = TIME2STEPS(def.end);
+        const SUMOTime begin = TIME2STEPS(def.begin);
+        const SUMOTime end = TIME2STEPS(def.end);
         size_t stepOfBegin = myTo->getIndexFromOffset(begin);
         if (stepOfBegin == actStep) {
             if (begin < startPos) {
@@ -841,6 +841,12 @@ MSTLLogicControl::getPhaseDef(const std::string& tlid) const {
 }
 
 
+void
+MSTLLogicControl::switchOffAll() {
+    for (std::map<std::string, TLSLogicVariants*>::const_iterator i = myLogics.begin(); i != myLogics.end(); ++i) {
+        (*i).second->addLogic("off",  new MSOffTrafficLightLogic(*this, (*i).first), true, true);
+    }
+}
 
 /****************************************************************************/
 
