@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Friedemann Wesner
 /// @date    Sept 2002
-/// @version $Id: MSTLLogicControl.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSTLLogicControl.h 20997 2016-06-20 07:00:52Z namdre $
 ///
 // A class that stores and controls tls and switching of their programs
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -36,10 +36,17 @@
 
 #include <vector>
 #include <map>
-#include "MSTrafficLightLogic.h"
-#include "MSSimpleTrafficLightLogic.h"
+#include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/Command.h>
-#include <utils/common/UtilExceptions.h>
+
+
+// ===========================================================================
+// class declarations
+// ===========================================================================
+class MSTrafficLightLogic;
+class MSLink;
+class MSLane;
+class MSPhaseDefinition;
 
 
 // ===========================================================================
@@ -134,8 +141,7 @@ public:
 
 
         void executeOnSwitchActions() const;
-        void addLink(MSLink* link, MSLane* lane, unsigned int pos);
-
+        void addLink(MSLink* link, MSLane* lane, int pos);
 
 
     private:
@@ -368,6 +374,9 @@ public:
      */
     std::pair<SUMOTime, MSPhaseDefinition> getPhaseDef(const std::string& tlid) const;
 
+    /// @brief switch all logic variants to 'off'
+    void switchOffAll();
+
 
 
 protected:
