@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: MSTrafficLightLogic.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSTrafficLightLogic.cpp 20521 2016-04-21 11:16:52Z behrisch $
 ///
 // The parent class for traffic light logics
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -36,16 +36,25 @@
 #include <map>
 #include <microsim/MSLink.h>
 #include <microsim/MSLane.h>
-#include "MSTrafficLightLogic.h"
 #include <microsim/MSEventControl.h>
-#include "MSTLLogicControl.h"
 #include <microsim/MSJunctionLogic.h>
+#include <microsim/MSNet.h>
+#include <microsim/MSEdge.h>
+#include <microsim/MSGlobals.h>
+#include "MSTLLogicControl.h"
+#include "MSTrafficLightLogic.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
 #endif // CHECK_MEMORY_LEAKS
 
 
+// ===========================================================================
+// static value definitions
+// ===========================================================================
+const MSTrafficLightLogic::LaneVector MSTrafficLightLogic::myEmptyLaneVector;
+
+
 // ===========================================================================
 // member method definitions
 // ===========================================================================
@@ -69,10 +78,10 @@ MSTrafficLightLogic::SwitchCommand::execute(SUMOTime t) {
         return 0;
     }
     //
-    bool isActive = myTLControl.isActive(myTLLogic);
-    size_t step1 = myTLLogic->getCurrentPhaseIndex();
-    SUMOTime next = myTLLogic->trySwitch(isActive);
-    size_t step2 = myTLLogic->getCurrentPhaseIndex();
+    const bool isActive = myTLControl.isActive(myTLLogic);
+    int step1 = myTLLogic->getCurrentPhaseIndex();
+    SUMOTime next = myTLLogic->trySwitch();
+    int step2 = myTLLogic->getCurrentPhaseIndex();
     if (step1 != step2) {
         if (isActive) {
             // execute any action connected to this tls
@@ -115,14 +124,26 @@ MSTrafficLightLogic::MSTrafficLightLogic(MSTLLogicControl& tlcontrol, const std:
 void
 MSTrafficLightLogic::init(NLDetectorBuilder&) {
     const Phases& phases = getPhases();
+    if (phases.size() > 0 && MSGlobals::gMesoTLSPenalty > 0) {
+        initMesoTLSPenalties();
+    }
     if (phases.size() > 1) {
-        // warn about transistions from green to red without intermediate yellow
+        bool haveWarnedAboutUnusedStates = false;
+        std::vector<bool> foundGreen(phases.front()->getState().size(), false);
         for (int i = 0; i < (int)phases.size(); ++i) {
+            // warn about unused stats
             const int iNext = (i + 1) % phases.size();
             const std::string& state1 = phases[i]->getState();
             const std::string& state2 = phases[iNext]->getState();
             assert(state1.size() == state2.size());
-            for (int j = 0; j < (int)MIN2(state1.size(), state2.size()); ++j) {
+            if (!haveWarnedAboutUnusedStates && state1.size() > myLanes.size()) {
+                WRITE_WARNING("Unused states in tlLogic '" + getID()
+                              + "', program '" + getProgramID() + "' in phase " + toString(i)
+                              + " after tl-index " + toString((int)myLanes.size() - 1));
+                haveWarnedAboutUnusedStates = true;
+            }
+            // warn about transitions from green to red without intermediate yellow
+            for (int j = 0; j < (int)MIN3(state1.size(), state2.size(), myLanes.size()); ++j) {
                 if ((LinkState)state2[j] == LINKSTATE_TL_RED
                         && ((LinkState)state1[j] == LINKSTATE_TL_GREEN_MAJOR
                             || (LinkState)state1[j] == LINKSTATE_TL_GREEN_MINOR)) {
@@ -136,6 +157,20 @@ MSTrafficLightLogic::init(NLDetectorBuilder&) {
                     }
                 }
             }
+            // warn about links that never get the green light
+            for (int j = 0; j < (int)state1.size(); ++j) {
+                LinkState ls = (LinkState)state1[j];
+                if (ls == LINKSTATE_TL_GREEN_MAJOR || ls == LINKSTATE_TL_GREEN_MINOR) {
+                    foundGreen[j] = true;
+                }
+            }
+        }
+        for (int j = 0; j < (int)foundGreen.size(); ++j) {
+            if (!foundGreen[j]) {
+                WRITE_WARNING("Missing green phase in tlLogic '" + getID()
+                              + "', program '" + getProgramID() + "' for tl-index " + toString(j));
+                break;
+            }
         }
     }
 }
@@ -149,16 +184,16 @@ MSTrafficLightLogic::~MSTrafficLightLogic() {
 
 // ----------- Handling of controlled links
 void
-MSTrafficLightLogic::addLink(MSLink* link, MSLane* lane, unsigned int pos) {
+MSTrafficLightLogic::addLink(MSLink* link, MSLane* lane, int pos) {
     // !!! should be done within the loader (checking necessary)
     myLinks.reserve(pos + 1);
-    while (myLinks.size() <= pos) {
+    while ((int)myLinks.size() <= pos) {
         myLinks.push_back(LinkVector());
     }
     myLinks[pos].push_back(link);
     //
     myLanes.reserve(pos + 1);
-    while (myLanes.size() <= pos) {
+    while ((int)myLanes.size() <= pos) {
         myLanes.push_back(LaneVector());
     }
     myLanes[pos].push_back(lane);
@@ -251,5 +286,54 @@ MSTrafficLightLogic::setCurrentDurationIncrement(SUMOTime delay) {
 }
 
 
+void MSTrafficLightLogic::initMesoTLSPenalties() {
+    // set mesoscopic time penalties
+    const Phases& phases = getPhases();
+    const int numLinks = (int)phases.front()->getState().size();
+    assert((int)myLinks.size() >= numLinks);
+    SUMOTime duration = 0;
+    std::vector<SUMOReal> redDuration(numLinks, 0);
+    std::vector<SUMOReal> penalty(numLinks, 0);
+    for (int i = 0; i < (int)phases.size(); ++i) {
+        const std::string& state = phases[i]->getState();
+        duration += phases[i]->duration;
+        // warn about transitions from green to red without intermediate yellow
+        for (int j = 0; j < numLinks; ++j) {
+            if ((LinkState)state[j] == LINKSTATE_TL_RED
+                    || (LinkState)state[j] == LINKSTATE_TL_REDYELLOW) {
+                redDuration[j] += STEPS2TIME(phases[i]->duration);
+            } else if (redDuration[j] > 0) {
+                penalty[j] += 0.5 * (redDuration[j] * redDuration[j] + redDuration[j]);
+                redDuration[j] = 0;
+            }
+        }
+    }
+    /// XXX penalty for wrap-around red phases is underestimated
+    for (int j = 0; j < numLinks; ++j) {
+        if (redDuration[j] > 0) {
+            penalty[j] += 0.5 * (redDuration[j] * redDuration[j] + redDuration[j]);
+            redDuration[j] = 0;
+        }
+    }
+    const SUMOReal durationSeconds = STEPS2TIME(duration);
+    std::set<const MSJunction*> controlledJunctions;
+    for (int j = 0; j < numLinks; ++j) {
+        for (int k = 0; k < (int)myLinks[j].size(); ++k) {
+            myLinks[j][k]->setMesoTLSPenalty(TIME2STEPS(MSGlobals::gMesoTLSPenalty * penalty[j] / durationSeconds));
+            controlledJunctions.insert(myLinks[j][k]->getLane()->getEdge().getFromJunction()); // MSLink::myJunction is not yet initialized
+            //std::cout << " tls=" << getID() << " link=" << j << " penalty=" << penalty[j] / durationSeconds << " durSecs=" << durationSeconds << "\n";
+        }
+    }
+    // initialize empty-net travel times
+    // XXX refactor after merging sharps (links know their incoming edge)
+    for (std::set<const MSJunction*>::iterator it = controlledJunctions.begin(); it != controlledJunctions.end(); ++it) {
+        const ConstMSEdgeVector incoming = (*it)->getIncoming();
+        for (ConstMSEdgeVector::const_iterator it_e = incoming.begin(); it_e != incoming.end(); ++it_e) {
+            const_cast<MSEdge*>(*it_e)->recalcCache();
+        }
+    }
+
+}
+
 /****************************************************************************/
 
