@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, May 2005
-/// @version $Id: MSCalibrator.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: MSCalibrator.cpp 21007 2016-06-20 12:47:05Z behrisch $
 ///
 // Calibrates the flow on an edge by removing an inserting vehicles
 /****************************************************************************/
@@ -126,7 +126,7 @@ MSCalibrator::init() {
 
 
 MSCalibrator::~MSCalibrator() {
-    if (myCurrentStateInterval != myIntervals.end()) {
+    if (myIntervals.size() > 0 && myCurrentStateInterval != myIntervals.end()) {
         writeXMLOutput();
     }
     for (std::vector<VehicleRemover*>::iterator it = myVehicleRemovers.begin(); it != myVehicleRemovers.end(); ++it) {
@@ -440,7 +440,7 @@ MSCalibrator::remainingVehicleCapacity(int laneIndex) const {
     }
     assert(laneIndex < (int)myEdge->getLanes().size());
     MSLane* lane = myEdge->getLanes()[laneIndex];
-    MSVehicle* last = lane->getLastVehicle();
+    MSVehicle* last = lane->getLastFullVehicle();
     const SUMOVehicleParameter* pars = myCurrentStateInterval->vehicleParameter;
     const MSVehicleType* vtype = MSNet::getInstance()->getVehicleControl().getVType(pars->vtypeid);
     const SUMOReal spacePerVehicle = vtype->getLengthWithGap() + myEdge->getSpeedLimit() * vtype->getCarFollowModel().getHeadwayTime();
