@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    29. March 2012
-/// @version $Id: NBAlgorithms_Ramps.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: NBAlgorithms_Ramps.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Algorithms for highway on-/off-ramps computation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -65,14 +65,23 @@ NBRampsComputer::computeRamps(NBNetBuilder& nb, OptionsCont& oc) {
     std::set<NBEdge*> incremented;
     // check whether on-off ramps shall be guessed
     if (oc.getBool("ramps.guess")) {
+        NBNodeCont& nc = nb.getNodeCont();
+        NBEdgeCont& ec = nb.getEdgeCont();
+        NBDistrictCont& dc = nb.getDistrictCont();
+        // collect join exclusions
         std::set<std::string> noramps;
         if (oc.isSet("ramps.unset")) {
             std::vector<std::string> edges = oc.getStringVector("ramps.unset");
             noramps.insert(edges.begin(), edges.end());
         }
-        NBNodeCont& nc = nb.getNodeCont();
-        NBEdgeCont& ec = nb.getEdgeCont();
-        NBDistrictCont& dc = nb.getDistrictCont();
+        // exclude roundabouts
+        const std::set<EdgeSet>& roundabouts = ec.getRoundabouts();
+        for (std::set<EdgeSet>::const_iterator it_round = roundabouts.begin();
+                it_round != roundabouts.end(); ++it_round) {
+            for (EdgeSet::const_iterator it_edge = it_round->begin(); it_edge != it_round->end(); ++it_edge) {
+                noramps.insert((*it_edge)->getID());
+            }
+        }
         // if an edge is part of two ramps, ordering is important
         std::set<NBNode*, Named::ComparatorIdLess> potOnRamps;
         std::set<NBNode*, Named::ComparatorIdLess> potOffRamps;
@@ -86,10 +95,10 @@ NBRampsComputer::computeRamps(NBNetBuilder& nb, OptionsCont& oc) {
             }
         }
         for (std::set<NBNode*, Named::ComparatorIdLess>::const_iterator i = potOnRamps.begin(); i != potOnRamps.end(); ++i) {
-            buildOnRamp(*i, nc, ec, dc, rampLength, dontSplit, incremented);
+            buildOnRamp(*i, nc, ec, dc, rampLength, dontSplit);
         }
         for (std::set<NBNode*, Named::ComparatorIdLess>::const_iterator i = potOffRamps.begin(); i != potOffRamps.end(); ++i) {
-            buildOffRamp(*i, nc, ec, dc, rampLength, dontSplit, incremented);
+            buildOffRamp(*i, nc, ec, dc, rampLength, dontSplit);
         }
     }
     // check whether on-off ramps are specified
@@ -106,7 +115,7 @@ NBRampsComputer::computeRamps(NBNetBuilder& nb, OptionsCont& oc) {
             }
             NBNode* from = e->getFromNode();
             if (from->getIncomingEdges().size() == 2 && from->getOutgoingEdges().size() == 1) {
-                buildOnRamp(from, nc, ec, dc, rampLength, dontSplit, incremented);
+                buildOnRamp(from, nc, ec, dc, rampLength, dontSplit);
             }
             // load edge again to check offramps
             e = ec.retrieve(*i);
@@ -116,7 +125,7 @@ NBRampsComputer::computeRamps(NBNetBuilder& nb, OptionsCont& oc) {
             }
             NBNode* to = e->getToNode();
             if (to->getIncomingEdges().size() == 1 && to->getOutgoingEdges().size() == 2) {
-                buildOffRamp(to, nc, ec, dc, rampLength, dontSplit, incremented);
+                buildOffRamp(to, nc, ec, dc, rampLength, dontSplit);
             }
         }
     }
@@ -148,7 +157,7 @@ NBRampsComputer::mayNeedOffRamp(NBNode* cur, SUMOReal minHighwaySpeed, SUMOReal
 
 
 void
-NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, SUMOReal rampLength, bool dontSplit, std::set<NBEdge*>& incremented) {
+NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, SUMOReal rampLength, bool dontSplit) {
     NBEdge* potHighway, *potRamp, *cont;
     getOnRampEdges(cur, &potHighway, &potRamp, &cont);
     // compute the number of lanes to append
@@ -157,15 +166,18 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
     NBEdge* first = cont;
     NBEdge* last = cont;
     NBEdge* curr = cont;
+    std::set<NBEdge*> incremented;
     if (toAdd > 0 && find(incremented.begin(), incremented.end(), cont) == incremented.end()) {
         SUMOReal currLength = 0;
         while (curr != 0 && currLength + curr->getGeometry().length() - POSITION_EPS < rampLength) {
             if (find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
                 curr->incLaneNo(toAdd);
-                curr->invalidateConnections(true);
+                if (curr->getStep() < NBEdge::LANES2LANES_USER) {
+                    curr->invalidateConnections(true);
+                }
                 incremented.insert(curr);
                 moveRampRight(curr, toAdd);
-                currLength += curr->getLength(); // !!! loaded length?
+                currLength += curr->getGeometry().length(); // !!! loaded length?
                 last = curr;
             }
             NBNode* nextN = curr->getToNode();
@@ -215,11 +227,15 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
         }
     }
     // set connections from ramp/highway to added ramp
-    if (!potHighway->addLane2LaneConnections(0, first, potRamp->getNumLanes(), MIN2(first->getNumLanes() - potRamp->getNumLanes(), potHighway->getNumLanes()), NBEdge::L2L_VALIDATED, true, true)) {
-        throw ProcessError("Could not set connection!");
+    if (potHighway->getStep() < NBEdge::LANES2LANES_USER) {
+        if (!potHighway->addLane2LaneConnections(0, first, potRamp->getNumLanes(), MIN2(first->getNumLanes() - potRamp->getNumLanes(), potHighway->getNumLanes()), NBEdge::L2L_VALIDATED, true, true)) {
+            throw ProcessError("Could not set connection!");
+        }
     }
-    if (!potRamp->addLane2LaneConnections(0, first, 0, potRamp->getNumLanes(), NBEdge::L2L_VALIDATED, true, true)) {
-        throw ProcessError("Could not set connection!");
+    if (potRamp->getStep() < NBEdge::LANES2LANES_USER) {
+        if (!potRamp->addLane2LaneConnections(0, first, 0, potRamp->getNumLanes(), NBEdge::L2L_VALIDATED, true, true)) {
+            throw ProcessError("Could not set connection!");
+        }
     }
     // patch ramp geometry
     PositionVector p = potRamp->getGeometry();
@@ -230,7 +246,7 @@ NBRampsComputer::buildOnRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDist
 
 
 void
-NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, SUMOReal rampLength, bool dontSplit, std::set<NBEdge*>& incremented) {
+NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDistrictCont& dc, SUMOReal rampLength, bool dontSplit) {
     NBEdge* potHighway, *potRamp, *prev;
     getOffRampEdges(cur, &potHighway, &potRamp, &prev);
     // compute the number of lanes to append
@@ -239,20 +255,27 @@ NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDis
     NBEdge* first = prev;
     NBEdge* last = prev;
     NBEdge* curr = prev;
+    std::set<NBEdge*> incremented;
     if (toAdd > 0 && find(incremented.begin(), incremented.end(), prev) == incremented.end()) {
         SUMOReal currLength = 0;
         while (curr != 0 && currLength + curr->getGeometry().length() - POSITION_EPS < rampLength) {
             if (find(incremented.begin(), incremented.end(), curr) == incremented.end()) {
                 curr->incLaneNo(toAdd);
-                curr->invalidateConnections(true);
+                if (curr->getStep() < NBEdge::LANES2LANES_USER) {
+                    curr->invalidateConnections(true);
+                }
                 incremented.insert(curr);
                 moveRampRight(curr, toAdd);
-                currLength += curr->getLength(); // !!! loaded length?
+                currLength += curr->getGeometry().length(); // !!! loaded length?
                 last = curr;
             }
             NBNode* prevN = curr->getFromNode();
             if (prevN->getIncomingEdges().size() == 1) {
                 curr = prevN->getIncomingEdges()[0];
+                if (curr->getStep() < NBEdge::LANES2LANES_USER && toAdd != 0) {
+                    // curr might be an onRamp. In this case connections need to be rebuilt
+                    curr->invalidateConnections();
+                }
                 if (curr->getNumLanes() != firstLaneNumber) {
                     // the number of lanes changes along the computation; we'll stop...
                     curr = 0;
@@ -297,16 +320,17 @@ NBRampsComputer::buildOffRamp(NBNode* cur, NBNodeCont& nc, NBEdgeCont& ec, NBDis
         }
     }
     // set connections from added ramp to ramp/highway
-    if (!first->addLane2LaneConnections(potRamp->getNumLanes(), potHighway, 0, MIN2(first->getNumLanes() - 1, potHighway->getNumLanes()), NBEdge::L2L_VALIDATED, true)) {
-        throw ProcessError("Could not set connection!");
-    }
-    if (!first->addLane2LaneConnections(0, potRamp, 0, potRamp->getNumLanes(), NBEdge::L2L_VALIDATED, false)) {
-        throw ProcessError("Could not set connection!");
+    if (first->getStep() < NBEdge::LANES2LANES_USER) {
+        if (!first->addLane2LaneConnections(potRamp->getNumLanes(), potHighway, 0, MIN2(first->getNumLanes() - 1, potHighway->getNumLanes()), NBEdge::L2L_VALIDATED, true)) {
+            throw ProcessError("Could not set connection!");
+        }
+        if (!first->addLane2LaneConnections(0, potRamp, 0, potRamp->getNumLanes(), NBEdge::L2L_VALIDATED, false)) {
+            throw ProcessError("Could not set connection!");
+        }
     }
     // patch ramp geometry
     PositionVector p = potRamp->getGeometry();
-    p.pop_front();
-    p.push_front(first->getLaneShape(0)[-1]);
+    p[0] = first->getLaneShape(0)[-1];
     potRamp->setGeometry(p);
 }
 
@@ -410,8 +434,8 @@ bool
 NBRampsComputer::fulfillsRampConstraints(
     NBEdge* potHighway, NBEdge* potRamp, NBEdge* other, SUMOReal minHighwaySpeed, SUMOReal maxRampSpeed,
     const std::set<std::string>& noramps) {
-    // do not build ramps on rail edges
-    if (isRailway(potHighway->getPermissions()) || isRailway(potRamp->getPermissions())) {
+    // check modes that are not appropriate for rampsdo not build ramps on rail edges
+    if (hasWrongMode(potHighway) || hasWrongMode(potRamp) || hasWrongMode(other)) {
         return false;
     }
     // do not build ramps on connectors
@@ -457,5 +481,20 @@ NBRampsComputer::fulfillsRampConstraints(
 }
 
 
+bool
+NBRampsComputer::hasWrongMode(NBEdge* edge) {
+    // must allow passenger vehicles
+    if ((edge->getPermissions() & SVC_PASSENGER) == 0) {
+        return true;
+    }
+    // must not have a green verge or a lane that is only for soft modes
+    for (int i = 0; i < (int)edge->getNumLanes(); ++i) {
+        if ((edge->getPermissions(i) & ~(SVC_PEDESTRIAN | SVC_BICYCLE)) == 0) {
+            return true;
+        }
+    }
+    return false;
+}
+
 /****************************************************************************/
 
