@@ -5,7 +5,7 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdgeCont.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: NBEdgeCont.cpp 21090 2016-07-01 18:50:17Z namdre $
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
@@ -41,6 +41,7 @@
 #include <utils/common/MsgHandler.h>
 #include <utils/common/ToString.h>
 #include <utils/common/TplConvert.h>
+#include <utils/common/IDSupplier.h>
 #include <utils/options/OptionsCont.h>
 #include "NBNetBuilder.h"
 #include "NBEdgeCont.h"
@@ -608,7 +609,23 @@ NBEdgeCont::computeLanes2Edges() {
 void
 NBEdgeCont::recheckLanes() {
     for (EdgeCont::iterator i = myEdges.begin(); i != myEdges.end(); i++) {
-        (*i).second->recheckLanes();
+        i->second->recheckLanes();
+        // check opposites
+        if (i->second->getNumLanes() > 0) {
+            const std::string& oppositeID = i->second->getLanes().back().oppositeID;
+            if (oppositeID != "" && oppositeID != "-") {
+                const NBEdge* oppEdge = retrieve(oppositeID.substr(0, oppositeID.rfind("_")));
+                if (oppEdge == 0) {
+                    throw ProcessError("Unknown opposite lane '" + oppositeID + "' for edge '" + i->second->getID() + "'!");
+                }
+                if (fabs(oppEdge->getLoadedLength() - i->second->getLoadedLength()) > POSITION_EPS) {
+                    throw ProcessError("Opposite lane '" + oppositeID + "' differs in length from edge '" + i->second->getID() + "'!");
+                }
+                if (oppEdge->getFromNode() != i->second->getToNode() || oppEdge->getToNode() != i->second->getFromNode()) {
+                    throw ProcessError("Opposite lane '" + oppositeID + "' does not connect the same nodes as edge '" + i->second->getID() + "'!");
+                }
+            }
+        }
     }
 }
 
@@ -732,6 +749,36 @@ NBEdgeCont::joinSameNodeConnectingEdges(NBDistrictCont& dc,
 }
 
 
+void
+NBEdgeCont::guessOpposites() {
+    //@todo magic values
+    const SUMOReal distanceThreshold = 7;
+    for (EdgeCont::iterator i = myEdges.begin(); i != myEdges.end(); ++i) {
+        NBEdge* edge = i->second;
+        const int numLanes = edge->getNumLanes();
+        if (numLanes > 0) {
+            NBEdge::Lane& lastLane = edge->getLaneStruct(numLanes - 1);
+            if (lastLane.oppositeID == "") {
+                NBEdge* opposite = 0;
+                //SUMOReal minOppositeDist = std::numeric_limits<SUMOReal>::max();
+                for (EdgeVector::const_iterator j = edge->getToNode()->getOutgoingEdges().begin(); j != edge->getToNode()->getOutgoingEdges().end(); ++j) {
+                    if ((*j)->getToNode() == edge->getFromNode() && !(*j)->getLanes().empty()) {
+                        const SUMOReal distance = VectorHelper<SUMOReal>::maxValue(lastLane.shape.distances((*j)->getLanes().back().shape));
+                        if (distance < distanceThreshold) {
+                            //minOppositeDist = distance;
+                            opposite = *j;
+                        }
+                    }
+                }
+                if (opposite != 0) {
+                    lastLane.oppositeID = opposite->getLaneID(opposite->getNumLanes() - 1);
+                }
+            }
+        }
+    }
+}
+
+
 void
 NBEdgeCont::recheckLaneSpread() {
     for (EdgeCont::iterator i = myEdges.begin(); i != myEdges.end(); ++i) {
@@ -947,10 +994,12 @@ NBEdgeCont::getRoundabouts() const {
 
 void
 NBEdgeCont::addRoundabout(const EdgeSet& roundabout) {
-    if (find(myRoundabouts.begin(), myRoundabouts.end(), roundabout) != myRoundabouts.end()) {
-        WRITE_WARNING("Ignoring duplicate roundabout: " + toString(roundabout));
-    } else {
-        myRoundabouts.insert(roundabout);
+    if (roundabout.size() > 0) {
+        if (find(myRoundabouts.begin(), myRoundabouts.end(), roundabout) != myRoundabouts.end()) {
+            WRITE_WARNING("Ignoring duplicate roundabout: " + toString(roundabout));
+        } else {
+            myRoundabouts.insert(roundabout);
+        }
     }
 }
 
@@ -1044,4 +1093,24 @@ NBEdgeCont::guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed,
 }
 
 
+int
+NBEdgeCont::mapToNumericalIDs() {
+    IDSupplier idSupplier("", getAllNames());
+    EdgeVector toChange;
+    for (EdgeCont::iterator it = myEdges.begin(); it != myEdges.end(); it++) {
+        try {
+            TplConvert::_str2int(it->first);
+        } catch (NumberFormatException&) {
+            toChange.push_back(it->second);
+        }
+    }
+    for (EdgeVector::iterator it = toChange.begin(); it != toChange.end(); ++it) {
+        NBEdge* edge = *it;
+        myEdges.erase(edge->getID());
+        edge->setID(idSupplier.getNext());
+        myEdges[edge->getID()] = edge;
+    }
+    return (int)toChange.size();
+}
+
 /****************************************************************************/
