@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    09.05.2011
-/// @version $Id: NBFrame.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: NBFrame.cpp 20975 2016-06-15 13:02:40Z palcraft $
 ///
 // Sets and checks options for netbuild
 /****************************************************************************/
@@ -65,6 +65,10 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.addSynonyme("default.lanenumber", "lanenumber", true);
     oc.addDescription("default.lanenumber", "Building Defaults", "The default number of lanes in an edge");
 
+    oc.doRegister("default.lanewidth", new Option_Float(NBEdge::UNSPECIFIED_WIDTH));
+    oc.addSynonyme("default.lanewidth", "lanewidth", true);
+    oc.addDescription("default.lanewidth", "Building Defaults", "The default width of lanes");
+
     oc.doRegister("default.speed", 'S', new Option_Float((SUMOReal) 13.9));
     oc.addSynonyme("default.speed", "speed", true);
     oc.addDescription("default.speed", "Building Defaults", "The default speed on an edge (in m/s)");
@@ -86,6 +90,9 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.doRegister("no-internal-links", new Option_Bool(false)); // !!! not described
     oc.addDescription("no-internal-links", "Processing", "Omits internal links");
 
+    oc.doRegister("numerical-ids", new Option_Bool(false));
+    oc.addDescription("numerical-ids", "Processing", "Remaps alphanumerical IDs of nodes and edges to ensure that all IDs are integers");
+
     if (!forNetgen) {
         oc.doRegister("dismiss-vclasses", new Option_Bool(false));
         oc.addDescription("dismiss-vclasses", "Processing", "Removes vehicle class restrictions from imported edges");
@@ -156,6 +163,9 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.addSynonyme("roundabouts.guess", "guess-roundabouts", true);
     oc.addDescription("roundabouts.guess", "Processing", "Enable roundabout-guessing");
 
+    oc.doRegister("opposites.guess", new Option_Bool(false));
+    oc.addDescription("opposites.guess", "Processing", "Enable guessing of opposite direction lanes usable for overtaking");
+
     oc.doRegister("lefthand", new Option_Bool(false));
     oc.addDescription("lefthand", "Processing", "Assumes left-hand traffic on the network");
 
@@ -180,6 +190,9 @@ NBFrame::fillOptions(bool forNetgen) {
 
         oc.doRegister("speed.factor", new Option_Float(1));
         oc.addDescription("speed.factor", "Processing", "Modifies all edge speeds by multiplying by FLOAT");
+
+        oc.doRegister("speed.minimum", new Option_Float());
+        oc.addDescription("speed.minimum", "Processing", "Modifies all edge speeds to at least FLOAT");
     }
 
     oc.doRegister("junctions.corner-detail", new Option_Integer(0));
@@ -370,7 +383,6 @@ NBFrame::fillOptions(bool forNetgen) {
     oc.addSynonyme("keep-nodes-unregulated.district-nodes", "keep-unregulated.district-nodes");
     oc.addDescription("keep-nodes-unregulated.district-nodes", "Unregulated Nodes", "Do not regulate district nodes");
 
-
     // ramp guessing options
     if (!forNetgen) {
         oc.doRegister("ramps.guess", new Option_Bool(false));
