@@ -4,13 +4,13 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mar 2011
-/// @version $Id: NBLoadedSUMOTLDef.cpp 19535 2015-12-05 13:47:18Z behrisch $
+/// @version $Id: NBLoadedSUMOTLDef.cpp 20551 2016-04-26 11:19:47Z namdre $
 ///
 // A complete traffic light logic loaded from a sumo-net. (opted to reimplement
 // since NBLoadedTLDef is quite vissim specific)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -77,9 +77,8 @@ NBLoadedSUMOTLDef::~NBLoadedSUMOTLDef() {
 
 
 NBTrafficLightLogic*
-NBLoadedSUMOTLDef::myCompute(const NBEdgeCont& ec, unsigned int brakingTimeSeconds) {
+NBLoadedSUMOTLDef::myCompute(unsigned int brakingTimeSeconds) {
     // @todo what to do with those parameters?
-    UNUSED_PARAMETER(ec);
     UNUSED_PARAMETER(brakingTimeSeconds);
     myTLLogic->closeBuilding();
     patchIfCrossingsAdded();
@@ -109,12 +108,6 @@ NBLoadedSUMOTLDef::addConnection(NBEdge* from, NBEdge* to, int fromLane, int toL
 }
 
 
-void
-NBLoadedSUMOTLDef::setTLControllingInformation(const NBEdgeCont&) const {
-    setTLControllingInformation();
-}
-
-
 void
 NBLoadedSUMOTLDef::setTLControllingInformation() const {
     // if nodes have been removed our links may have been invalidated as well
@@ -131,7 +124,9 @@ NBLoadedSUMOTLDef::setTLControllingInformation() const {
                                "' with " + toString(myTLLogic->getNumLinks()) + " links.");
         }
         NBEdge* edge = c.getFrom();
-        edge->setControllingTLInformation(c, getID());
+        if (edge != 0) {
+            edge->setControllingTLInformation(c, getID());
+        }
     }
 }
 
@@ -333,31 +328,25 @@ NBLoadedSUMOTLDef::patchIfCrossingsAdded() {
                 // patch states for the newly added crossings
 
                 // collect edges
-                assert(size > 0);
                 EdgeVector fromEdges(size, 0);
                 EdgeVector toEdges(size, 0);
-                for (NBConnectionVector::const_iterator it = myControlledLinks.begin(); it != myControlledLinks.end(); it++) {
-                    const NBConnection& c = *it;
-                    if (c.getTLIndex() != NBConnection::InvalidTlIndex) {
-                        if (c.getTLIndex() >= (int)size) {
-                            throw ProcessError("Invalid linkIndex " + toString(c.getTLIndex()) + " for traffic light '" + getID() +
-                                               "' with " + toString(size) + " links.");
-                        }
-
-
-                        fromEdges[c.getTLIndex()] = c.getFrom();
-                        toEdges[c.getTLIndex()] = c.getTo();
-                    }
-                }
+                std::vector<int> fromLanes(size, 0);
+                collectEdgeVectors(fromEdges, toEdges, fromLanes);
                 const std::string crossingDefaultState(newCrossings, 'r');
 
                 // rebuild the logic (see NBOwnTLDef.cpp::myCompute)
                 const std::vector<NBTrafficLightLogic::PhaseDefinition> phases = myTLLogic->getPhases();
                 NBTrafficLightLogic* newLogic = new NBTrafficLightLogic(getID(), getProgramID(), 0, myOffset, myType);
+                SUMOTime brakingTime = TIME2STEPS(3);
                 //std::cout << "patchIfCrossingsAdded for " << getID() << " numPhases=" << phases.size() << "\n";
                 for (std::vector<NBTrafficLightLogic::PhaseDefinition>::const_iterator it = phases.begin(); it != phases.end(); it++) {
+                    if ((*it).state.find_first_of("yY") != std::string::npos) {
+                        brakingTime = MAX2(brakingTime, it->duration);
+                    }
                     NBOwnTLDef::addPedestrianPhases(newLogic, it->duration, it->state + crossingDefaultState, crossings, fromEdges, toEdges);
                 }
+                NBOwnTLDef::addPedestrianScramble(newLogic, noLinksAll, TIME2STEPS(10), brakingTime, crossings, fromEdges, toEdges);
+
                 delete myTLLogic;
                 myTLLogic = newLogic;
             } else if (phases.front().state.size() != noLinksAll) {
@@ -368,5 +357,82 @@ NBLoadedSUMOTLDef::patchIfCrossingsAdded() {
 }
 
 
+void
+NBLoadedSUMOTLDef::collectEdgeVectors(EdgeVector& fromEdges, EdgeVector& toEdges, std::vector<int>& fromLanes) const {
+    assert(fromEdges.size() > 0);
+    assert(fromEdges.size() == toEdges.size());
+    const int size = (int)fromEdges.size();
+
+    for (NBConnectionVector::const_iterator it = myControlledLinks.begin(); it != myControlledLinks.end(); it++) {
+        const NBConnection& c = *it;
+        if (c.getTLIndex() != NBConnection::InvalidTlIndex) {
+            if (c.getTLIndex() >= size) {
+                throw ProcessError("Invalid linkIndex " + toString(c.getTLIndex()) + " for traffic light '" + getID() +
+                                   "' with " + toString(size) + " links.");
+            }
+            fromEdges[c.getTLIndex()] = c.getFrom();
+            toEdges[c.getTLIndex()] = c.getTo();
+            fromLanes[c.getTLIndex()] = c.getFromLane();
+        }
+    }
+}
+
+
+void
+NBLoadedSUMOTLDef::initNeedsContRelation() const {
+    if (!amInvalid() && !myNeedsContRelationReady) {
+        myNeedsContRelation.clear();
+        myRightOnRedConflicts.clear();
+        const bool controlledWithin = !OptionsCont::getOptions().getBool("tls.uncontrolled-within");
+        const std::vector<NBTrafficLightLogic::PhaseDefinition> phases = myTLLogic->getPhases();
+        for (std::vector<NBTrafficLightLogic::PhaseDefinition>::const_iterator it = phases.begin(); it != phases.end(); it++) {
+            const std::string state = (*it).state;
+            for (NBConnectionVector::const_iterator it1 = myControlledLinks.begin(); it1 != myControlledLinks.end(); it1++) {
+                const NBConnection& c1 = *it1;
+                const int i1 = c1.getTLIndex();
+                if (i1 == NBConnection::InvalidTlIndex || state[i1] != 'g' || c1.getFrom() == 0 || c1.getTo() == 0) {
+                    continue;
+                }
+                for (NBConnectionVector::const_iterator it2 = myControlledLinks.begin(); it2 != myControlledLinks.end(); it2++) {
+                    const NBConnection& c2 = *it2;
+                    const int i2 = c2.getTLIndex();
+                    if (i2 != NBConnection::InvalidTlIndex
+                            && i2 != i1
+                            && (state[i2] == 'G' || state[i2] == 'g')
+                            && c2.getFrom() != 0 && c2.getTo() != 0) {
+                        const bool rightTurnConflict = NBNode::rightTurnConflict(
+                                                           c1.getFrom(), c1.getTo(), c1.getFromLane(), c2.getFrom(), c2.getTo(), c2.getFromLane());
+                        const bool forbidden = forbids(c2.getFrom(), c2.getTo(), c1.getFrom(), c1.getTo(), true, controlledWithin);
+                        const bool isFoes = foes(c2.getFrom(), c2.getTo(), c1.getFrom(), c1.getTo());
+                        if (forbidden || rightTurnConflict) {
+                            myNeedsContRelation.insert(StreamPair(c1.getFrom(), c1.getTo(), c2.getFrom(), c2.getTo()));
+                        }
+                        if (isFoes) {
+                            myRightOnRedConflicts.insert(std::make_pair(i1, i2));
+                        }
+                        //std::cout << getID() << " i1=" << i1 << " i2=" << i2 << " rightTurnConflict=" << rightTurnConflict << " forbidden=" << forbidden << " isFoes=" << isFoes << "\n";
+                    }
+                }
+            }
+        }
+    }
+    myNeedsContRelationReady = true;
+    myRightOnRedConflictsReady = true;
+}
+
+
+bool
+NBLoadedSUMOTLDef::rightOnRedConflict(int index, int foeIndex) const {
+    if (amInvalid()) {
+        return false;
+    }
+    if (!myRightOnRedConflictsReady) {
+        initNeedsContRelation();
+        assert(myRightOnRedConflictsReady);
+    }
+    return std::find(myRightOnRedConflicts.begin(), myRightOnRedConflicts.end(), std::make_pair(index, foeIndex)) != myRightOnRedConflicts.end();
+}
+
+
 /****************************************************************************/
 
