@@ -3,13 +3,13 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mar 2011
-/// @version $Id: NBLoadedSUMOTLDef.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: NBLoadedSUMOTLDef.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A complete traffic light logic loaded from a sumo-net. (opted to reimplement
 // since NBLoadedTLDef is quite vissim specific)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2011-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -71,9 +71,8 @@ public:
     ~NBLoadedSUMOTLDef();
 
     /** @brief Informs edges about being controlled by a tls
-     * @param[in] ec The container of edges
      */
-    void setTLControllingInformation(const NBEdgeCont& ec) const;
+    void setTLControllingInformation() const;
 
     /** @brief Replaces occurences of the removed edge in incoming/outgoing edges of all definitions
      * @param[in] removed The removed edge
@@ -131,6 +130,9 @@ public:
      */
     void setType(TrafficLightType type);
 
+    /// @brief whether the given index must yield to the foeIndex while turing right on a red light
+    bool rightOnRedConflict(int index, int foeIndex) const;
+
 protected:
     /** @brief Collects the links participating in this traffic light
      *    (only if not previously loaded)
@@ -142,15 +144,16 @@ protected:
     void collectEdges();
 
     /** @brief Computes the traffic light logic finally in dependence to the type
-     * @param[in] ec The edge container
-     * @param[in] brakingTime Duration a vehicle needs for braking in front of the tls
+     * @param[in] brakingTime Duration a vehicle needs for braking in front of the tls in seconds
      * @return The computed logic
      */
-    NBTrafficLightLogic* myCompute(const NBEdgeCont& ec,
-                                   unsigned int brakingTime);
+    NBTrafficLightLogic* myCompute(unsigned int brakingTimeSeconds);
 
     bool amInvalid() const;
 
+    /* initialize myNeedsContRelation and set myNeedsContRelationReady to true */
+    void initNeedsContRelation() const;
+
 private:
 
     /** @brief phases are added directly to myTLLogic which is then returned in myCompute() */
@@ -159,15 +162,19 @@ private:
     /// @brief The original nodes for which the loaded logic is valid
     std::set<NBNode*> myOriginalNodes;
 
-    /** @brief Informs edges about being controlled by a tls */
-    void setTLControllingInformation() const;
-
     /// @brief repair the plan if controlled nodes received pedestrian crossings
     void patchIfCrossingsAdded();
 
     /// @brief set of edges with shifted lane indices (to avoid shifting twice)
     std::set<NBEdge*> myShifted;
 
+    /** @brief Collects the edges for each tlIndex
+     * @param[out] fromEdges The from-edge for each controlled tlIndex
+     * @param[out] toEdges The to-edge for each controlled tlIndex
+     * @param[out] fromLanes The from-lanes for each controlled tlIndex
+     */
+    void collectEdgeVectors(EdgeVector& fromEdges, EdgeVector& toEdges, std::vector<int>& fromLanes) const;
+
 private:
     /// @brief class for identifying connections
     class connection_equal {
