@@ -8,7 +8,7 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBNodeCont.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: NBNodeCont.cpp 20896 2016-06-07 10:40:32Z behrisch $
 ///
 // Container for nodes during the netbuilding process
 /****************************************************************************/
@@ -46,6 +46,8 @@
 #include <utils/common/StringTokenizer.h>
 #include <utils/common/StdDefs.h>
 #include <utils/common/ToString.h>
+#include <utils/common/TplConvert.h>
+#include <utils/common/IDSupplier.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/iodevices/OutputDevice.h>
@@ -53,7 +55,6 @@
 #include "NBDistrict.h"
 #include "NBEdgeCont.h"
 #include "NBTrafficLightLogicCont.h"
-#include "NBJoinedEdgesMap.h"
 #include "NBOwnTLDef.h"
 #include "NBNodeCont.h"
 
@@ -317,7 +318,7 @@ NBNodeCont::removeIsolatedRoads(NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLig
 
 unsigned int
 NBNodeCont::removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec,
-                                NBJoinedEdgesMap& je, NBTrafficLightLogicCont& tlc,
+                                NBTrafficLightLogicCont& tlc,
                                 bool removeGeometryNodes) {
     // load edges that shall not be modified
     std::set<std::string> edges2keep;
@@ -372,7 +373,6 @@ NBNodeCont::removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec,
             begin->append(continuation);
             continuation->getToNode()->replaceIncoming(continuation, begin, 0);
             tlc.replaceRemoved(continuation, -1, begin, -1);
-            je.appended(begin->getID(), continuation->getID());
             ec.extract(dc, continuation, true);
         }
         toRemove.push_back(current);
@@ -1220,5 +1220,26 @@ NBNodeCont::discardTrafficLights(NBTrafficLightLogicCont& tlc, bool geometryLike
     }
 }
 
+
+int
+NBNodeCont::mapToNumericalIDs() {
+    IDSupplier idSupplier("", getAllNames());
+    std::vector<NBNode*> toChange;
+    for (NodeCont::iterator it = myNodes.begin(); it != myNodes.end(); it++) {
+        try {
+            TplConvert::_str2int(it->first);
+        } catch (NumberFormatException&) {
+            toChange.push_back(it->second);
+        }
+    }
+    for (std::vector<NBNode*>::iterator it = toChange.begin(); it != toChange.end(); ++it) {
+        NBNode* node = *it;
+        myNodes.erase(node->getID());
+        node->setID(idSupplier.getNext());
+        myNodes[node->getID()] = node;
+    }
+    return (int)toChange.size();
+}
+
 /****************************************************************************/
 
