@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBNodeCont.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBNodeCont.h 20896 2016-06-07 10:40:32Z behrisch $
 ///
 // Container for nodes during the netbuilding process
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -51,7 +51,6 @@
 class NBDistrict;
 class OptionsCont;
 class OutputDevice;
-class NBJoinedEdgesMap;
 
 
 // ===========================================================================
@@ -189,6 +188,11 @@ public:
     void joinSimilarEdges(NBDistrictCont& dc, NBEdgeCont& ec, NBTrafficLightLogicCont& tlc);
 
 
+    /** @brief fix overlap
+     */
+    void avoidOverlap();
+
+
     /** @brief Removes sequences of edges that are not connected with a junction.
      * Simple roads without junctions sometimes remain when converting from OpenStreetMake,
      * but they make no sense. Remaining empty nodes are also deleted.
@@ -210,12 +214,11 @@ public:
      * The node is removed from the list of tls-controlled nodes.
      * @param[in, opt. changed] dc The district container needed if a node shall be removed
      * @param[in, opt. changed] ec The edge container needed for joining edges
-     * @param[in, opt. changed] je The map of joined edges (changes are stored here)
      * @param[in, opt. changed] tlc The traffic lights container to remove nodes from
      * @param[in] removeGeometryNodes Whether geometry nodes shall also be removed
      * @return The number of removed nodes
      */
-    unsigned int removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec, NBJoinedEdgesMap& je,
+    unsigned int removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec,
                                      NBTrafficLightLogicCont& tlc, bool removeGeometryNodes);
     /// @}
 
@@ -257,7 +260,7 @@ public:
 
 
     /// divides the incoming lanes on outgoing lanes
-    void computeLanes2Lanes(const bool buildCrossingsAndWalkingAreas);
+    void computeLanes2Lanes();
 
     /// build the list of outgoing edges and lanes
     void computeLogics(const NBEdgeCont& ec, OptionsCont& oc);
@@ -277,10 +280,9 @@ public:
     std::string getFreeID();
 
     /** @brief Compute the junction shape for this node
-     * @param[in] lefhand Whether the network uses left-hand traffic
      * @param[in] mismatchThreshold The threshold for warning about shapes which are away from myPosition
      */
-    void computeNodeShapes(bool leftHand, SUMOReal mismatchThreshold = -1);
+    void computeNodeShapes(SUMOReal mismatchThreshold = -1);
 
     /** @brief Prints statistics about built nodes
      *
@@ -318,12 +320,21 @@ public:
      */
     void discardTrafficLights(NBTrafficLightLogicCont& tlc, bool geometryLike, bool guessSignals);
 
+    /// @brief mark a node as being created form a split
+    void markAsSplit(const NBNode* node) {
+        mySplit.insert(node);
+    }
+
+    /// @brief ensure that all node ids are integers
+    int mapToNumericalIDs();
+
 private:
     /// @name Helper methods for for joining nodes
     /// @{
 
     /// @brief Definition of a node cluster container
     typedef std::vector<std::set<NBNode*> > NodeClusters;
+    typedef std::pair<NBNode*, SUMOReal> NodeAndDist;
 
 
     /** @brief Builds node clusters
@@ -379,6 +390,9 @@ private:
     /// @brief ids found in loaded join clusters used for error checking
     std::set<std::string> myJoined;
 
+    /// @brief nodes that were created when splitting an edge
+    std::set<const NBNode*> mySplit;
+
     /// @brief node positions for faster lookup
     NamedRTree myRTree;
 
