@@ -2,7 +2,7 @@
 /// @file    GNEEdge.cpp
 /// @author  Jakob Erdmann
 /// @date    Feb 2011
-/// @version $Id: GNEEdge.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: GNEEdge.cpp 21143 2016-07-11 12:37:50Z palcraft $
 ///
 // A road/street connecting two junctions (netedit-version, adapted from GUIEdge)
 // Basically a container for an NBEdge with drawing and editing capabilities
@@ -47,6 +47,8 @@
 #include "GNEChange_Lane.h"
 #include "GNEJunction.h"
 #include "GNELane.h"
+#include "GNEAdditional.h"
+#include "GNEAdditionalSet.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -63,15 +65,14 @@ const SUMOReal GNEEdge::SNAP_RADIUS = SUMO_const_halfLaneWidth;
 // members methods
 // ===========================================================================
 GNEEdge::GNEEdge(NBEdge& nbe, GNENet* net, bool wasSplit, bool loaded):
-    GUIGlObject(GLO_EDGE, nbe.getID()),
-    GNEAttributeCarrier(SUMO_TAG_EDGE),
+    GNENetElement(net, nbe.getID(), GLO_EDGE, SUMO_TAG_EDGE),
     myNBEdge(nbe) ,
     myOrigShape(nbe.getInnerGeometry()),
     myLanes(0),
-    myNet(net),
     myAmResponsible(false),
     myWasSplit(wasSplit),
     myConnectionStatus(loaded ? LOADED : GUESSED) {
+    // Create lanes
     int numLanes = myNBEdge.getNumLanes();
     myLanes.reserve(numLanes);
     for (int i = 0; i < numLanes; i++) {
@@ -91,9 +92,17 @@ GNEEdge::~GNEEdge() {
     if (myAmResponsible) {
         delete &myNBEdge;
     }
+    // Remove all references to this edge in their additionals
+    for (AdditionalVector::iterator i = myAdditionals.begin(); i != myAdditionals.end(); i++) {
+        (*i)->removeEdgeReference();
+    }
 }
 
 
+void
+GNEEdge::updateGeometry() {}
+
+
 Boundary
 GNEEdge::getBoundary() const {
     Boundary ret;
@@ -212,6 +221,11 @@ GNEEdge::updateJunctionPosition(GNEJunction* junction, const Position& origPos)
     setGeometry(geom, false);
 }
 
+NBEdge*
+GNEEdge::getNBEdge() {
+    return &myNBEdge;
+}
+
 
 Position
 GNEEdge::getSplitPos(const Position& clickPos) {
@@ -350,7 +364,7 @@ GNEEdge::resetEndpoint(const Position& pos, GNEUndoList* undoList) {
 void
 GNEEdge::setGeometry(PositionVector geom, bool inner) {
     myNBEdge.setGeometry(geom, inner);
-    updateLaneGeometries();
+    updateLaneGeometriesAndAdditionals();
     getSource()->invalidateShape();
     getDest()->invalidateShape();
     myNet->refreshElement(this);
@@ -358,10 +372,19 @@ GNEEdge::setGeometry(PositionVector geom, bool inner) {
 
 
 void
-GNEEdge::updateLaneGeometries() {
+GNEEdge::updateLaneGeometriesAndAdditionals() {
+    // Update geometry of lanes
     for (LaneVector::iterator i = myLanes.begin(); i != myLanes.end(); ++i) {
         (*i)->updateGeometry();
     }
+    // Update geometry of additionals vinculated to this edge
+    for (AdditionalVector::iterator i = myAdditionals.begin(); i != myAdditionals.end(); ++i) {
+        (*i)->updateGeometry();
+    }
+    // Update geometry of additionalSets vinculated to this edge
+    for (AdditionalSetVector::iterator i = myAdditionalSets.begin(); i != myAdditionalSets.end(); ++i) {
+        (*i)->updateGeometry();
+    }
 }
 
 
@@ -397,6 +420,18 @@ GNEEdge::getLaneGlIDs() {
 }
 
 
+const std::vector<GNELane*>&
+GNEEdge::getLanes() {
+    return myLanes;
+}
+
+
+bool
+GNEEdge::wasSplit() {
+    return myWasSplit;
+}
+
+
 std::string
 GNEEdge::getAttribute(SumoXMLAttr key) const {
     switch (key) {
@@ -559,7 +594,7 @@ GNEEdge::isValid(SumoXMLAttr key, const std::string& value) {
             return canParse<int>(value);
             break;
         case SUMO_ATTR_LENGTH:
-            return isPositive<SUMOReal>(value) || parse<SUMOReal>(value) == NBEdge::UNSPECIFIED_LOADED_LENGTH;
+            return canParse<SUMOReal>(value) && (isPositive<SUMOReal>(value) || parse<SUMOReal>(value) == NBEdge::UNSPECIFIED_LOADED_LENGTH);
             break;
         case SUMO_ATTR_ALLOW:
         case SUMO_ATTR_DISALLOW:
@@ -582,7 +617,7 @@ GNEEdge::isValid(SumoXMLAttr key, const std::string& value) {
             return true;
             break;
         case SUMO_ATTR_WIDTH:
-            return isPositive<SUMOReal>(value) || parse<SUMOReal>(value) == NBEdge::UNSPECIFIED_WIDTH;
+            return canParse<SUMOReal>(value) && (isPositive<SUMOReal>(value) || parse<SUMOReal>(value) == NBEdge::UNSPECIFIED_WIDTH);
             break;
         case SUMO_ATTR_ENDOFFSET:
             return canParse<SUMOReal>(value);
@@ -593,6 +628,11 @@ GNEEdge::isValid(SumoXMLAttr key, const std::string& value) {
 }
 
 
+void
+GNEEdge::setResponsible(bool newVal) {
+    myAmResponsible = newVal;
+}
+
 // ===========================================================================
 // private
 // ===========================================================================
@@ -725,13 +765,16 @@ GNEEdge::addLane(GNELane* lane, const NBEdge::Lane& laneAttrs) {
 
 void
 GNEEdge::removeLane(GNELane* lane) {
+    if (myLanes.size() == 0) {
+        throw ProcessError("Should not remove the last lane from an edge\n");
+    }
     if (lane == 0) {
         lane = myLanes.back();
     }
     myNBEdge.deleteLane(lane->getIndex());
     lane->decRef("GNEEdge::removeLane");
     myLanes.erase(myLanes.begin() + lane->getIndex());
-    if (myLanes.back()->unreferenced()) {
+    if (lane->unreferenced()) {
         delete lane;
     }
     // udate indices
@@ -775,4 +818,71 @@ GNEEdge::setMicrosimID(const std::string& newID) {
 }
 
 
+bool
+GNEEdge::addAdditional(GNEAdditional* additional) {
+    // Check if additional already exists before insertion
+    for (AdditionalVector::iterator i = myAdditionals.begin(); i != myAdditionals.end(); i++) {
+        if ((*i) == additional) {
+            return false;
+        }
+    }
+    // Insert it and retur true
+    myAdditionals.push_back(additional);
+    return true;
+}
+
+
+bool
+GNEEdge::removeAdditional(GNEAdditional* additional) {
+    // search additional and remove it
+    for (AdditionalVector::iterator i = myAdditionals.begin(); i != myAdditionals.end(); i++) {
+        if ((*i) == additional) {
+            myAdditionals.erase(i);
+            return true;
+        }
+    }
+    // If additional wasn't found, return false
+    return false;
+}
+
+
+const std::vector<GNEAdditional*>&
+GNEEdge::getAdditionals() const {
+    return myAdditionals;
+}
+
+
+bool
+GNEEdge::addAdditionalSet(GNEAdditionalSet* additionalSet) {
+    // Check if additionalSet already exists before insertion
+    for (AdditionalSetVector::iterator i = myAdditionalSets.begin(); i != myAdditionalSets.end(); i++) {
+        if ((*i) == additionalSet) {
+            return false;
+        }
+    }
+    // Insert it and retur true
+    myAdditionalSets.push_back(additionalSet);
+    return true;
+}
+
+
+bool
+GNEEdge::removeAdditionalSet(GNEAdditionalSet* additionalSet) {
+    // search additionalSet and remove it
+    for (AdditionalSetVector::iterator i = myAdditionalSets.begin(); i != myAdditionalSets.end(); i++) {
+        if ((*i) == additionalSet) {
+            myAdditionalSets.erase(i);
+            return true;
+        }
+    }
+    // If additionalSet wasn't found, return false
+    return false;
+}
+
+
+const std::vector<GNEAdditionalSet*>&
+GNEEdge::getAdditionalSets() {
+    return myAdditionalSets;
+}
+
 /****************************************************************************/
