@@ -2,7 +2,7 @@
 /// @file    GNENet.cpp
 /// @author  Jakob Erdmann
 /// @date    Feb 2011
-/// @version $Id: GNENet.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: GNENet.cpp 21143 2016-07-11 12:37:50Z palcraft $
 ///
 // A visual container for GNE-network-components such as GNEEdge and GNEJunction.
 // GNE components wrap netbuild-components and supply visualisation and editing
@@ -51,6 +51,7 @@
 #include <utils/gui/globjects/GUIGLObjectPopupMenu.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
 #include <utils/gui/div/GUIParameterTableWindow.h>
+#include <utils/gui/images/GUITextureSubSys.h>
 #include <utils/common/StringUtils.h>
 #include <netbuild/NBEdgeCont.h>
 #include <netbuild/NBNodeCont.h>
@@ -68,6 +69,11 @@
 #include "GNEChange_Lane.h"
 #include "GNEChange_Connection.h"
 #include "GNEChange_Selection.h"
+#include "GNEChange_Additional.h"
+#include "GNEAdditional.h"
+#include "GNEAdditionalSet.h"
+#include "GNEStoppingPlace.h"
+#include "GNEDetector.h"
 
 
 #ifdef CHECK_MEMORY_LEAKS
@@ -133,9 +139,12 @@ GNENet::GNENet(NBNetBuilder* netBuilder) :
     */
     //if (myGrid.count() == 0) // myGrid methods will return garbage
 
+    // Init AdditionalHandler
+
     if (myZBoundary.ymin() != Z_INITIALIZED) {
         myZBoundary.add(0, 0);
     }
+
 }
 
 
@@ -204,6 +213,24 @@ GNENet::getCenteringBoundary() const {
 }
 
 
+const Boundary&
+GNENet::getZBoundary() const {
+    return myZBoundary;
+}
+
+
+SUMORTree&
+GNENet::getVisualisationSpeedUp() {
+    return myGrid;
+}
+
+
+const SUMORTree&
+GNENet::getVisualisationSpeedUp() const {
+    return myGrid;
+}
+
+
 GNEJunction*
 GNENet::createJunction(const Position& pos, GNEUndoList* undoList) {
     std::string id = myJunctionIDSupplier.getNext();
@@ -298,12 +325,29 @@ GNENet::deleteJunction(GNEJunction* junction, GNEUndoList* undoList) {
 void
 GNENet::deleteEdge(GNEEdge* edge, GNEUndoList* undoList) {
     undoList->p_begin("delete edge");
+
+    // Iterate over lanes to remove additionals
+    for (std::vector<GNELane*>::const_iterator i = edge->getLanes().begin(); i != edge->getLanes().end(); i++) {
+        for (std::vector<GNEAdditional*>::const_iterator j = (*i)->getAdditionals().begin(); j != (*i)->getAdditionals().end(); j++) {
+            undoList->add(new GNEChange_Additional(this, *j, false), true);
+        }
+    }
+
+    // Remove additionals of edge
+    for (std::vector<GNEAdditional*>::const_iterator i = edge->getAdditionals().begin(); i != edge->getAdditionals().end(); i++) {
+        undoList->add(new GNEChange_Additional(this, *i, false), true);
+    }
+
+    // Delete edge
     undoList->add(new GNEChange_Edge(this, edge, false), true);
+
+    // If previously was selected
     if (gSelected.isSelected(GLO_EDGE, edge->getGlID())) {
         std::set<GUIGlID> deselected;
         deselected.insert(edge->getGlID());
         undoList->add(new GNEChange_Selection(std::set<GUIGlID>(), deselected, true), true);
     }
+
     edge->getSource()->removeFromCrossings(edge, undoList);
     edge->getDest()->removeFromCrossings(edge, undoList);
     edge->getSource()->setLogicValid(false, undoList);
@@ -315,24 +359,29 @@ GNENet::deleteEdge(GNEEdge* edge, GNEUndoList* undoList) {
 
 void
 GNENet::deleteLane(GNELane* lane, GNEUndoList* undoList) {
-    undoList->p_begin("delete lane");
     GNEEdge* edge = &lane->getParentEdge();
-    const NBEdge::Lane& laneAttrs = edge->getNBEdge()->getLaneStruct(lane->getIndex());
-    const bool sidewalk = laneAttrs.permissions == SVC_PEDESTRIAN;
-    undoList->add(new GNEChange_Lane(edge, lane, laneAttrs, false), true);
-    if (gSelected.isSelected(GLO_LANE, lane->getGlID())) {
-        std::set<GUIGlID> deselected;
-        deselected.insert(lane->getGlID());
-        undoList->add(new GNEChange_Selection(std::set<GUIGlID>(), deselected, true), true);
+    if (edge->getNBEdge()->getNumLanes() == 1) {
+        // remove the whole edge instead
+        deleteEdge(edge, undoList);
+    } else {
+        undoList->p_begin("delete lane");
+        const NBEdge::Lane& laneAttrs = edge->getNBEdge()->getLaneStruct(lane->getIndex());
+        const bool sidewalk = laneAttrs.permissions == SVC_PEDESTRIAN;
+        undoList->add(new GNEChange_Lane(edge, lane, laneAttrs, false), true);
+        if (gSelected.isSelected(GLO_LANE, lane->getGlID())) {
+            std::set<GUIGlID> deselected;
+            deselected.insert(lane->getGlID());
+            undoList->add(new GNEChange_Selection(std::set<GUIGlID>(), deselected, true), true);
+        }
+        if (sidewalk) {
+            edge->getSource()->removeFromCrossings(edge, undoList);
+            edge->getDest()->removeFromCrossings(edge, undoList);
+            edge->getSource()->setLogicValid(false, undoList);
+            edge->getDest()->setLogicValid(false, undoList);
+        }
+        requireRecompute();
+        undoList->p_end();
     }
-    if (sidewalk) {
-        edge->getSource()->removeFromCrossings(edge, undoList);
-        edge->getDest()->removeFromCrossings(edge, undoList);
-        edge->getSource()->setLogicValid(false, undoList);
-        edge->getDest()->setLogicValid(false, undoList);
-    }
-    requireRecompute();
-    undoList->p_end();
 }
 
 
@@ -511,6 +560,25 @@ GNENet::save(OptionsCont& oc) {
 }
 
 
+void
+GNENet::saveAdditionals(const std::string& filename) {
+    // Extract path of filename
+    std::string path = filename;
+    while (!path.empty() && path[path.size() - 1] != '\\') {
+        path = path.substr(0, path.size() - 1);
+    }
+    OutputDevice& device = OutputDevice::getDevice(filename);
+    device.openTag("additionals");
+    for (GNEAdditionals::const_iterator i = myAdditionals.begin(); i != myAdditionals.end(); ++i) {
+        // Only write additional if don't belong to another additionalSet
+        if (i->second->getAdditionalSetParent() == NULL) {
+            i->second->writeAdditional(device, path);
+        }
+    }
+    device.close();
+}
+
+
 void
 GNENet::savePlain(OptionsCont& oc) {
     computeAndUpdate(oc);
@@ -525,26 +593,36 @@ GNENet::saveJoined(OptionsCont& oc) {
 }
 
 
+void
+GNENet::setUpdateTarget(FXWindow* updateTarget) {
+    myUpdateTarget = updateTarget;
+}
+
+
 GNEJunction*
 GNENet::retrieveJunction(const std::string& id, bool failHard) {
     if (myJunctions.count(id)) {
         return myJunctions[id];
     } else if (failHard) {
+        // If junction wasn't found, throw exception
         throw UnknownElement("Junction " + id);
     } else {
-        return 0;
+        return NULL;
     }
 }
 
 
 GNEEdge*
 GNENet::retrieveEdge(const std::string& id, bool failHard) {
-    if (myEdges.count(id)) {
-        return myEdges[id];
+    GNEEdges::const_iterator i = myEdges.find(id);
+    // If edge was fund
+    if (i != myEdges.end()) {
+        return i->second;
     } else if (failHard) {
+        // If edge wasn't found, throw exception
         throw UnknownElement("Edge " + id);
     } else {
-        return 0;
+        return NULL;
     }
 }
 
@@ -576,6 +654,26 @@ GNENet::retrieveLanes(bool onlySelected) {
 }
 
 
+GNELane*
+GNENet::retrieveLane(const std::string& id, bool failHard) {
+    for (GNEEdges::const_iterator it = myEdges.begin(); it != myEdges.end(); ++it) {
+        const GNEEdge::LaneVector& lanes = it->second->getLanes();
+        for (GNEEdge::LaneVector::const_iterator it_lane = lanes.begin(); it_lane != lanes.end(); ++it_lane) {
+            if ((*it_lane)->getID() == id) {
+                return (*it_lane);
+            }
+        }
+    }
+    // If lane wasn't found:
+    if (failHard) {
+        // Throw exception if failHard is enabled
+        throw UnknownElement("lane " + id);
+    } else {
+        return NULL;
+    }
+}
+
+
 std::vector<GNEJunction*>
 GNENet::retrieveJunctions(bool onlySelected) {
     std::vector<GNEJunction*> result;
@@ -615,6 +713,13 @@ GNENet::retrieveAttributeCarriers(const std::set<GUIGlID>& ids, GUIGlObjectType
                 case GLO_LANE:
                     ac = dynamic_cast<GNELane*>(object);
                     break;
+                case GLO_ADDITIONAL:
+                    if (dynamic_cast<GNEAdditional*>(object)) {
+                        ac = dynamic_cast<GNEAdditional*>(object);
+                    } else if (dynamic_cast<GNEAdditionalSet*>(object)) {
+                        ac = dynamic_cast<GNEAdditionalSet*>(object);
+                    }
+                    break;
                 default:
                     break;
             }
@@ -641,6 +746,7 @@ GNENet::getGlIDs(GUIGlObjectType type) {
             knownTypes.insert(GLO_EDGE);
             knownTypes.insert(GLO_LANE);
             // knownTypes.insert(GLO_TLLOGIC); makes no sense to include them
+            knownTypes.insert(GLO_ADDITIONAL);
             for (std::set<GUIGlObjectType>::const_iterator it = knownTypes.begin(); it != knownTypes.end(); it++) {
                 const std::set<GUIGlID> tmp = getGlIDs(*it);
                 result.insert(tmp.begin(), tmp.end());
@@ -676,6 +782,12 @@ GNENet::getGlIDs(GUIGlObjectType type) {
             }
             break;
         }
+        case GLO_ADDITIONAL: {
+            for (GNEAdditionals::iterator it = myAdditionals.begin(); it != myAdditionals.end(); it++) {
+                result.insert(it->second->getGlID());
+            }
+            break;
+        }
         default: // add other types once we know them
             break;
     }
@@ -724,6 +836,18 @@ GNENet::computeJunction(GNEJunction* junction) {
 }
 
 
+void
+GNENet::requireRecompute() {
+    myNeedRecompute = true;
+}
+
+
+FXApp*
+GNENet::getApp() {
+    return myUpdateTarget->getApp();
+}
+
+
 void
 GNENet::joinSelectedJunctions(GNEUndoList* undoList) {
     std::vector<GNEJunction*> selected = retrieveJunctions(true);
@@ -831,6 +955,12 @@ GNENet::changeEdgeEndpoints(GNEEdge* edge, const std::string& newSource, const s
 }
 
 
+NBTrafficLightLogicCont&
+GNENet::getTLLogicCont() {
+    return myNetBuilder->getTLLogicCont();
+};
+
+
 void
 GNENet::renameJunction(GNEJunction* junction, const std::string& newID) {
     myJunctions.erase(junction->getNBNode()->getID());
@@ -905,9 +1035,110 @@ GNENet::finishMoveSelection(GNEUndoList* undoList) {
     undoList->p_end();
 }
 
+
+ShapeContainer&
+GNENet::getShapeContainer() {
+    return myShapeContainer;
+}
+
+
+void
+GNENet::insertAdditional(GNEAdditional* additional, bool hardFail) {
+    // Check if additional element exists before insertion
+    if (myAdditionals.find(std::pair<std::string, SumoXMLTag>(additional->getID(), additional->getTag())) != myAdditionals.end()) {
+        // Throw exception only if hardFail is enabled
+        if(hardFail) {
+            throw ProcessError("additional element with ID='" + additional->getID() + "' already exist");
+        }
+    } else {
+        myAdditionals[std::pair<std::string, SumoXMLTag>(additional->getID(), additional->getTag())] = additional;
+        myGrid.addAdditionalGLObject(additional);
+        if (additional->getAdditionalSetParent() != NULL) {
+            additional->getAdditionalSetParent()->addAdditionalChild(additional);
+        }
+        update();
+    }
+}
+
+
+void
+GNENet::deleteAdditional(GNEAdditional* additional) {
+    GNEAdditionals::iterator positionToRemove = myAdditionals.find(std::pair<std::string, SumoXMLTag>(additional->getID(), additional->getTag()));
+    // Check if additional element exists before deletion
+    if (positionToRemove == myAdditionals.end()) {
+        throw ProcessError("additional element with ID='" + additional->getID() + "' don't exist");
+    } else {
+        myAdditionals.erase(positionToRemove);
+        myGrid.removeAdditionalGLObject(additional);
+        if (additional->getAdditionalSetParent() != NULL) {
+            additional->getAdditionalSetParent()->removeAdditionalChild(additional);
+        }
+        update();
+    }
+}
+
+
+void
+GNENet::updateAdditionalID(const std::string& oldID, GNEAdditional* additional) {
+    GNEAdditionals::iterator additionalToUpdate = myAdditionals.find(std::pair<std::string, SumoXMLTag>(oldID, additional->getTag()));
+    if (additionalToUpdate != myAdditionals.end()) {
+        // remove an insert additional again into container
+        myAdditionals.erase(additionalToUpdate);
+        myAdditionals[std::pair<std::string, SumoXMLTag>(additional->getID(), additional->getTag())] = additional;
+    }
+}
+
+
+GNEAdditional*
+GNENet::getAdditional(SumoXMLTag type, const std::string& id) const {
+    if (myAdditionals.empty()) {
+        return NULL;
+    } else if (myAdditionals.find(std::pair<std::string, SumoXMLTag>(id, type)) != myAdditionals.end())  {
+        return myAdditionals.at(std::pair<std::string, SumoXMLTag>(id, type));
+    } else {
+        return NULL;
+    }
+}
+
+
+std::string
+GNENet::getAdditionalID(SumoXMLTag type, const GNELane* lane, const SUMOReal pos) const {
+    for (GNEAdditionals::const_iterator it = myAdditionals.begin(); it != myAdditionals.end(); ++it) {
+        if ((it->second->getTag() == type) && (it->second->getLane() != NULL) && (it->second->getLane() == lane) && (fabs(it->second->getPositionInView().x() - pos) < POSITION_EPS)) {
+            return it->second->getID();
+        }
+    }
+    return "";
+}
+
+
+std::vector<GNEAdditional*>
+GNENet::getAdditionals(SumoXMLTag type) {
+    std::vector<GNEAdditional*> vectorOfAdditionals;
+    for (GNEAdditionals::iterator i = myAdditionals.begin(); i != myAdditionals.end(); i++) {
+        if (type == SUMO_TAG_NOTHING || type == i->second->getTag()) {
+            vectorOfAdditionals.push_back(i->second);
+        }
+    }
+    return vectorOfAdditionals;
+}
+
+
+int
+GNENet::getNumberOfAdditionals(SumoXMLTag type) {
+    int counter = 0;
+    for (GNEAdditionals::iterator i = myAdditionals.begin(); i != myAdditionals.end(); i++) {
+        if (type == SUMO_TAG_NOTHING || type == i->second->getTag()) {
+            counter++;
+        }
+    }
+    return counter;
+}
+
 // ===========================================================================
 // private
 // ===========================================================================
+
 void
 GNENet::insertJunction(GNEJunction* junction) {
     myNetBuilder->getNodeCont().insert(junction->getNBNode());
@@ -969,6 +1200,7 @@ GNENet::deleteSingleJunction(GNEJunction* junction) {
 
 void
 GNENet::deleteSingleEdge(GNEEdge* edge) {
+
     myGrid.removeAdditionalGLObject(edge);
     myEdges.erase(edge->getMicrosimID());
     myNetBuilder->getEdgeCont().extract(
@@ -1012,12 +1244,12 @@ GNENet::computeAndUpdate(OptionsCont& oc) {
     myNetBuilder->compute(oc, liveExplicitTurnarounds, false);
     // update precomputed lane geometries
     for (GNEEdges::const_iterator it = myEdges.begin(); it != myEdges.end(); it++) {
-        it->second->updateLaneGeometries();
+        it->second->updateLaneGeometriesAndAdditionals();
     }
     for (GNEJunctions::const_iterator it = myJunctions.begin(); it != myJunctions.end(); it++) {
         it->second->setLogicValid(true);
         // updated shape
-        it->second->updateBoundary();
+        it->second->updateGeometry();
         refreshElement(it->second);
     }
     myNeedRecompute = false;
