@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_DlrNavteq.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NIImporter_DlrNavteq.cpp 20921 2016-06-08 10:18:15Z behrisch $
 ///
 // Importer for networks stored in Elmar's format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -211,8 +211,8 @@ NIImporter_DlrNavteq::EdgesHandler::EdgesHandler(NBNodeCont& nc, NBEdgeCont& ec,
     myGeoms(geoms),
     myStreetNames(streetNames),
     myVersion(0),
-    myFile(file)
-{ }
+    myFile(file) {
+}
 
 
 NIImporter_DlrNavteq::EdgesHandler::~EdgesHandler() {}
@@ -329,10 +329,10 @@ NIImporter_DlrNavteq::EdgesHandler::report(const std::string& result) {
         if (getColumn(st, CONNECTION, "0") == "1") {
             geoms = geoms.reverse();
         }
-        geoms.push_front(from->getPosition());
+        geoms.insert(geoms.begin(), from->getPosition());
         geoms.push_back(to->getPosition());
         e = new NBEdge(id, from, to, "", speed, numLanes, priority,
-                       NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET, geoms, streetName, LANESPREAD_CENTER);
+                       NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET, geoms, streetName, id, LANESPREAD_CENTER);
     }
     // add vehicle type information to the edge
     if (myVersion < 6.0) {
