@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_SUMO.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: NIImporter_SUMO.cpp 20550 2016-04-26 10:57:45Z namdre $
 ///
 // Importer for networks stored in SUMO format
 /****************************************************************************/
@@ -162,6 +162,7 @@ NIImporter_SUMO::_loadNetwork(OptionsCont& oc) {
         ed->builtEdge = myNetBuilder.getEdgeCont().retrieve(ed->id);
     }
     // assign further lane attributes (edges are built)
+    EdgeVector toRemove;
     for (std::map<std::string, EdgeAttrs*>::const_iterator i = myEdges.begin(); i != myEdges.end(); ++i) {
         EdgeAttrs* ed = (*i).second;
         NBEdge* nbe = ed->builtEdge;
@@ -213,6 +214,7 @@ NIImporter_SUMO::_loadNetwork(OptionsCont& oc) {
             nbe->setLaneWidth(fromLaneIndex, lane->width);
             nbe->setEndOffset(fromLaneIndex, lane->endOffset);
             nbe->setSpeed(fromLaneIndex, lane->maxSpeed);
+            nbe->getLaneStruct(fromLaneIndex).oppositeID = lane->oppositeID;
         }
         nbe->declareConnectionsAsLoaded();
         if (!nbe->hasLaneSpecificWidth() && nbe->getLanes()[0].width != NBEdge::UNSPECIFIED_WIDTH) {
@@ -221,6 +223,14 @@ NIImporter_SUMO::_loadNetwork(OptionsCont& oc) {
         if (!nbe->hasLaneSpecificEndOffset() && nbe->getEndOffset(0) != NBEdge::UNSPECIFIED_OFFSET) {
             nbe->setEndOffset(-1, nbe->getEndOffset(0));
         }
+        // check again after permissions are set
+        if (myNetBuilder.getEdgeCont().ignoreFilterMatch(nbe)) {
+            myNetBuilder.getEdgeCont().ignore(nbe->getID());
+            toRemove.push_back(nbe);
+        }
+    }
+    for (EdgeVector::iterator i = toRemove.begin(); i != toRemove.end(); ++i) {
+        myNetBuilder.getEdgeCont().erase(myNetBuilder.getDistrictCont(), *i);
     }
     // insert loaded prohibitions
     for (std::vector<Prohibition>::const_iterator it = myProhibitions.begin(); it != myProhibitions.end(); it++) {
@@ -329,6 +339,9 @@ NIImporter_SUMO::myStartElement(int element,
         case SUMO_TAG_LANE:
             addLane(attrs);
             break;
+        case SUMO_TAG_NEIGH:
+            myCurrentLane->oppositeID = attrs.getString(SUMO_ATTR_LANE);
+            break;
         case SUMO_TAG_JUNCTION:
             addJunction(attrs);
             break;
@@ -458,7 +471,7 @@ NIImporter_SUMO::addLane(const SUMOSAXAttributes& attrs) {
         const std::string nodeID = NBNode::getNodeIDFromInternalLane(id);
         myCustomShapeMaps[nodeID][id] = attrs.get<PositionVector>(SUMO_ATTR_SHAPE, id.c_str(), ok);
     }
-    myCurrentLane = new LaneAttrs;
+    myCurrentLane = new LaneAttrs();
     if (myCurrentEdge->func == EDGEFUNC_CROSSING) {
         // save the width and the lane id of the crossing but don't do anything else
         std::vector<Crossing>& crossings = myPedestrianCrossings[SUMOXMLDefinitions::getJunctionIDFromInternalEdge(myCurrentEdge->id)];
