@@ -6,12 +6,12 @@
 /// @author  Walter Bamberger
 /// @author  Laura Bieker
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NIXMLEdgesHandler.cpp 18756 2015-08-31 19:16:33Z behrisch $
+/// @version $Id: NIXMLEdgesHandler.cpp 21094 2016-07-03 13:57:51Z behrisch $
 ///
 // Importer for network edges stored in XML
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -60,11 +60,6 @@
 #endif // CHECK_MEMORY_LEAKS
 
 
-// ===========================================================================
-// used constants
-// ===========================================================================
-const SUMOReal SUMOXML_INVALID_POSITION = -999999.;
-
 // ===========================================================================
 // method definitions
 // ===========================================================================
@@ -84,8 +79,8 @@ NIXMLEdgesHandler::NIXMLEdgesHandler(NBNodeCont& nc,
       myCurrentEdge(0), myHaveReportedAboutOverwriting(false),
       myHaveReportedAboutTypeOverride(false),
       myHaveWarnedAboutDeprecatedLaneId(false),
-      myKeepEdgeShape(!options.getBool("plain.extend-edge-shape"))
-{}
+      myKeepEdgeShape(!options.getBool("plain.extend-edge-shape")) {
+}
 
 
 NIXMLEdgesHandler::~NIXMLEdgesHandler() {}
@@ -101,6 +96,9 @@ NIXMLEdgesHandler::myStartElement(int element,
         case SUMO_TAG_LANE:
             addLane(attrs);
             break;
+        case SUMO_TAG_NEIGH:
+            myCurrentEdge->getLaneStruct((int)myCurrentEdge->getNumLanes() - 1).oppositeID = attrs.getString(SUMO_ATTR_LANE);
+            break;
         case SUMO_TAG_SPLIT:
             addSplit(attrs);
             break;
@@ -275,7 +273,7 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
         } else {
             myCurrentEdge = new NBEdge(myCurrentID, myFromNode, myToNode, myCurrentType, myCurrentSpeed,
                                        myCurrentLaneNo, myCurrentPriority, myCurrentWidth, myCurrentEndOffset,
-                                       myShape, myCurrentStreetName, myLanesSpread,
+                                       myShape, myCurrentStreetName, "", myLanesSpread,
                                        myKeepEdgeShape);
         }
     }
@@ -372,8 +370,8 @@ void NIXMLEdgesHandler::addSplit(const SUMOSAXAttributes& attrs) {
             }
         }
         if (e.lanes.empty()) {
-            for (size_t l = 0; l < myCurrentEdge->getNumLanes(); ++l) {
-                e.lanes.push_back((int) l);
+            for (int l = 0; l < myCurrentEdge->getNumLanes(); ++l) {
+                e.lanes.push_back(l);
             }
         }
         e.speed = attrs.getOpt(SUMO_ATTR_SPEED, 0, ok, myCurrentEdge->getSpeed());
@@ -521,7 +519,7 @@ NIXMLEdgesHandler::myEndElement(int element) {
             }
             // split the edge
             std::vector<int> currLanes;
-            for (unsigned int l = 0; l < e->getNumLanes(); ++l) {
+            for (int l = 0; l < e->getNumLanes(); ++l) {
                 currLanes.push_back(l);
             }
             if (e->getNumLanes() != mySplits.back().lanes.size()) {
@@ -580,10 +578,11 @@ NIXMLEdgesHandler::myEndElement(int element) {
                         WRITE_WARNING("Error on parsing a split (edge '" + myCurrentID + "').");
                     }
                 }  else if (exp.pos == 0) {
-                    if (e->getNumLanes() < exp.lanes.size()) {
-                        e->incLaneNo((int) exp.lanes.size() - e->getNumLanes());
+                    const int laneCountDiff = e->getNumLanes() - (int)exp.lanes.size();
+                    if (laneCountDiff < 0) {
+                        e->incLaneNo(-laneCountDiff);
                     } else {
-                        e->decLaneNo(e->getNumLanes() - (int) exp.lanes.size());
+                        e->decLaneNo(laneCountDiff);
                     }
                     currLanes = exp.lanes;
                     // invalidate traffic light definition loaded from a SUMO network
