@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimTL.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: NIVissimTL.cpp 21130 2016-07-08 05:50:46Z namdre $
 ///
 // -------------------
 /****************************************************************************/
@@ -163,7 +163,7 @@ NIVissimTL::NIVissimTLSignal::addTo(NBEdgeCont& ec, NBLoadedTLDef* tl) const {
             }
         } else {
             WRITE_WARNING("Edge : Lanes were not assigned(!)");
-            for (unsigned int j = 0; j < edge->getNumLanes(); j++) {
+            for (int j = 0; j < edge->getNumLanes(); j++) {
                 std::vector<NBEdge::Connection> connections = edge->getConnectionsFromLane(j);
                 for (std::vector<NBEdge::Connection>::iterator i = connections.begin(); i != connections.end(); i++) {
                     const NBEdge::Connection& conn = *i;
