@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLEdgeControlBuilder.h 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: NLEdgeControlBuilder.h 20550 2016-04-26 10:57:45Z namdre $
 ///
 // Interface for building edges
 /****************************************************************************/
@@ -88,10 +88,7 @@ public:
                           const std::string& streetName, const std::string& edgeType, int priority);
 
 
-    /** @brief Adds a lane to the current edge;
-     *
-     * This method throws an ProcessError when the lane is marked to be the depart lane
-     *  and another so marked lane was added before
+    /** @brief Adds a lane to the current edge
      *
      * @param[in] id The lane's id
      * @param[in] maxSpeed The speed allowed on this lane
@@ -109,6 +106,14 @@ public:
                             SUMOReal width, SVCPermissions permissions, int index);
 
 
+    /** @brief Adds a neighbor to the current lane
+     *
+     * @param[in] id The lane's id
+     * @see MSLane
+     */
+    virtual void addNeigh(const std::string id);
+
+
     /** @brief Closes the building of an edge;
         The edge is completely described by now and may not be opened again */
     virtual MSEdge* closeEdge();
