@@ -7,7 +7,7 @@
 /// @author  Michael Behrisch
 /// @author  Felix Brack
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLHandler.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: NLHandler.cpp 20928 2016-06-08 19:37:36Z behrisch $
 ///
 // The XML-Handler for network loading
 /****************************************************************************/
@@ -79,6 +79,7 @@ NLHandler::NLHandler(const std::string& file, MSNet& net,
     myHaveWarnedAboutDeprecatedLanes(false),
     myLastParameterised(0),
     myHaveSeenInternalEdge(false),
+    myHaveSeenNeighs(false),
     myLefthand(false),
     myNetworkVersion(0),
     myNetIsLoaded(false) {
@@ -105,6 +106,10 @@ NLHandler::myStartElement(int element,
             case SUMO_TAG_LANE:
                 addLane(attrs);
                 break;
+            case SUMO_TAG_NEIGH:
+                myEdgeControlBuilder.addNeigh(attrs.getString(SUMO_ATTR_LANE));
+                myHaveSeenNeighs = true;
+                break;
             case SUMO_TAG_JUNCTION:
                 openJunction(attrs);
                 initJunctionLogic(attrs);
@@ -514,8 +519,9 @@ NLHandler::parseLanes(const std::string& junctionID,
 void
 NLHandler::addParam(const SUMOSAXAttributes& attrs) {
     bool ok = true;
-    std::string key = attrs.get<std::string>(SUMO_ATTR_KEY, 0, ok);
-    std::string val = attrs.get<std::string>(SUMO_ATTR_VALUE, 0, ok);
+    const std::string key = attrs.get<std::string>(SUMO_ATTR_KEY, 0, ok);
+    // circumventing empty string test
+    const std::string val = attrs.hasAttribute(SUMO_ATTR_VALUE) ? attrs.getString(SUMO_ATTR_VALUE) : "";
     if (myLastParameterised != 0) {
         myLastParameterised->addParameter(key, val);
     }
@@ -1030,11 +1036,12 @@ NLHandler::addConnection(const SUMOSAXAttributes& attrs) {
         assert(fromLane);
         assert(toLane);
 
+        MSTrafficLightLogic* logic = 0;
         int tlLinkIdx = -1;
         if (tlID != "") {
             tlLinkIdx = attrs.get<int>(SUMO_ATTR_TLLINKINDEX, 0, ok);
             // make sure that the index is in range
-            MSTrafficLightLogic* logic = myJunctionControlBuilder.getTLLogic(tlID).getActive();
+            logic = myJunctionControlBuilder.getTLLogic(tlID).getActive();
             if ((tlLinkIdx < 0 || tlLinkIdx >= (int)logic->getCurrentPhaseDef().getState().size())
                     && logic->getLogicType() != "railSignal"
                     && logic->getLogicType() != "railCrossing") {
@@ -1060,20 +1067,21 @@ NLHandler::addConnection(const SUMOSAXAttributes& attrs) {
             }
             length = via->getLength();
         }
-        link = new MSLink(toLane, via, dir, state, length, keepClear);
+        link = new MSLink(fromLane, toLane, via, dir, state, length, keepClear, logic, tlLinkIdx);
         if (via != 0) {
             via->addIncomingLane(fromLane, link);
         } else {
             toLane->addIncomingLane(fromLane, link);
         }
 #else
-        link = new MSLink(toLane, dir, state, length, keepClear);
+        link = new MSLink(fromLane, toLane, dir, state, length, keepClear, logic, tlLinkIdx);
         toLane->addIncomingLane(fromLane, link);
 #endif
         toLane->addApproachingLane(fromLane, myNetworkVersion < 0.25);
 
         // if a traffic light is responsible for it, inform the traffic light
         // check whether this link is controlled by a traffic light
+        // we can not reuse logic here because it might be an inactive one
         if (tlID != "") {
             myJunctionControlBuilder.getTLLogic(tlID).addLink(link, fromLane, tlLinkIdx);
         }
