@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLJunctionControlBuilder.h 18213 2015-04-16 11:55:42Z behrisch $
+/// @version $Id: NLJunctionControlBuilder.h 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // Builder of microsim-junctions and tls
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -123,7 +123,7 @@ public:
      * @todo Throwing ProcessError would get unneeded if the container would be built by default (see prepare)
      * @todo The type of the junctions shoould probably be checked when supprted (in openJunction)
      */
-    void closeJunction();
+    void closeJunction(const std::string& basePath);
 
 
     /** @brief Builds the MSJunctionControl which holds all of the simulations junctions
@@ -182,6 +182,30 @@ public:
     void addPhase(SUMOTime duration, const std::string& state,
                   SUMOTime min, SUMOTime max);
 
+    /** @brief Adds a phase to the currently built traffic lights logic
+     *
+     * @param[in] duration The duration of the phase
+     * @param[in] state The state of the tls
+     * @param[in] minDuration The minimum duration of the phase
+     * @param[in] maxDuration The maximum duration of the phase
+     * @param[in] transient_notdecisional Specifies if this is a transient phase (true) or a decisional one (false)
+     * @param[in] commit Specifies if this is a commit phase
+     */
+    void addPhase(SUMOTime duration, const std::string& state, SUMOTime minDuration, SUMOTime maxDuration, bool transient_notdecisional, bool commit) throw();
+
+
+    /** @brief Adds a phase to the currently built traffic lights logic
+     *
+     * @param[in] duration The duration of the phase
+     * @param[in] state The state of the tls
+     * @param[in] minDuration The minimum duration of the phase
+     * @param[in] maxDuration The maximum duration of the phase
+     * @param[in] transient_notdecisional Specifies if this is a transient phase (true) or a decisional one (false)
+     * @param[in] commit Specifies if this is a commit phase
+     * @param[in] targetLanes A reference to the vector containing targeted sensor lanes for this phase, given by lane id
+     */
+    void addPhase(SUMOTime duration, const std::string& state, SUMOTime minDuration, SUMOTime maxDuration, bool transient_notdecisional, bool commit, MSPhaseDefinition::LaneIdVector& targetLanes) throw();
+
 
     /** @brief Returns a previously build tls logic
      *
@@ -202,7 +226,7 @@ public:
      * @exception ProcessError If the traffic lights were not correct
      * @see MSTLLogicControl::closeNetworkReading
      */
-    MSTLLogicControl* buildTLLogics() const;
+    MSTLLogicControl* buildTLLogics();
 
 
     /** @brief Ends the building of a traffic lights logic
@@ -213,7 +237,7 @@ public:
      *
      * @exception InvalidArgument If another tls logic with the same name as the currently built was loaded before
      */
-    virtual void closeTrafficLightLogic();
+    virtual void closeTrafficLightLogic(const std::string& basePath);
 
 
     /** @brief Ends the building of a junction logic (row-logic)
@@ -341,7 +365,7 @@ protected:
     MSBitsetLogic::Foes myActiveFoes;
 
     /// @brief The description about which lanes have an internal follower
-    std::bitset<64> myActiveConts;
+    std::bitset<SUMO_MAX_CONNECTIONS> myActiveConts;
 
     /// @brief The current phase definitions for a simple traffic light
     MSSimpleTrafficLightLogic::Phases myActivePhases;
