@@ -3,7 +3,7 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Tue, 04.05.2011
-/// @version $Id: NWWriter_OpenDrive.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: NWWriter_OpenDrive.cpp 21133 2016-07-08 09:25:46Z namdre $
 ///
 // Exporter writing networks using the openDRIVE format
 /****************************************************************************/
@@ -305,10 +305,24 @@ NWWriter_OpenDrive::getLaneType(SVCPermissions permissions) {
         case 0:
             // ambiguous
             return "none";
+        case SVC_RAIL:
+        case SVC_RAIL_URBAN:
+        case SVC_RAIL_ELECTRIC:
+            return "rail";
         case SVC_TRAM:
             return "tram";
-        default:
-            return "driving";
+        default: {
+            // complex permissions
+            if (permissions == SVCAll) {
+                return "driving";
+            } else if (isRailway(permissions)) {
+                return "rail";
+            } else if ((permissions & SVC_PASSENGER) != 0) {
+                return "driving";
+            } else {
+                return "restricted";
+            }
+        }
     }
 }
 
