@@ -4,7 +4,7 @@
 /// @author  Michael Behrisch
 /// @author  Yun-Pang Floetteroed
 /// @date    05. Apr. 2006
-/// @version $Id: ODMatrix.h 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: ODMatrix.h 20836 2016-06-01 09:00:35Z behrisch $
 ///
 // An O/D (origin/destination) matrix
 /****************************************************************************/
@@ -39,6 +39,7 @@
 #include <cstdlib>
 #include <ctime>
 #include <algorithm>
+#include <set>
 #include <string>
 #include <utils/common/SUMOTime.h>
 #include "ODCell.h"
@@ -105,8 +106,9 @@ public:
      * @param[in] origin The origin district to use for the cell's flows
      * @param[in] destination The destination district to use for the cell's flows
      * @param[in] vehicleType The vehicle type to use for the cell's flows
+     * @return whether the cell could be added
      */
-    void add(SUMOReal vehicleNumber, SUMOTime begin,
+    bool add(SUMOReal vehicleNumber, SUMOTime begin,
              SUMOTime end, const std::string& origin, const std::string& destination,
              const std::string& vehicleType);
 
@@ -119,8 +121,9 @@ public:
      * @param[in] depart The departure time of the vehicle
      * @param[in] od The origin and destination district to use for the cell's flows
      * @param[in] vehicleType The vehicle type to use for the cell's flows
+     * @return whether the vehicle could be added
      */
-    void add(const std::string& id, const SUMOTime depart,
+    bool add(const std::string& id, const SUMOTime depart,
              const std::pair<const std::string, const std::string>& od,
              const std::string& vehicleType);
 
@@ -184,7 +187,7 @@ public:
      *
      * @return The number of loaded vehicles
      */
-    SUMOReal getNoLoaded() const;
+    SUMOReal getNumLoaded() const;
 
 
     /** @brief Returns the number of written vehicles
@@ -193,7 +196,7 @@ public:
      *
      * @return The number of written vehicles
      */
-    SUMOReal getNoWritten() const;
+    SUMOReal getNumWritten() const;
 
 
     /** @brief Returns the number of discarded vehicles
@@ -202,7 +205,7 @@ public:
      *
      * @return The number of discarded vehicles
      */
-    SUMOReal getNoDiscarded() const;
+    SUMOReal getNumDiscarded() const;
 
 
     /** @brief Splits the stored cells dividing them on the given time line
@@ -336,7 +339,7 @@ private:
     SUMOReal readFactor(LineReader& lr, SUMOReal scale);
 
 
-protected:
+private:
     /// @brief The loaded cells
     std::vector<ODCell*> myContainer;
 
@@ -346,14 +349,17 @@ protected:
     /// @brief The districts to retrieve sources/sinks from
     const ODDistrictCont& myDistricts;
 
+    /// @brief The missing districts already warned about
+    std::set<std::string> myMissingDistricts;
+
     /// @brief Number of loaded vehicles
-    SUMOReal myNoLoaded;
+    SUMOReal myNumLoaded;
 
     /// @brief Number of written vehicles
-    SUMOReal myNoWritten;
+    SUMOReal myNumWritten;
 
     /// @brief Number of discarded vehicles
-    SUMOReal myNoDiscarded;
+    SUMOReal myNumDiscarded;
 
 
     /**
