@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: PCLoaderArcView.cpp 18752 2015-08-30 19:09:05Z namdre $
+/// @version $Id: PCLoaderArcView.cpp 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // A reader of pois and polygons from shape files
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -79,11 +79,17 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
     std::string prefix = oc.getString("prefix");
     std::string type = oc.getString("type");
     RGBColor color = RGBColor::parseColor(oc.getString("color"));
-    int layer = oc.getInt("layer");
+    SUMOReal layer = oc.getFloat("layer");
     std::string idField = oc.getString("shapefile.id-column");
     bool useRunningID = oc.getBool("shapefile.use-running-id");
     // start parsing
     std::string shpName = file + ".shp";
+    int fillType = -1;
+    if (oc.getString("shapefile.fill") == "true") {
+        fillType = 1;
+    } else if (oc.getString("shapefile.fill") == "false") {
+        fillType = 0;
+    }
 #if GDAL_VERSION_MAJOR < 2
     OGRRegisterAll();
     OGRDataSource* poDS = OGRSFDriverRegistrar::Open(shpName.c_str(), FALSE);
@@ -145,13 +151,14 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
                 if (!geoConvHelper.x2cartesian(pos)) {
                     WRITE_ERROR("Unable to project coordinates for POI '" + id + "'.");
                 }
-                PointOfInterest* poi = new PointOfInterest(id, type, color, pos, (SUMOReal)layer);
-                if (toFill.insert(id, poi, layer)) {
+                PointOfInterest* poi = new PointOfInterest(id, type, color, pos, layer);
+                if (toFill.add(poi)) {
                     parCont.push_back(poi);
                 }
             }
             break;
             case wkbLineString: {
+                bool fill = fillType < 0 ? false : (fillType == 1);
                 OGRLineString* cgeom = (OGRLineString*) poGeometry;
                 PositionVector shape;
                 for (int j = 0; j < cgeom->getNumPoints(); j++) {
@@ -161,13 +168,14 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
                     }
                     shape.push_back_noDoublePos(pos);
                 }
-                Polygon* poly = new Polygon(id, type, color, shape, false, (SUMOReal)layer);
-                if (toFill.insert(id, poly, layer)) {
+                SUMO::Polygon* poly = new SUMO::Polygon(id, type, color, shape, fill, layer);
+                if (toFill.add(poly)) {
                     parCont.push_back(poly);
                 }
             }
             break;
             case wkbPolygon: {
+                bool fill = fillType < 0 ? true : (fillType == 1);
                 OGRLinearRing* cgeom = ((OGRPolygon*) poGeometry)->getExteriorRing();
                 PositionVector shape;
                 for (int j = 0; j < cgeom->getNumPoints(); j++) {
@@ -177,8 +185,8 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
                     }
                     shape.push_back_noDoublePos(pos);
                 }
-                Polygon* poly = new Polygon(id, type, color, shape, true, (SUMOReal)layer);
-                if (toFill.insert(id, poly, layer)) {
+                SUMO::Polygon* poly = new SUMO::Polygon(id, type, color, shape, fill, layer);
+                if (toFill.add(poly)) {
                     parCont.push_back(poly);
                 }
             }
@@ -192,14 +200,15 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
                     if (!geoConvHelper.x2cartesian(pos)) {
                         WRITE_ERROR("Unable to project coordinates for POI '" + tid + "'.");
                     }
-                    PointOfInterest* poi = new PointOfInterest(tid, type, color, pos, (SUMOReal)layer);
-                    if (toFill.insert(tid, poi, layer)) {
+                    PointOfInterest* poi = new PointOfInterest(tid, type, color, pos, layer);
+                    if (toFill.add(poi)) {
                         parCont.push_back(poi);
                     }
                 }
             }
             break;
             case wkbMultiLineString: {
+                bool fill = fillType < 0 ? false : (fillType == 1);
                 OGRMultiLineString* cgeom = (OGRMultiLineString*) poGeometry;
                 for (int i = 0; i < cgeom->getNumGeometries(); ++i) {
                     OGRLineString* cgeom2 = (OGRLineString*) cgeom->getGeometryRef(i);
@@ -212,14 +221,15 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
                         }
                         shape.push_back_noDoublePos(pos);
                     }
-                    Polygon* poly = new Polygon(tid, type, color, shape, false, (SUMOReal)layer);
-                    if (toFill.insert(tid, poly, layer)) {
+                    SUMO::Polygon* poly = new SUMO::Polygon(tid, type, color, shape, fill, layer);
+                    if (toFill.add(poly)) {
                         parCont.push_back(poly);
                     }
                 }
             }
             break;
             case wkbMultiPolygon: {
+                bool fill = fillType < 0 ? true : (fillType == 1);
                 OGRMultiPolygon* cgeom = (OGRMultiPolygon*) poGeometry;
                 for (int i = 0; i < cgeom->getNumGeometries(); ++i) {
                     OGRLinearRing* cgeom2 = ((OGRPolygon*) cgeom->getGeometryRef(i))->getExteriorRing();
@@ -232,8 +242,8 @@ PCLoaderArcView::load(const std::string& file, OptionsCont& oc, PCPolyContainer&
                         }
                         shape.push_back_noDoublePos(pos);
                     }
-                    Polygon* poly = new Polygon(tid, type, color, shape, true, (SUMOReal)layer);
-                    if (toFill.insert(tid, poly, layer)) {
+                    SUMO::Polygon* poly = new SUMO::Polygon(tid, type, color, shape, fill, layer);
+                    if (toFill.add(poly)) {
                         parCont.push_back(poly);
                     }
                 }
