@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Thu, 02.11.2006
-/// @version $Id: PCLoaderXML.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: PCLoaderXML.h 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // A reader for polygons and pois stored in XML-format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -35,7 +35,7 @@
 #include <string>
 #include "PCPolyContainer.h"
 #include "PCTypeMap.h"
-#include <utils/xml/SUMOSAXHandler.h>
+#include <utils/shapes/ShapeHandler.h>
 #include <utils/common/UtilExceptions.h>
 
 
@@ -55,7 +55,7 @@ class OptionsCont;
  * Reads pois stored as XML definition. The definitions must match
  *  the format POLYCONVERT generates.
  */
-class PCLoaderXML : public SUMOSAXHandler {
+class PCLoaderXML : public ShapeHandler {
 public:
     /** @brief Loads pois/polygons assumed to be stored as XML
      *
@@ -72,6 +72,8 @@ public:
                           PCTypeMap& tm);
 
 
+    Position getLanePos(const std::string& poiID, const std::string& laneID, SUMOReal lanePos);
+
 protected:
     /** @brief Constructor
      * @param[in] toFill The poly/pois container to add loaded polys/pois to
@@ -102,9 +104,6 @@ protected:
 
 
 private:
-    /// @brief The container to store the converted polygons/pois into
-    PCPolyContainer& myCont;
-
     /// @brief The type map to use
     PCTypeMap& myTypeMap;
 
