@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONet.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: RONet.cpp 20811 2016-05-30 11:05:58Z behrisch $
 ///
 // The router's network representation
 /****************************************************************************/
@@ -94,6 +94,24 @@ RONet::RONet()
 
 
 RONet::~RONet() {
+    for (RoutablesMap::iterator routables = myRoutables.begin(); routables != myRoutables.end(); ++routables) {
+        for (std::deque<RORoutable*>::iterator r = routables->second.begin(); r != routables->second.end(); ++r) {
+            const ROVehicle* const veh = dynamic_cast<const ROVehicle*>(*r);
+            // delete routes and the vehicle
+            if (veh != 0 && veh->getRouteDefinition()->getID()[0] == '!') {
+                if (!myRoutes.erase(veh->getRouteDefinition()->getID())) {
+                    delete veh->getRouteDefinition();
+                }
+            }
+            delete *r;
+        }
+    }
+    for (std::map<std::string, SUMOVehicleParameter::Stop*>::iterator it = myBusStops.begin(); it != myBusStops.end(); ++it) {
+        delete it->second;
+    }
+    for (std::map<std::string, SUMOVehicleParameter::Stop*>::iterator it = myContainerStops.begin(); it != myContainerStops.end(); ++it) {
+        delete it->second;
+    }
     myNodes.clear();
     myEdges.clear();
     myVehicleTypes.clear();
@@ -209,20 +227,19 @@ RONet::addRouteDef(RORouteDef* def) {
 
 
 void
-RONet::openOutput(const std::string& filename, const std::string altFilename, const std::string typeFilename) {
-    if (filename != "") {
-        myRoutesOutput = &OutputDevice::getDevice(filename);
-        myRoutesOutput->writeHeader<ROEdge>(SUMO_TAG_ROUTES);
-        myRoutesOutput->writeAttr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance").writeAttr("xsi:noNamespaceSchemaLocation", "http://sumo.dlr.de/xsd/routes_file.xsd");
+RONet::openOutput(const OptionsCont& options, const std::string altFilename) {
+    if (options.isSet("output-file") && options.getString("output-file") != "") {
+        OutputDevice::createDeviceByOption("output-file", "routes", "routes_file.xsd");
+        myRoutesOutput = &OutputDevice::getDeviceByOption("output-file");
     }
     if (altFilename != "") {
         myRouteAlternativesOutput = &OutputDevice::getDevice(altFilename);
         myRouteAlternativesOutput->writeHeader<ROEdge>(SUMO_TAG_ROUTES);
         myRouteAlternativesOutput->writeAttr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance").writeAttr("xsi:noNamespaceSchemaLocation", "http://sumo.dlr.de/xsd/routes_file.xsd");
     }
-    if (typeFilename != "") {
-        myTypesOutput = &OutputDevice::getDevice(typeFilename);
-        myTypesOutput->writeXMLHeader("routes", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/routes_file.xsd\"");
+    if (options.isSet("vtype-output") && options.getString("vtype-output") != "") {
+        OutputDevice::createDeviceByOption("vtype-output", "routes", "routes_file.xsd");
+        myTypesOutput = &OutputDevice::getDeviceByOption("vtype-output");
     }
 }
 
@@ -590,6 +607,7 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, const RORouterProvider& pr
             if (routables->second.empty()) {
                 myRoutables.erase(routables);
             }
+            delete r;
         }
         if (containerTime == minTime) {
             myRoutesOutput->writePreformattedTag(container->second);
