@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: ROVehicle.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: ROVehicle.cpp 20542 2016-04-25 11:40:14Z namdre $
 ///
 // A vehicle as used by router
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -54,30 +54,43 @@
 // method definitions
 // ===========================================================================
 ROVehicle::ROVehicle(const SUMOVehicleParameter& pars,
-                     RORouteDef* route, const SUMOVTypeParameter* type, const RONet* net)
-    : myParameter(pars), myType(type), myRoute(route), myRoutingSuccess(false) {
+                     RORouteDef* route, const SUMOVTypeParameter* type,
+                     const RONet* net, MsgHandler* errorHandler)
+    : RORoutable(pars, type), myRoute(route) {
     myParameter.stops.clear();
-    if (route != 0) {
+    if (route != 0 && route->getFirstRoute() != 0) {
         for (std::vector<SUMOVehicleParameter::Stop>::const_iterator s = route->getFirstRoute()->getStops().begin(); s != route->getFirstRoute()->getStops().end(); ++s) {
-            addStop(*s, net);
+            addStop(*s, net, errorHandler);
         }
     }
     for (std::vector<SUMOVehicleParameter::Stop>::const_iterator s = pars.stops.begin(); s != pars.stops.end(); ++s) {
-        addStop(*s, net);
+        addStop(*s, net, errorHandler);
+    }
+    if (pars.via.size() != 0) {
+        // via takes precedence over stop edges
+        // XXX check for inconsistencies #2275
+        myStopEdges.clear();
+        for (std::vector<std::string>::const_iterator it = pars.via.begin(); it != pars.via.end(); ++it) {
+            assert(net->getEdge(*it) != 0);
+            myStopEdges.push_back(net->getEdge(*it));
+        }
     }
 }
 
 
 void
-ROVehicle::addStop(const SUMOVehicleParameter::Stop& stopPar, const RONet* net) {
-    const ROEdge* stopEdge = net->getEdge(stopPar.lane.substr(0, stopPar.lane.rfind("_")));
-    if (stopEdge == 0) {
-        // warn here?
+ROVehicle::addStop(const SUMOVehicleParameter::Stop& stopPar, const RONet* net, MsgHandler* errorHandler) {
+    const ROEdge* stopEdge = net->getEdgeForLaneID(stopPar.lane);
+    assert(stopEdge != 0); // was checked when parsing the stop
+    if (stopEdge->prohibits(this)) {
+        if (errorHandler != 0) {
+            errorHandler->inform("Stop edge '" + stopEdge->getID() + "' does not allow vehicle '" + getID() + "'.");
+        }
         return;
     }
     // where to insert the stop
     std::vector<SUMOVehicleParameter::Stop>::iterator iter = myParameter.stops.begin();
-    std::vector<const ROEdge*>::iterator edgeIter = myStopEdges.begin();
+    ConstROEdgeVector::iterator edgeIter = myStopEdges.begin();
     if (stopPar.index == STOP_INDEX_END || stopPar.index >= static_cast<int>(myParameter.stops.size())) {
         if (myParameter.stops.size() > 0) {
             iter = myParameter.stops.end();
@@ -85,8 +98,8 @@ ROVehicle::addStop(const SUMOVehicleParameter::Stop& stopPar, const RONet* net)
         }
     } else {
         if (stopPar.index == STOP_INDEX_FIT) {
-            const std::vector<const ROEdge*> edges = myRoute->getFirstRoute()->getEdgeVector();
-            std::vector<const ROEdge*>::const_iterator stopEdgeIt = std::find(edges.begin(), edges.end(), stopEdge);
+            const ConstROEdgeVector edges = myRoute->getFirstRoute()->getEdgeVector();
+            ConstROEdgeVector::const_iterator stopEdgeIt = std::find(edges.begin(), edges.end(), stopEdge);
             if (stopEdgeIt == edges.end()) {
                 iter = myParameter.stops.end();
                 edgeIter = myStopEdges.end();
@@ -112,48 +125,74 @@ ROVehicle::addStop(const SUMOVehicleParameter::Stop& stopPar, const RONet* net)
 ROVehicle::~ROVehicle() {}
 
 
-void
-ROVehicle::saveAllAsXML(OutputDevice& os, OutputDevice* const altos,
-                        OutputDevice* const typeos, bool withExitTimes) const {
-    // check whether the vehicle's type was saved before
-    if (myType != 0 && !myType->saved) {
-        // ... save if not
-        if (typeos != 0) {
-            myType->write(*typeos);
-        } else {
-            myType->write(os);
-            if (altos != 0) {
-                myType->write(*altos);
-            }
-        }
-        myType->saved = true;
-    }
-
-    // write the vehicle (new style, with included routes)
-    myParameter.write(os, OptionsCont::getOptions());
-    if (altos != 0) {
-        myParameter.write(*altos, OptionsCont::getOptions());
-    }
-
-    // save the route
-    myRoute->writeXMLDefinition(os, this, false, withExitTimes);
-    if (altos != 0) {
-        myRoute->writeXMLDefinition(*altos, this, true, withExitTimes);
-    }
-    myParameter.writeStops(os);
-    if (altos != 0) {
-        myParameter.writeStops(*altos);
-    }
-    os.closeTag();
-    if (altos != 0) {
-        altos->closeTag();
-    }
+const ROEdge*
+ROVehicle:: getDepartEdge() const {
+    return myRoute->getFirstRoute()->getFirst();
 }
 
 
-SUMOReal
-ROVehicle::getMaxSpeed() const {
-    return myType->maxSpeed;
+void
+ROVehicle::computeRoute(const RORouterProvider& provider,
+                        const bool removeLoops, MsgHandler* errorHandler) {
+    SUMOAbstractRouter<ROEdge, ROVehicle>& router = provider.getVehicleRouter();
+    std::string noRouteMsg = "The vehicle '" + getID() + "' has no valid route.";
+    RORouteDef* const routeDef = getRouteDefinition();
+    // check if the route definition is valid
+    if (routeDef == 0) {
+        errorHandler->inform(noRouteMsg);
+        myRoutingSuccess = false;
+        return;
+    }
+    RORoute* current = routeDef->buildCurrentRoute(router, getDepartureTime(), *this);
+    if (current == 0 || current->size() == 0) {
+        delete current;
+        errorHandler->inform(noRouteMsg);
+        myRoutingSuccess = false;
+        return;
+    }
+    // check whether we have to evaluate the route for not containing loops
+    if (removeLoops) {
+        current->recheckForLoops();
+        // check whether the route is still valid
+        if (current->size() == 0) {
+            delete current;
+            errorHandler->inform(noRouteMsg + " (after removing loops)");
+            myRoutingSuccess = false;
+            return;
+        }
+    }
+    // add built route
+    routeDef->addAlternative(router, this, current, getDepartureTime());
+    myRoutingSuccess = true;
+}
+
+
+void
+ROVehicle::saveAsXML(OutputDevice& os, OutputDevice* const typeos, bool asAlternatives, OptionsCont& options) const {
+    if (typeos != 0  && myType != 0 && !myType->saved) {
+        myType->write(*typeos);
+        myType->saved = true;
+    }
+    if (myType != 0 && !myType->saved) {
+        myType->write(os);
+        myType->saved = asAlternatives;
+    }
+
+    // write the vehicle (new style, with included routes)
+    myParameter.write(os, options);
+
+    // save the route
+    myRoute->writeXMLDefinition(os, this, asAlternatives, options.getBool("exit-times"));
+    for (std::vector<SUMOVehicleParameter::Stop>::const_iterator stop = myParameter.stops.begin(); stop != myParameter.stops.end(); ++stop) {
+        stop->write(os);
+    }
+    for (std::map<std::string, std::string>::const_iterator j = myParameter.getMap().begin(); j != myParameter.getMap().end(); ++j) {
+        os.openTag(SUMO_TAG_PARAM);
+        os.writeAttr(SUMO_ATTR_KEY, (*j).first);
+        os.writeAttr(SUMO_ATTR_VALUE, (*j).second);
+        os.closeTag();
+    }
+    os.closeTag();
 }
 
 
