@@ -6,7 +6,7 @@
 /// @author  Jakob Erdmann
 /// @author  Christoph Sommer
 /// @date    Sept 2002
-/// @version $Id: TraCIServerAPI_Polygon.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: TraCIServerAPI_Polygon.cpp 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // APIs for getting/setting polygon values via TraCI
 /****************************************************************************/
@@ -79,7 +79,7 @@ TraCIServerAPI_Polygon::processGet(TraCIServer& server, tcpip::Storage& inputSto
             tempMsg.writeInt((int) ids.size());
         }
     } else {
-        Polygon* p = getPolygon(id);
+        SUMO::Polygon* p = getPolygon(id);
         if (p == 0) {
             return server.writeErrorStatusCmd(CMD_GET_POLYGON_VARIABLE, "Polygon '" + id + "' is not known", outputStorage);
         }
@@ -138,7 +138,7 @@ TraCIServerAPI_Polygon::processSet(TraCIServer& server, tcpip::Storage& inputSto
     }
     // id
     std::string id = inputStorage.readString();
-    Polygon* p = 0;
+    SUMO::Polygon* p = 0;
     ShapeContainer& shapeCont = MSNet::getInstance()->getShapeContainer();
     if (variable != ADD && variable != REMOVE) {
         p = getPolygon(id);
@@ -252,7 +252,7 @@ TraCIServerAPI_Polygon::processSet(TraCIServer& server, tcpip::Storage& inputSto
 
 bool
 TraCIServerAPI_Polygon::getShape(const std::string& id, PositionVector& shape) {
-    Polygon* poly = getPolygon(id);
+    SUMO::Polygon* poly = getPolygon(id);
     if (poly == 0) {
         return false;
     }
@@ -261,7 +261,7 @@ TraCIServerAPI_Polygon::getShape(const std::string& id, PositionVector& shape) {
 }
 
 
-Polygon*
+SUMO::Polygon*
 TraCIServerAPI_Polygon::getPolygon(const std::string& id) {
     return MSNet::getInstance()->getShapeContainer().getPolygons().get(id);
 }
@@ -271,8 +271,8 @@ NamedRTree*
 TraCIServerAPI_Polygon::getTree() {
     NamedRTree* t = new NamedRTree();
     ShapeContainer& shapeCont = MSNet::getInstance()->getShapeContainer();
-    const std::map<std::string, Polygon*>& polygons = shapeCont.getPolygons().getMyMap();
-    for (std::map<std::string, Polygon*>::const_iterator i = polygons.begin(); i != polygons.end(); ++i) {
+    const std::map<std::string, SUMO::Polygon*>& polygons = shapeCont.getPolygons().getMyMap();
+    for (std::map<std::string, SUMO::Polygon*>::const_iterator i = polygons.begin(); i != polygons.end(); ++i) {
         Boundary b = (*i).second->getShape().getBoxBoundary();
         const float cmin[2] = {(float) b.xmin(), (float) b.ymin()};
         const float cmax[2] = {(float) b.xmax(), (float) b.ymax()};
