@@ -5,7 +5,7 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Sept 2002
-/// @version $Id: PositionVector.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: PositionVector.cpp 21057 2016-06-29 09:04:33Z palcraft $
 ///
 // A list of positions
 /****************************************************************************/
@@ -55,6 +55,7 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
+
 PositionVector::PositionVector() {}
 
 
@@ -145,9 +146,7 @@ PositionVector::intersects(const PositionVector& v1) const {
 
 
 Position
-PositionVector::intersectionPosition2D(const Position& p1,
-                                       const Position& p2,
-                                       const SUMOReal withinDist) const {
+PositionVector::intersectionPosition2D(const Position& p1, const Position& p2, const SUMOReal withinDist) const {
     for (const_iterator i = begin(); i != end() - 1; i++) {
         SUMOReal x, y, m;
         if (intersects(*i, *(i + 1), p1, p2, withinDist, &x, &y, &m)) {
@@ -265,10 +264,9 @@ PositionVector::slopeDegreeAtOffset(SUMOReal pos) const {
     return RAD2DEG(atan2(p2.z() - p1.z(), p1.distanceTo2D(p2)));
 }
 
+
 Position
-PositionVector::positionAtOffset(const Position& p1,
-                                 const Position& p2,
-                                 SUMOReal pos, SUMOReal lateralOffset) {
+PositionVector::positionAtOffset(const Position& p1, const Position& p2, SUMOReal pos, SUMOReal lateralOffset) {
     const SUMOReal dist = p1.distanceTo(p2);
     if (pos < 0 || dist < pos) {
         return Position::INVALID;
@@ -288,9 +286,7 @@ PositionVector::positionAtOffset(const Position& p1,
 
 
 Position
-PositionVector::positionAtOffset2D(const Position& p1,
-                                   const Position& p2,
-                                   SUMOReal pos, SUMOReal lateralOffset) {
+PositionVector::positionAtOffset2D(const Position& p1, const Position& p2, SUMOReal pos, SUMOReal lateralOffset) {
     const SUMOReal dist = p1.distanceTo2D(p2);
     if (pos < 0 || dist < pos) {
         return Position::INVALID;
@@ -408,6 +404,7 @@ PositionVector::length() const {
     return len;
 }
 
+
 SUMOReal
 PositionVector::length2D() const {
     SUMOReal len = 0;
@@ -532,25 +529,24 @@ PositionVector::mirrorX() {
 }
 
 
+PositionVector::as_poly_cw_sorter::as_poly_cw_sorter() {};
+
+
 int
-PositionVector::as_poly_cw_sorter::operator()(const Position& p1,
-        const Position& p2) const {
+PositionVector::as_poly_cw_sorter::operator()(const Position& p1, const Position& p2) const {
     return atan2(p1.x(), p1.y()) < atan2(p2.x(), p2.y());
 }
 
 
-
 void
 PositionVector::sortByIncreasingXY() {
     std::sort(begin(), end(), increasing_x_y_sorter());
 }
 
 
-
 PositionVector::increasing_x_y_sorter::increasing_x_y_sorter() {}
 
 
-
 int
 PositionVector::increasing_x_y_sorter::operator()(const Position& p1,
         const Position& p2) const {
@@ -561,10 +557,8 @@ PositionVector::increasing_x_y_sorter::operator()(const Position& p1,
 }
 
 
-
 SUMOReal
-PositionVector::isLeft(const Position& P0, const Position& P1,
-                       const Position& P2) const {
+PositionVector::isLeft(const Position& P0, const Position& P1,  const Position& P2) const {
     return (P1.x() - P0.x()) * (P2.y() - P0.y()) - (P2.x() - P0.x()) * (P1.y() - P0.y());
 }
 
@@ -718,10 +712,10 @@ PositionVector::nearest_offset_to_point2D(const Position& p, bool perpendicular)
 
 Position
 PositionVector::transformToVectorCoordinates(const Position& p, bool extend) const {
-    // XXX this duplicates most of the code in nearest_offset_to_point2D. It should be refactored
+    // @toDo this duplicates most of the code in nearest_offset_to_point2D. It should be refactored
     if (extend) {
         PositionVector extended = *this;
-        const SUMOReal dist = 2 * distance(p);
+        const SUMOReal dist = 2 * distance2D(p);
         extended.extrapolate(dist);
         return extended.transformToVectorCoordinates(p) - Position(dist, 0);
     }
@@ -950,13 +944,13 @@ PositionVector::distances(const PositionVector& s, bool perpendicular) const {
     std::vector<SUMOReal> ret;
     const_iterator i;
     for (i = begin(); i != end(); i++) {
-        const SUMOReal dist = s.distance(*i, perpendicular);
+        const SUMOReal dist = s.distance2D(*i, perpendicular);
         if (dist != GeomHelper::INVALID_OFFSET) {
             ret.push_back(dist);
         }
     }
     for (i = s.begin(); i != s.end(); i++) {
-        const SUMOReal dist = distance(*i, perpendicular);
+        const SUMOReal dist = distance2D(*i, perpendicular);
         if (dist != GeomHelper::INVALID_OFFSET) {
             ret.push_back(dist);
         }
@@ -966,7 +960,7 @@ PositionVector::distances(const PositionVector& s, bool perpendicular) const {
 
 
 SUMOReal
-PositionVector::distance(const Position& p, bool perpendicular) const {
+PositionVector::distance2D(const Position& p, bool perpendicular) const {
     if (size() == 0) {
         return std::numeric_limits<double>::max();
     } else if (size() == 1) {
@@ -980,6 +974,7 @@ PositionVector::distance(const Position& p, bool perpendicular) const {
     }
 }
 
+
 void
 PositionVector::push_back_noDoublePos(const Position& p) {
     if (size() == 0 || !p.almostSame(back())) {
@@ -1048,10 +1043,7 @@ PositionVector::hasElevation() const {
 
 
 bool
-PositionVector::intersects(const Position& p11, const Position& p12,
-                           const Position& p21, const Position& p22,
-                           const SUMOReal withinDist,
-                           SUMOReal* x, SUMOReal* y, SUMOReal* mu) {
+PositionVector::intersects(const Position& p11, const Position& p12, const Position& p21, const Position& p22, const SUMOReal withinDist, SUMOReal* x, SUMOReal* y, SUMOReal* mu) {
     const SUMOReal eps = std::numeric_limits<SUMOReal>::epsilon();
     const double denominator = (p22.y() - p21.y()) * (p12.x() - p11.x()) - (p22.x() - p21.x()) * (p12.y() - p11.y());
     const double numera = (p22.x() - p21.x()) * (p11.y() - p21.y()) - (p22.y() - p21.y()) * (p11.x() - p21.x());
