@@ -5,7 +5,7 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Oct 2002
-/// @version $Id: GUIGlObject.h 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: GUIGlObject.h 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // Base class for all objects that may be displayed within the openGL-gui
 /****************************************************************************/
@@ -46,12 +46,13 @@
 // ===========================================================================
 // definitions
 // ===========================================================================
-typedef unsigned int GUIGlID;
 
+typedef unsigned int GUIGlID;
 
 // ===========================================================================
 // class declarations
 // ===========================================================================
+
 class GUIGlObjectStorage;
 class GUIParameterTableWindow;
 class GUIMainWindow;
@@ -65,12 +66,15 @@ class Node;
 }
 #endif
 
-
 // ===========================================================================
 // class definitions
 // ===========================================================================
+
 class GUIGlObject {
 public:
+    /// @brief associates object types with strings
+    static StringBijection<GUIGlObjectType> TypeNames;
+
     /** @brief Constructor
      *
      * This is the standard constructor that assures that the object is known
@@ -81,7 +85,6 @@ public:
      */
     GUIGlObject(GUIGlObjectType type, const std::string& microsimID);
 
-
     /** @brief Constructor
      *
      * This constructor should be used only for compound objects, that share
@@ -92,43 +95,26 @@ public:
      */
     GUIGlObject(const std::string& prefix, GUIGlObjectType type, const std::string& microsimID);
 
-
     /// @brief Destructor
     virtual ~GUIGlObject();
 
-    /// @brief associates object types with strings
-    static StringBijection<GUIGlObjectType> TypeNames;
-
     /// @name Atomar getter methods
     /// @{
+    /// @brief Returns the full name appearing in the tool tip
+    /// @return This object's typed id
+    const std::string& getFullName() const;
 
-    /** @brief Returns the full name appearing in the tool tip
-     * @return This object's typed id
-     */
-    const std::string& getFullName() const {
-        return myFullName;
-    }
+    /// @brief Returns the name of the parent object (if any)
+    /// @return This object's parent id
+    virtual const std::string& getParentName() const;
 
-    /** @brief Returns the name of the parent object (if any)
-     * @return This object's parent id
-     */
-    virtual const std::string& getParentName() const {
-        return StringUtils::emptyString;
-    }
-
-
-    /** @brief Returns the numerical id of the object
-     * @return This object's gl-id
-     */
-    GUIGlID getGlID() const {
-        return myGlID;
-    }
+    /// @brief Returns the numerical id of the object
+    /// @return This object's gl-id
+    GUIGlID getGlID() const;
     /// @}
 
-
     /// @name interfaces to be implemented by derived classes
-    //@{
-
+    /// @{
     /** @brief Returns an own popup-menu
      *
      * @param[in] app The application needed to build the popup-menu
@@ -137,7 +123,6 @@ public:
      */
     virtual GUIGLObjectPopupMenu* getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) = 0;
 
-
     /** @brief Returns an own parameter window
      *
      * @param[in] app The application needed to build the parameter window
@@ -146,141 +131,100 @@ public:
      */
     virtual GUIParameterTableWindow* getParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent) = 0;
 
-
     /** @brief Returns an own type parameter window (optional)
      *
      * @param[in] app The application needed to build the parameter window
      * @param[in] parent The parent window needed to build the parameter window
      * @return The built parameter window
      */
-    virtual GUIParameterTableWindow* getTypeParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent) {
-        UNUSED_PARAMETER(&app);
-        UNUSED_PARAMETER(&parent);
-        return 0;
-    }
+    virtual GUIParameterTableWindow* getTypeParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent);
 
+    /// @brief Returns the id of the object as known to microsim
+    virtual const std::string& getMicrosimID() const;
 
-    /** @brief Returns the id of the object as known to microsim
-     *
-     * @return The id of the object
-     */
-    const std::string& getMicrosimID() const {
-        return myMicrosimID;
-    };
-
-
-    /** @brief Changes the microsimID of the object (happens in NETEDIT) */
+    /// @brief Changes the microsimID of the object
+    /// @note happens in NETEDIT
     virtual void setMicrosimID(const std::string& newID);
 
+    /// @brief Returns the type of the object as coded in GUIGlObjectType
+    /// @see GUIGlObjectType
+    GUIGlObjectType getType() const;
 
-    /** @brief Returns the type of the object as coded in GUIGlObjectType
-     * @return The type of the object
-     * @see GUIGlObjectType
-     */
-    GUIGlObjectType getType() const {
-        return myGLObjectType;
-    };
-
-
-    /** @brief Returns the boundary to which the view shall be centered in order to show the object
-     *
-     * @return The boundary the object is within
-     */
+    //// @brief Returns the boundary to which the view shall be centered in order to show the object
     virtual Boundary getCenteringBoundary() const = 0;
 
-
-    /** @brief Draws the object
-     * @param[in] s The settings for the current view (may influence drawing)
-     */
+    /// @brief Draws the object
+    /// @param[in] s The settings for the current view (may influence drawing)
     virtual void drawGL(const GUIVisualizationSettings& s) const = 0;
-    //@}
-
+    /// @}
 
     /** @brief Draws additional, user-triggered visualisations
      * @param[in] parent The view
      * @param[in] s The settings for the current view (may influence drawing)
      */
-    virtual void drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualizationSettings& s) const {
-        UNUSED_PARAMETER(&s);
-        UNUSED_PARAMETER(parent);
-    };
+    virtual void drawGLAdditional(GUISUMOAbstractView* const parent, const GUIVisualizationSettings& s) const;
 
 #ifdef HAVE_OSG
-    void setNode(osg::Node* node) {
-        myOSGNode = node;
-    }
+    /// @brief get OSG Node
+    osg::Node* getNode() const;
 
-    osg::Node* getNode() const {
-        return myOSGNode;
-    }
+    /// @brief set OSG Node
+    void setNode(osg::Node* node);
 #endif
 
     /// @name Parameter table window I/O
     /// @{
-
-    /** @brief Lets this object know a parameter window showing the object's values was opened
-     * @param[in] w The opened parameter window
-     */
+    /// @brief Lets this object know a parameter window showing the object's values was opened
+    /// @param[in] w The opened parameter window
     void addParameterTable(GUIParameterTableWindow* w);
 
-
-    /** @brief Lets this object know a parameter window showing the object's values was closed
-     * @param[in] w The closed parameter window
-     */
+    /// @brief Lets this object know a parameter window showing the object's values was closed
+    /// @param[in] w The closed parameter window
     void removeParameterTable(GUIParameterTableWindow* w);
     /// @}
 
-
-    void drawName(const Position& pos, const SUMOReal scale,
-                  const GUIVisualizationTextSettings& settings, const SUMOReal angle = 0) const;
-
+    /// @brief draw name of item
+    void drawName(const Position& pos, const SUMOReal scale, const GUIVisualizationTextSettings& settings, const SUMOReal angle = 0) const;
 
 protected:
     /// @name helper methods for building popup-menus
-    //@{
-
+    /// @{
     /** @brief Builds the header
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildPopupHeader(GUIGLObjectPopupMenu* ret, GUIMainWindow& app, bool addSeparator = true);
 
-
     /** @brief Builds an entry which allows to center to the object
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildCenterPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
 
-
     /** @brief Builds entries which allow to copy the name / typed name into the clipboard
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildNameCopyPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
 
-
     /** @brief Builds an entry which allows to (de)select the object
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildSelectionPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
 
-
     /** @brief Builds an entry which allows to open the parameter window
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildShowParamsPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
 
-
     /** @brief Builds an entry which allows to open the type parameter window
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildShowTypeParamsPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
 
-
     /** @brief Builds an entry which allows to copy the cursor position
      *   if geo projection is used, also builds an entry for copying the geo-position
      * @param[in, filled] ret The popup menu to add the entry to
@@ -288,17 +232,15 @@ protected:
      */
     void buildPositionCopyEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
 
-
     /** @brief Builds an entry which allows to open the manipulator window
      * @param[in, filled] ret The popup menu to add the entry to
      * @param[in] addSeparator Whether a separator shall be added, too
      */
     void buildShowManipulatorPopupEntry(GUIGLObjectPopupMenu* ret, bool addSeparator = true);
-    //@}
+    /// @}
 
 protected:
-    /* usually names are prefixed by a type-specific string. this method can be
-     * used to change the default */
+    /// @brief usually names are prefixed by a type-specific string. this method can be used to change the default
     void setPrefix(const std::string& prefix);
 
 private:
@@ -308,23 +250,30 @@ private:
     /// @brief The type of the object
     const GUIGlObjectType myGLObjectType;
 
+    /// @brief ID of GL object
     std::string myMicrosimID;
 
+    /// @brief prefix of GL Object
     std::string myPrefix;
 
+    /// @brief full name of GL Object
     std::string myFullName;
 
     /// @brief Parameter table windows which refer to this object
     std::set<GUIParameterTableWindow*> myParamWindows;
 
+    /// @brief create full name
     std::string createFullName() const;
 
 #ifdef HAVE_OSG
+    /// @brief OSG Node of this GL object
     osg::Node* myOSGNode;
 #endif
 
+    /// @brief LinkStates (Currently unused)
     // static StringBijection<SumoXMLLinkStateValue> LinkStates;
 
+    /// @brief vector for TypeNames Initializer
     static StringBijection<GUIGlObjectType>::Entry GUIGlObjectTypeNamesInitializer[];
 
 private:
@@ -333,7 +282,6 @@ private:
 
     /// @brief Invalidated assignment operator.
     GUIGlObject& operator=(const GUIGlObject&);
-
 };
 #endif
 
