@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIPerspectiveChanger.cpp 20433 2016-04-13 08:00:14Z behrisch $
+/// @version $Id: GUIPerspectiveChanger.cpp 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // A class that allows to steer the visual output in dependence to user
 /****************************************************************************/
@@ -41,18 +41,19 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-GUIPerspectiveChanger::GUIPerspectiveChanger(
-    GUISUMOAbstractView& callBack,
-    const Boundary& viewPort) :
+GUIPerspectiveChanger::GUIPerspectiveChanger(GUISUMOAbstractView& callBack, const Boundary& viewPort) :
     myCallback(callBack),
-    myViewPort(viewPort) {}
+    myViewPort(viewPort) {
+}
 
 
-GUIPerspectiveChanger::~GUIPerspectiveChanger() {}
+GUIPerspectiveChanger::~GUIPerspectiveChanger() {
+}
 
 
 void
-GUIPerspectiveChanger::onLeftBtnPress(void*) {}
+GUIPerspectiveChanger::onLeftBtnPress(void*) {
+}
 
 
 bool
@@ -62,7 +63,8 @@ GUIPerspectiveChanger::onLeftBtnRelease(void*) {
 
 
 void
-GUIPerspectiveChanger::onRightBtnPress(void*) {}
+GUIPerspectiveChanger::onRightBtnPress(void*) {
+}
 
 
 bool
@@ -70,24 +72,33 @@ GUIPerspectiveChanger::onRightBtnRelease(void*) {
     return false;
 }
 
-
 void
-GUIPerspectiveChanger::onMouseWheel(void*) {}
+GUIPerspectiveChanger::onDoubleClicked(void* data) {
+}
 
 
 void
-GUIPerspectiveChanger::onMouseMove(void*) {}
+GUIPerspectiveChanger::onMouseWheel(void*) {
+}
+
+
+void
+GUIPerspectiveChanger::onMouseMove(void*) {
+}
+
 
 long
 GUIPerspectiveChanger::onKeyPress(void*) {
     return 0;
 }
 
+
 long
 GUIPerspectiveChanger::onKeyRelease(void*) {
     return 0;
 }
 
+
 FXint
 GUIPerspectiveChanger::getMouseXPosition() const {
     return myMouseXPosition;
@@ -100,6 +111,22 @@ GUIPerspectiveChanger::getMouseYPosition() const {
 }
 
 
+Boundary
+GUIPerspectiveChanger::getViewport(bool fixRatio) {
+    if (fixRatio) {
+        return patchedViewPort();
+    } else {
+        return myViewPort;
+    }
+}
+
+
+void
+GUIPerspectiveChanger::setViewport(const Boundary& viewPort) {
+    myViewPort = viewPort;
+}
+
+
 Boundary
 GUIPerspectiveChanger::patchedViewPort() {
     // avoid division by zero
