@@ -6,7 +6,7 @@
 /// @author  Laura Bieker
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id: GUISUMOAbstractView.cpp 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: GUISUMOAbstractView.cpp 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // The base class for a view
 /****************************************************************************/
@@ -84,19 +84,20 @@
  * GUISUMOAbstractView - FOX callback mapping
  * ----------------------------------------------------------------------- */
 FXDEFMAP(GUISUMOAbstractView) GUISUMOAbstractViewMap[] = {
-    FXMAPFUNC(SEL_CONFIGURE,           0,                 GUISUMOAbstractView::onConfigure),
-    FXMAPFUNC(SEL_PAINT,               0,                 GUISUMOAbstractView::onPaint),
-    FXMAPFUNC(SEL_LEFTBUTTONPRESS,     0,                 GUISUMOAbstractView::onLeftBtnPress),
-    FXMAPFUNC(SEL_LEFTBUTTONRELEASE,   0,                 GUISUMOAbstractView::onLeftBtnRelease),
-    FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,   0,                 GUISUMOAbstractView::onMiddleBtnPress),
-    FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE, 0,                 GUISUMOAbstractView::onMiddleBtnRelease),
-    FXMAPFUNC(SEL_RIGHTBUTTONPRESS,    0,                 GUISUMOAbstractView::onRightBtnPress),
-    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,  0,                 GUISUMOAbstractView::onRightBtnRelease),
-    FXMAPFUNC(SEL_MOUSEWHEEL,          0,                 GUISUMOAbstractView::onMouseWheel),
-    FXMAPFUNC(SEL_MOTION,              0,                 GUISUMOAbstractView::onMouseMove),
-    FXMAPFUNC(SEL_LEAVE,               0,                 GUISUMOAbstractView::onMouseLeft),
-    FXMAPFUNC(SEL_KEYPRESS,            0,                 GUISUMOAbstractView::onKeyPress),
-    FXMAPFUNC(SEL_KEYRELEASE,          0,                 GUISUMOAbstractView::onKeyRelease),
+    FXMAPFUNC(SEL_CONFIGURE,            0,      GUISUMOAbstractView::onConfigure),
+    FXMAPFUNC(SEL_PAINT,                0,      GUISUMOAbstractView::onPaint),
+    FXMAPFUNC(SEL_LEFTBUTTONPRESS,      0,      GUISUMOAbstractView::onLeftBtnPress),
+    FXMAPFUNC(SEL_LEFTBUTTONRELEASE,    0,      GUISUMOAbstractView::onLeftBtnRelease),
+    FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,    0,      GUISUMOAbstractView::onMiddleBtnPress),
+    FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,  0,      GUISUMOAbstractView::onMiddleBtnRelease),
+    FXMAPFUNC(SEL_RIGHTBUTTONPRESS,     0,      GUISUMOAbstractView::onRightBtnPress),
+    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,   0,      GUISUMOAbstractView::onRightBtnRelease),
+    FXMAPFUNC(SEL_DOUBLECLICKED,        0,      GUISUMOAbstractView::onDoubleClicked),
+    FXMAPFUNC(SEL_MOUSEWHEEL,           0,      GUISUMOAbstractView::onMouseWheel),
+    FXMAPFUNC(SEL_MOTION,               0,      GUISUMOAbstractView::onMouseMove),
+    FXMAPFUNC(SEL_LEAVE,                0,      GUISUMOAbstractView::onMouseLeft),
+    FXMAPFUNC(SEL_KEYPRESS,             0,      GUISUMOAbstractView::onKeyPress),
+    FXMAPFUNC(SEL_KEYRELEASE,           0,      GUISUMOAbstractView::onKeyRelease),
 
 };
 
@@ -107,26 +108,22 @@ FXIMPLEMENT_ABSTRACT(GUISUMOAbstractView, FXGLCanvas, GUISUMOAbstractViewMap, AR
 /* -------------------------------------------------------------------------
  * GUISUMOAbstractView - methods
  * ----------------------------------------------------------------------- */
-GUISUMOAbstractView::GUISUMOAbstractView(FXComposite* p,
-        GUIMainWindow& app,
-        GUIGlChildWindow* parent,
-        const SUMORTree& grid,
-        FXGLVisual* glVis, FXGLCanvas* share)
-    : FXGLCanvas(p, glVis, share, p, MID_GLCANVAS,
-                 LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0),
-      myApp(&app),
-      myParent(parent),
-      myGrid(&((SUMORTree&)grid)),
-      myChanger(0),
-      myMouseHotspotX(app.getDefaultCursor()->getHotX()),
-      myMouseHotspotY(app.getDefaultCursor()->getHotY()),
-      myPopup(0),
-      myUseToolTips(false),
-      myAmInitialised(false),
-      myViewportChooser(0),
-      myWindowCursorPositionX(getWidth() / 2),
-      myWindowCursorPositionY(getHeight() / 2),
-      myVisualizationChanger(0) {
+GUISUMOAbstractView::GUISUMOAbstractView(FXComposite* p, GUIMainWindow& app, GUIGlChildWindow* parent, const SUMORTree& grid, FXGLVisual* glVis, FXGLCanvas* share) :
+    FXGLCanvas(p, glVis, share, p, MID_GLCANVAS,
+               LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0),
+    myApp(&app),
+    myParent(parent),
+    myGrid(&((SUMORTree&)grid)),
+    myChanger(0),
+    myMouseHotspotX(app.getDefaultCursor()->getHotX()),
+    myMouseHotspotY(app.getDefaultCursor()->getHotY()),
+    myPopup(0),
+    myUseToolTips(false),
+    myAmInitialised(false),
+    myViewportChooser(0),
+    myWindowCursorPositionX(getWidth() / 2),
+    myWindowCursorPositionY(getHeight() / 2),
+    myVisualizationChanger(0) {
     setTarget(this);
     enable();
     flags |= FLAG_ENABLED;
@@ -159,6 +156,12 @@ GUISUMOAbstractView::isInEditMode() {
 }
 
 
+GUIPerspectiveChanger&
+GUISUMOAbstractView::getChanger() const {
+    return *myChanger;
+}
+
+
 void
 GUISUMOAbstractView::updateToolTip() {
     if (!myUseToolTips) {
@@ -178,6 +181,18 @@ GUISUMOAbstractView::getPositionInformation() const {
 }
 
 
+void
+GUISUMOAbstractView::addDecals(const std::vector<Decal>& decals) {
+    myDecals.insert(myDecals.end(), decals.begin(), decals.end());
+}
+
+
+GUIVisualizationSettings*
+GUISUMOAbstractView::getVisualisationSettings() {
+    return myVisualizationSettings;
+}
+
+
 void
 GUISUMOAbstractView::updatePositionInformation() const {
     Position pos = getPositionInformation();
@@ -193,11 +208,23 @@ GUISUMOAbstractView::updatePositionInformation() const {
 }
 
 
+int
+GUISUMOAbstractView::doPaintGL(int /*mode*/, const Boundary& /*boundary*/) {
+    return 0;
+}
+
+
+void
+GUISUMOAbstractView::doInit() {
+}
+
+
 Boundary
 GUISUMOAbstractView::getVisibleBoundary() const {
     return myChanger->getViewport();
 }
 
+
 void
 GUISUMOAbstractView::paintGL() {
     if (getWidth() == 0 || getHeight() == 0) {
@@ -608,6 +635,10 @@ GUISUMOAbstractView::onLeftBtnPress(FXObject*, FXSelector , void* data) {
     }
     myChanger->onLeftBtnPress(data);
     grab();
+    // Check there are double click
+    if (e->click_count == 2) {
+        handle(this, FXSEL(SEL_DOUBLECLICKED, 0), data);
+    }
     return 1;
 }
 
@@ -624,6 +655,18 @@ GUISUMOAbstractView::onLeftBtnRelease(FXObject*, FXSelector , void* data) {
 }
 
 
+long
+GUISUMOAbstractView::onMiddleBtnPress(FXObject*, FXSelector, void*) {
+    return 1;
+}
+
+
+long
+GUISUMOAbstractView::onMiddleBtnRelease(FXObject*, FXSelector, void*) {
+    return 1;
+}
+
+
 long
 GUISUMOAbstractView::onRightBtnPress(FXObject*, FXSelector , void* data) {
     destroyPopup();
@@ -645,6 +688,12 @@ GUISUMOAbstractView::onRightBtnRelease(FXObject* o, FXSelector sel, void* data)
 }
 
 
+long
+GUISUMOAbstractView::onDoubleClicked(FXObject*, FXSelector, void*) {
+    return 1;
+}
+
+
 long
 GUISUMOAbstractView::onMouseWheel(FXObject*, FXSelector , void* data) {
     if (!myApp->isGaming()) {
@@ -882,6 +931,13 @@ GUISUMOAbstractView::makeSnapshot(const std::string& destFile) {
 }
 
 
+void
+GUISUMOAbstractView::saveFrame(const std::string& destFile, FXColor* buf) {
+    UNUSED_PARAMETER(destFile);
+    UNUSED_PARAMETER(buf);
+}
+
+
 void
 GUISUMOAbstractView::checkSnapshots() {
     std::map<SUMOTime, std::string>::iterator snapIt = mySnapshots.find(getCurrentTimeStep());
@@ -894,6 +950,12 @@ GUISUMOAbstractView::checkSnapshots() {
 }
 
 
+SUMOTime
+GUISUMOAbstractView::getCurrentTimeStep() const {
+    return 0;
+}
+
+
 void
 GUISUMOAbstractView::showViewschemeEditor() {
     if (myVisualizationChanger == 0) {
@@ -908,6 +970,7 @@ GUISUMOAbstractView::showViewschemeEditor() {
     myVisualizationChanger->show();
 }
 
+
 GUIDialog_EditViewport*
 GUISUMOAbstractView::getViewportEditor() {
     if (myViewportChooser == 0) {
@@ -942,6 +1005,29 @@ GUISUMOAbstractView::showToolTips(bool val) {
 }
 
 
+bool
+GUISUMOAbstractView::setColorScheme(const std::string&) {
+    return true;
+}
+
+
+GUIVisualizationSettings*
+GUISUMOAbstractView::getVisualisationSettings() const {
+    return myVisualizationSettings;
+}
+
+
+void
+GUISUMOAbstractView::remove(GUIDialog_EditViewport*) {
+    myViewportChooser = 0;
+}
+
+
+void
+GUISUMOAbstractView::remove(GUIDialog_ViewSettings*) {
+    myVisualizationChanger = 0;
+}
+
 
 SUMOReal
 GUISUMOAbstractView::getGridWidth() const {
@@ -955,6 +1041,27 @@ GUISUMOAbstractView::getGridHeight() const {
 }
 
 
+void
+GUISUMOAbstractView::startTrack(int /*id*/) {
+}
+
+
+void
+GUISUMOAbstractView::stopTrack() {
+}
+
+
+int
+GUISUMOAbstractView::getTrackedID() const {
+    return -1;
+}
+
+
+void
+GUISUMOAbstractView::onGamingClick(Position /*pos*/) {
+}
+
+
 FXComboBox&
 GUISUMOAbstractView::getColoringSchemesCombo() {
     return myParent->getColoringSchemesCombo();
@@ -1055,9 +1162,7 @@ GUISUMOAbstractView::drawDecals() {
                 if (img == 0) {
                     img = MFXImageHelper::loadImage(getApp(), d.filename);
                 }
-                if (MFXImageHelper::scalePower2(img, GUITexturesHelper::getMaxTextureSize())) {
-                    WRITE_WARNING("Scaling '" + d.filename + "'.");
-                }
+                MFXImageHelper::scalePower2(img, GUITexturesHelper::getMaxTextureSize());
                 d.glID = GUITexturesHelper::add(img);
                 d.initialised = true;
                 d.image = img;
@@ -1082,7 +1187,7 @@ GUISUMOAbstractView::drawDecals() {
 
 // ------------ Additional visualisations
 bool
-GUISUMOAbstractView::addAdditionalGLVisualisation(GUIGlObject* const which) {
+GUISUMOAbstractView::addAdditionalGLVisualisation(const GUIGlObject* const which) {
     if (myAdditionallyDrawn.find(which) == myAdditionallyDrawn.end()) {
         myAdditionallyDrawn[which] = 1;
     } else {
@@ -1094,7 +1199,7 @@ GUISUMOAbstractView::addAdditionalGLVisualisation(GUIGlObject* const which) {
 
 
 bool
-GUISUMOAbstractView::removeAdditionalGLVisualisation(GUIGlObject* const which) {
+GUISUMOAbstractView::removeAdditionalGLVisualisation(const GUIGlObject* const which) {
     if (getTrackedID() == static_cast<int>(which->getGlID())) {
         stopTrack();
     }
@@ -1112,6 +1217,16 @@ GUISUMOAbstractView::removeAdditionalGLVisualisation(GUIGlObject* const which) {
 }
 
 
+bool
+GUISUMOAbstractView::isAdditionalGLVisualisationEnabled(GUIGlObject* const which) const {
+    if (myAdditionallyDrawn.find(which) == myAdditionallyDrawn.end()) {
+        return false;
+    } else {
+        return true;
+    }
+}
+
+
 void
 GUISUMOAbstractView::applyGLTransform(bool fixRatio) {
     Boundary bound = myChanger->getViewport(fixRatio);
@@ -1141,5 +1256,25 @@ GUISUMOAbstractView::setDelay(SUMOReal delay) {
     myApp->setDelay(delay);
 }
 
+
+GUISUMOAbstractView::Decal::Decal() :
+    filename(),
+    centerX(0),
+    centerY(0),
+    centerZ(0),
+    width(0),
+    height(0),
+    altitude(0),
+    rot(0),
+    tilt(0),
+    roll(0),
+    layer(0),
+    initialised(false),
+    skip2D(false),
+    glID(-1),
+    image(0) {
+}
+
+
 /****************************************************************************/
 
