@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    Mon, 17 Dec 2001
-/// @version $Id: OptionsCont.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: OptionsCont.cpp 20433 2016-04-13 08:00:14Z behrisch $
 ///
 // A storage for options (typed value containers)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -38,6 +38,8 @@
 #include <cstdlib>
 #include <cassert>
 #include <ctime>
+#include <cstring>
+#include <cerrno>
 #include <iterator>
 #include "Option.h"
 #include "OptionsCont.h"
@@ -71,7 +73,7 @@ OptionsCont::getOptions() {
 
 OptionsCont::OptionsCont()
     : myAddresses(), myValues(), myDeprecatedSynonymes(), myHaveInformedAboutDeprecatedDivider(false) {
-    myCopyrightNotices.push_back("Copyright (C) 2001-2015 DLR and contributors; http://sumo.dlr.de");
+    myCopyrightNotices.push_back("Copyright (C) 2001-2016 DLR and contributors; http://sumo.dlr.de");
 }
 
 
@@ -129,6 +131,12 @@ OptionsCont::addSynonyme(const std::string& name1, const std::string& name2, boo
 }
 
 
+void
+OptionsCont::addXMLDefault(const std::string& name, const std::string& xmlRoot) {
+    myXMLDefaults[xmlRoot] = name;
+}
+
+
 bool
 OptionsCont::exists(const std::string& name) const {
     return myValues.count(name) > 0;
@@ -255,6 +263,28 @@ OptionsCont::set(const std::string& name, const std::string& value) {
 }
 
 
+bool
+OptionsCont::setDefault(const std::string& name, const std::string& value) {
+    if (set(name, value)) {
+        getSecure(name)->resetDefault();
+        return true;
+    }
+    return false;
+}
+
+
+bool
+OptionsCont::setByRootElement(const std::string& root, const std::string& value) {
+    if (myXMLDefaults.count(root) > 0) {
+        return set(myXMLDefaults[root], value);
+    }
+    if (myXMLDefaults.count("") > 0) {
+        return set(myXMLDefaults[""], value);
+    }
+    return false;
+}
+
+
 std::vector<std::string>
 OptionsCont::getSynonymes(const std::string& name) const {
     Option* o = getSecure(name);
@@ -316,10 +346,14 @@ OptionsCont::relocateFiles(const std::string& configuration) const {
                 if (!FileHelpers::isAbsolute(tmp)) {
                     tmp = FileHelpers::getConfigurationRelative(configuration, tmp);
                 }
-                conv += tmp;
+                conv += StringUtils::urlDecode(tmp);
             }
             if (conv != (*i)->getString()) {
+                const bool hadDefault = (*i)->isDefault();
                 (*i)->set(conv);
+                if (hadDefault) {
+                    (*i)->resetDefault();
+                }
             }
         }
     }
@@ -344,7 +378,7 @@ OptionsCont::isUsableFileList(const std::string& name) const {
     for (std::vector<std::string>::const_iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt) {
         if (!FileHelpers::isReadable(*fileIt)) {
             if (*fileIt != "") {
-                WRITE_ERROR("File '" + *fileIt + "' is not accessible.");
+                WRITE_ERROR("File '" + *fileIt + "' is not accessible (" + std::strerror(errno) + ").");
                 ok = false;
             } else {
                 WRITE_WARNING("Empty file name given; ignoring.");
@@ -532,6 +566,7 @@ OptionsCont::processMetaOptions(bool missingOptions) {
     if (missingOptions) {
         // no options are given
         std::cout << myFullName << std::endl;
+        std::cout << " Build features: " << HAVE_ENABLED << std::endl;
         for (std::vector<std::string>::const_iterator it =
                     myCopyrightNotices.begin(); it != myCopyrightNotices.end(); ++it) {
             std::cout << " " << *it << std::endl;
@@ -554,6 +589,7 @@ OptionsCont::processMetaOptions(bool missingOptions) {
     // check whether the help shall be printed
     if (getBool("version")) {
         std::cout << myFullName << std::endl;
+        std::cout << " Build features: " << HAVE_ENABLED << std::endl;
         for (std::vector<std::string>::const_iterator it =
                     myCopyrightNotices.begin(); it != myCopyrightNotices.end(); ++it) {
             std::cout << " " << *it << std::endl;
