@@ -4,7 +4,7 @@
 /// @author  Mario Krumnow
 /// @author  Michael Behrisch
 /// @date    30.05.2012
-/// @version $Id: TraCIAPI.h 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: TraCIAPI.h 21131 2016-07-08 07:59:22Z behrisch $
 ///
 // C++ TraCI client API implementation
 /****************************************************************************/
@@ -34,8 +34,10 @@
 
 #include <vector>
 #include <string>
+#include <sstream>
+#include <iomanip>
 #include <foreign/tcpip/socket.h>
-#include <utils/common/SUMOTime.h>
+#include <traci-server/TraCIConstants.h>
 
 
 // ===========================================================================
@@ -51,6 +53,11 @@
  * @class TraCIAPI
  * @brief C++ TraCI client API implementation
  */
+
+
+typedef long long int SUMOTime; // <utils/common/SUMOTime.h>
+
+
 class TraCIAPI {
 public:
     /// @name Structures definitions
@@ -83,7 +90,14 @@ public:
         double xMax, yMax, zMax;
     };
 
-
+    struct TraCIValue {
+        union {
+            double scalar;
+            TraCIPosition position;
+            TraCIColor color;
+        };
+        std::string string;
+    };
 
     class TraCIPhase {
     public:
@@ -224,6 +238,7 @@ public:
         SUMOReal getNOxEmission(const std::string& edgeID) const;
         SUMOReal getFuelConsumption(const std::string& edgeID) const;
         SUMOReal getNoiseEmission(const std::string& edgeID) const;
+        SUMOReal getElectricityConsumption(const std::string& edgeID) const;
         SUMOReal getLastStepMeanSpeed(const std::string& edgeID) const;
         SUMOReal getLastStepOccupancy(const std::string& edgeID) const;
         SUMOReal getLastStepLength(const std::string& edgeID) const;
@@ -379,6 +394,7 @@ public:
         SUMOReal getNOxEmission(const std::string& laneID) const;
         SUMOReal getFuelConsumption(const std::string& laneID) const;
         SUMOReal getNoiseEmission(const std::string& laneID) const;
+        SUMOReal getElectricityConsumption(const std::string& laneID) const;
         SUMOReal getLastStepMeanSpeed(const std::string& laneID) const;
         SUMOReal getLastStepOccupancy(const std::string& laneID) const;
         SUMOReal getLastStepLength(const std::string& laneID) const;
@@ -538,6 +554,10 @@ public:
 
 
 
+    /// @brief {object->{variable->value}}
+    typedef std::map<int, TraCIValue> TraCIValues;
+    typedef std::map<std::string, TraCIValues> SubscribedValues;
+    typedef std::map<std::string, SubscribedValues> SubscribedContextValues;
 
 
     /** @class SimulationScope
@@ -563,6 +583,15 @@ public:
         TraCIBoundary getNetBoundary() const;
         unsigned int getMinExpectedNumber() const;
 
+        void subscribe(int domID, const std::string& objID, SUMOTime beginTime, SUMOTime endTime, const std::vector<int>& vars) const;
+        void subscribeContext(int domID, const std::string& objID, SUMOTime beginTime, SUMOTime endTime, int domain, SUMOReal range, const std::vector<int>& vars) const;
+
+        SubscribedValues getSubscriptionResults();
+        TraCIValues getSubscriptionResults(const std::string& objID);
+
+        SubscribedContextValues getContextSubscriptionResults();
+        SubscribedValues getContextSubscriptionResults(const std::string& objID);
+
     private:
         /// @brief invalidated copy constructor
         SimulationScope(const SimulationScope& src);
@@ -672,48 +701,95 @@ public:
         VehicleScope(TraCIAPI& parent) : TraCIScopeWrapper(parent) {}
         virtual ~VehicleScope() {}
 
+
+        struct NextTLSData {
+            /* @brief Constructor */
+            NextTLSData() {}
+            /// @brief The id of the next tls
+            std::string id;
+            /// @brief The tls index of the controlled link
+            int tlIndex;
+            /// @brief The distance to the tls
+            SUMOReal dist;
+            /// @brief The current state of the tls
+            char state;
+        };
+
+
         std::vector<std::string> getIDList() const;
         unsigned int getIDCount() const;
-        SUMOReal getSpeed(const std::string& typeID) const;
-        TraCIPosition getPosition(const std::string& typeID) const;
-        SUMOReal getAngle(const std::string& typeID) const;
-        std::string getRoadID(const std::string& typeID) const;
-        std::string getLaneID(const std::string& typeID) const;
-        unsigned int getLaneIndex(const std::string& typeID) const;
-        std::string getTypeID(const std::string& typeID) const;
-        std::string getRouteID(const std::string& typeID) const;
-        unsigned int getRouteIndex(const std::string& typeID) const;
-        std::vector<std::string> getEdges(const std::string& typeID) const;
-        TraCIColor getColor(const std::string& typeID) const;
-        SUMOReal getLanePosition(const std::string& typeID) const;
-        unsigned int getSignalStates(const std::string& typeID) const;
-        SUMOReal getCO2Emissions(const std::string& typeID) const;
-        SUMOReal getCOEmissions(const std::string& typeID) const;
-        SUMOReal getHCEmissions(const std::string& typeID) const;
-        SUMOReal getPMxEmissions(const std::string& typeID) const;
-        SUMOReal getNOxEmissions(const std::string& typeID) const;
-        SUMOReal getFuelConsumption(const std::string& typeID) const;
-        SUMOReal getNoiseEmission(const std::string& typeID) const;
-        unsigned int getBestLanes(const std::string& typeID) const;
-        unsigned int getStopState(const std::string& typeID) const;
-        SUMOReal getLength(const std::string& typeID) const;
-        SUMOReal getMaxSpeed(const std::string& typeID) const;
-        SUMOReal getAccel(const std::string& typeID) const;
-        SUMOReal getDecel(const std::string& typeID) const;
-        SUMOReal getTau(const std::string& typeID) const;
-        SUMOReal getImperfection(const std::string& typeID) const;
-        SUMOReal getSpeedFactor(const std::string& typeID) const;
-        SUMOReal getSpeedDeviation(const std::string& typeID) const;
-        std::string getVClass(const std::string& typeID) const;
-        std::string getEmissionClass(const std::string& typeID) const;
-        std::string getShapeClass(const std::string& typeID) const;
-        SUMOReal getMinGap(const std::string& typeID) const;
-        SUMOReal getWidth(const std::string& typeID) const;
-        SUMOReal getWaitingTime(const std::string& typeID) const;
+        SUMOReal getSpeed(const std::string& vehicleID) const;
+        TraCIPosition getPosition(const std::string& vehicleID) const;
+        SUMOReal getAngle(const std::string& vehicleID) const;
+        std::string getRoadID(const std::string& vehicleID) const;
+        std::string getLaneID(const std::string& vehicleID) const;
+        unsigned int getLaneIndex(const std::string& vehicleID) const;
+        std::string getTypeID(const std::string& vehicleID) const;
+        std::string getRouteID(const std::string& vehicleID) const;
+        unsigned int getRouteIndex(const std::string& vehicleID) const;
+        std::vector<std::string> getEdges(const std::string& vehicleID) const;
+        TraCIColor getColor(const std::string& vehicleID) const;
+        SUMOReal getLanePosition(const std::string& vehicleID) const;
+        SUMOReal getCO2Emission(const std::string& vehicleID) const;
+        SUMOReal getCOEmission(const std::string& vehicleID) const;
+        SUMOReal getHCEmission(const std::string& vehicleID) const;
+        SUMOReal getPMxEmission(const std::string& vehicleID) const;
+        SUMOReal getNOxEmission(const std::string& vehicleID) const;
+        SUMOReal getFuelConsumption(const std::string& vehicleID) const;
+        SUMOReal getNoiseEmission(const std::string& vehicleID) const;
+        SUMOReal getElectricityConsumption(const std::string& vehicleID) const;
+        unsigned int getSignalStates(const std::string& vehicleID) const;
+        SUMOReal getWaitingTime(const std::string& vehicleID) const;
+        std::vector<NextTLSData> getNextTLS(const std::string& vehID) const;
+        int getSpeedMode(const std::string& vehicleID) const;
+        SUMOReal getSlope(const std::string& vehicleID) const;
 
-        void moveTo(const std::string& typeID, const std::string& laneID, SUMOReal position) const;
-        void slowDown(const std::string& typeID, SUMOReal speed, int duration) const;
-        void setSpeed(const std::string& typeID, SUMOReal speed) const;
+        /* /// not yet implemented
+        SUMOReal getCO2Emissions(const std::string& vehicleID) const;
+        SUMOReal getCOEmissions(const std::string& vehicleID) const;
+        SUMOReal getHCEmissions(const std::string& vehicleID) const;
+        SUMOReal getPMxEmissions(const std::string& vehicleID) const;
+        SUMOReal getNOxEmissions(const std::string& vehicleID) const;
+        SUMOReal getFuelConsumption(const std::string& vehicleID) const;
+        SUMOReal getNoiseEmission(const std::string& vehicleID) const;
+        unsigned int getBestLanes(const std::string& vehicleID) const;
+        unsigned int getStopState(const std::string& vehicleID) const;
+        SUMOReal getLength(const std::string& vehicleID) const;
+        SUMOReal getMaxSpeed(const std::string& vehicleID) const;
+        SUMOReal getAccel(const std::string& vehicleID) const;
+        SUMOReal getDecel(const std::string& vehicleID) const;
+        SUMOReal getTau(const std::string& vehicleID) const;
+        SUMOReal getImperfection(const std::string& vehicleID) const;
+        SUMOReal getSpeedFactor(const std::string& vehicleID) const;
+        SUMOReal getSpeedDeviation(const std::string& vehicleID) const;
+        std::string getVClass(const std::string& vehicleID) const;
+        std::string getEmissionClass(const std::string& vehicleID) const;
+        std::string getShapeClass(const std::string& vehicleID) const;
+        SUMOReal getMinGap(const std::string& vehicleID) const;
+        SUMOReal getWidth(const std::string& vehicleID) const;
+        */
+
+        void add(const std::string& vehicleID,
+                 const std::string& routeID,
+                 const std::string& typeID = "DEFAULT_VEHTYPE",
+                 std::string depart = "-1",
+                 const std::string& departLane = "first",
+                 const std::string& departPos = "base",
+                 const std::string& departSpeed = "0",
+                 const std::string& arrivalLane = "current",
+                 const std::string& arrivalPos = "max",
+                 const std::string& arrivalSpeed = "current",
+                 const std::string& fromTaz = "",
+                 const std::string& toTaz = "",
+                 const std::string& line = "",
+                 int personCapacity = 0,
+                 int personNumber = 0) const;
+
+        void moveTo(const std::string& vehicleID, const std::string& laneID, SUMOReal position) const;
+        void moveToXY(const std::string& vehicleID, const std::string& edgeID, const int lane, const SUMOReal x, const SUMOReal y, const SUMOReal angle, const int keepRoute) const;
+        void slowDown(const std::string& vehicleID, SUMOReal speed, int duration) const;
+        void setSpeed(const std::string& vehicleID, SUMOReal speed) const;
+        void remove(const std::string& vehicleID, char reason = REMOVE_VAPORIZED) const;
 
     private:
         /// @brief invalidated copy constructor
@@ -724,6 +800,33 @@ public:
 
     };
 
+    /** @class PersonScope
+     * * @brief Scope for interaction with vehicles
+     * */
+    class PersonScope : public TraCIScopeWrapper {
+    public:
+        PersonScope(TraCIAPI& parent) : TraCIScopeWrapper(parent) {}
+        virtual ~PersonScope() {}
+
+        std::vector<std::string> getIDList() const;
+        unsigned int getIDCount() const;
+        SUMOReal getSpeed(const std::string& typeID) const;
+        TraCIPosition getPosition(const std::string& typeID) const;
+        std::string getRoadID(const std::string& typeID) const;
+        std::string getTypeID(const std::string& typeID) const;
+        SUMOReal getWaitingTime(const std::string& typeID) const;
+        std::string getNextEdge(const std::string& typeID) const;
+
+    private:
+        /// @brief invalidated copy constructor
+        PersonScope(const PersonScope& src);
+
+        /// @brief invalidated assignment operator
+        PersonScope& operator=(const PersonScope& src);
+    };
+
+
+
 public:
     /// @brief Scope for interaction with edges
     EdgeScope edge;
@@ -751,6 +854,8 @@ public:
     VehicleTypeScope vehicletype;
     /// @brief Scope for interaction with vehicles
     VehicleScope vehicle;
+    /// @brief Scope for interaction with persons
+    PersonScope person;
 
 
 protected:
@@ -809,6 +914,9 @@ protected:
     /// @}
 
 
+    void send_commandMoveToXY(const std::string& vehicleID, const std::string& edgeID, const int lane,
+                              const SUMOReal x, const SUMOReal y, const SUMOReal angle, const int keepRoute) const;
+
 
     /// @name Command sending methods
     /// @{
@@ -821,17 +929,33 @@ protected:
      */
     void check_resultState(tcpip::Storage& inMsg, int command, bool ignoreCommandId = false, std::string* acknowledgement = 0) const;
 
-    void check_commandGetResult(tcpip::Storage& inMsg, int command, int expectedType = -1, bool ignoreCommandId = false) const;
+    /** @brief Validates the result state of a command
+     * @return The command Id
+     */
+    int check_commandGetResult(tcpip::Storage& inMsg, int command, int expectedType = -1, bool ignoreCommandId = false) const;
 
     void processGET(tcpip::Storage& inMsg, int command, int expectedType, bool ignoreCommandId = false) const;
     /// @}
 
+    void readVariableSubscription(tcpip::Storage& inMsg);
+    void readContextSubscription(tcpip::Storage& inMsg);
+    void readVariables(tcpip::Storage& inMsg, const std::string& objectID, int variableCount, SubscribedValues& into);
+
+    template <class T>
+    static inline std::string toString(const T& t, std::streamsize accuracy = OUTPUT_ACCURACY) {
+        std::ostringstream oss;
+        oss.setf(std::ios::fixed , std::ios::floatfield);
+        oss << std::setprecision(accuracy);
+        oss << t;
+        return oss.str();
+    }
 
 protected:
     /// @brief The socket
     tcpip::Socket* mySocket;
 
-
+    SubscribedValues mySubscribedValues;
+    SubscribedContextValues mySubscribedContextValues;
 };
 
 
