@@ -4,12 +4,12 @@
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    February 2012
-/// @version $Id: CHRouter.h 18467 2015-05-29 03:50:41Z behrisch $
+/// @version $Id: CHRouter.h 20482 2016-04-18 20:49:42Z behrisch $
 ///
 // Shortest Path search using a Contraction Hierarchy
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -107,12 +107,12 @@ public:
     class EdgeInfo {
     public:
         /// Constructor
-        EdgeInfo(size_t id) :
-            edge(E::dictionary(id)),
+        EdgeInfo(const E* e) :
+            edge(e),
             traveltime(std::numeric_limits<SUMOReal>::max()),
             prev(0),
-            visited(false)
-        {}
+            visited(false) {
+        }
 
         /// The current edge
         const E* edge;
@@ -146,11 +146,11 @@ public:
     class Unidirectional: public PF {
     public:
         /// @brief Constructor
-        Unidirectional(size_t numEdges, bool forward):
+        Unidirectional(const std::vector<E*>& edges, bool forward):
             myAmForward(forward),
             myVehicle(0) {
-            for (size_t i = 0; i < numEdges; i++) {
-                myEdgeInfos.push_back(EdgeInfo(i));
+            for (typename std::vector<E*>::const_iterator i = edges.begin(); i != edges.end(); ++i) {
+                myEdgeInfos.push_back(EdgeInfo(*i));
             }
         }
 
@@ -318,21 +318,22 @@ public:
      *            hierarchy is tailored to the vClass of the defaultVehicle
      * @note: defaultVehicle is not transient and must be kept after constructor finishes
      */
-    CHRouter(size_t numEdges, bool unbuildIsWarning, Operation operation,
+    CHRouter(const std::vector<E*>& edges, bool unbuildIsWarning, Operation operation,
              const SUMOVehicleClass svc,
              SUMOTime weightPeriod,
              bool validatePermissions):
         SUMOAbstractRouter<E, V>(operation, "CHRouter"),
+        myEdges(edges),
         myErrorMsgHandler(unbuildIsWarning ? MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance()),
-        myForwardSearch(numEdges, true),
-        myBackwardSearch(numEdges, false),
+        myForwardSearch(edges, true),
+        myBackwardSearch(edges, false),
         mySPTree(new SPTree<CHInfo, CHConnection>(4, validatePermissions)),
         myWeightPeriod(weightPeriod),
         myValidUntil(0),
         mySVC(svc),
         myUpdateCount(0) {
-        for (size_t i = 0; i < numEdges; i++) {
-            myCHInfos.push_back(CHInfo(i));
+        for (typename std::vector<E*>::const_iterator i = edges.begin(); i != edges.end(); ++i) {
+            myCHInfos.push_back(CHInfo(*i));
         }
     }
 
@@ -342,8 +343,8 @@ public:
     }
 
 
-    virtual SUMOAbstractRouter<E, V>* clone() const {
-        return new CHRouter<E, V, PF>(myCHInfos.size(), myErrorMsgHandler == MsgHandler::getWarningInstance(), this->myOperation,
+    virtual SUMOAbstractRouter<E, V>* clone() {
+        return new CHRouter<E, V, PF>(myEdges, myErrorMsgHandler == MsgHandler::getWarningInstance(), this->myOperation,
                                       mySVC, myWeightPeriod, mySPTree->validatePermissions());
     }
 
@@ -351,7 +352,7 @@ public:
      * @note: since the contracted graph is static (weights averaged over time)
      * the computed routes only approximated shortest paths in the real graph
      * */
-    virtual void compute(const E* from, const E* to, const V* const vehicle,
+    virtual bool compute(const E* from, const E* to, const V* const vehicle,
                          SUMOTime msTime, Result& into) {
         assert(from != 0 && to != 0);
         assert(mySPTree->validatePermissions() || vehicle->getVClass() == mySVC || mySVC == SVC_IGNORING);
@@ -372,6 +373,7 @@ public:
         bool continueBackward = true;
         int num_visited_fw = 0;
         int num_visited_bw = 0;
+        bool result = true;
         while (continueForward || continueBackward) {
             if (continueForward) {
                 continueForward = myForwardSearch.step(myBackwardSearch, minTTSeen, meeting);
@@ -385,12 +387,14 @@ public:
         if (minTTSeen < std::numeric_limits<SUMOReal>::max()) {
             buildPathFromMeeting(meeting, into);
         } else {
-            myErrorMsgHandler->inform("No connection between '" + from->getID() + "' and '" + to->getID() + "' found.");
+            myErrorMsgHandler->inform("No connection between edge '" + from->getID() + "' and edge '" + to->getID() + "' found.");
+            result = false;
         }
 #ifdef CHRouter_DEBUG_QUERY_PERF
         std::cout << "visited " << num_visited_fw + num_visited_bw << " edges (" << num_visited_fw << "," << num_visited_bw << ") ,final path length: " + toString(into.size()) + ")\n";
 #endif
         this->endQuery(num_visited_bw + num_visited_fw);
+        return result;
     }
 
 
@@ -464,15 +468,15 @@ public:
     class CHInfo {
     public:
         /// @brief Constructor
-        CHInfo(size_t id) :
-            edge(E::dictionary(id)),
+        CHInfo(E* e) :
+            edge(e),
             contractedNeighbors(0),
             rank(-1),
             level(0),
             underlyingTotal(0),
             visited(false),
-            traveltime(std::numeric_limits<SUMOReal>::max())
-        {}
+            traveltime(std::numeric_limits<SUMOReal>::max()) {
+        }
 
         /// @brief recompute the contraction priority and report whether it changed
         bool updatePriority(SPTree<CHInfo, CHConnection>* spTree) {
@@ -831,6 +835,9 @@ private:
     }
 
 private:
+    /// @brief all edges with numerical ids
+    const std::vector<E*>& myEdges;
+
     /// @brief the handler for routing errors
     MsgHandler* const myErrorMsgHandler;
 
