@@ -5,7 +5,7 @@
 /// @author  Axel Wegener
 /// @author  Michael Behrisch
 /// @date    2006-01-24
-/// @version $Id: SUMOVehicleParameter.h 20482 2016-04-18 20:49:42Z behrisch $
+/// @version $Id: SUMOVehicleParameter.h 20687 2016-05-10 11:27:00Z behrisch $
 ///
 // Structure representing possible vehicle parameter
 /****************************************************************************/
@@ -68,6 +68,7 @@ const int VEHPARS_FORCE_REROUTE = 2 << 13;
 const int VEHPARS_PERSON_CAPACITY_SET = 2 << 14;
 const int VEHPARS_PERSON_NUMBER_SET = 2 << 15;
 const int VEHPARS_CONTAINER_NUMBER_SET = 2 << 16;
+const int VEHPARS_DEPARTPOSLAT_SET = 2 << 16;
 
 const int STOP_INDEX_END = -1;
 const int STOP_INDEX_FIT = -2;
@@ -150,6 +151,32 @@ enum DepartPosDefinition {
 };
 
 
+/**
+ * @enum DepartPosDefinition
+ * @brief Possible ways to choose the departure position
+ */
+enum DepartPosLatDefinition {
+    /// @brief No information given; use default
+    DEPART_POSLAT_DEFAULT,
+    /// @brief The position is given
+    DEPART_POSLAT_GIVEN,
+    /// @brief At the rightmost side of the lane
+    DEPART_POSLAT_RIGHT,
+    /// @brief At the center of the lane
+    DEPART_POSLAT_CENTER,
+    /// @brief At the leftmost side of the lane
+    DEPART_POSLAT_LEFT,
+    /// @brief The lateral position is chosen randomly
+    DEPART_POSLAT_RANDOM,
+    /// @brief A free lateral position is chosen
+    DEPART_POSLAT_FREE,
+    /// @brief If a fixed number of random choices fails, a free lateral position is chosen
+    DEPART_POSLAT_RANDOM_FREE,
+    /// @brief Tag for the last element in the enum for safe int casting
+    DEPART_POSLAT_DEF_MAX
+};
+
+
 /**
  * @enum DepartSpeedDefinition
  * @brief Possible ways to choose the departure speed
@@ -310,6 +337,19 @@ public:
                                SUMOReal& pos, DepartPosDefinition& dpd, std::string& error);
 
 
+    /** @brief Validates a given departPosLat value
+     * @param[in] val The departPosLat value to parse
+     * @param[in] element The name of the type of the parsed element, for building the error message
+     * @param[in] id The id of the parsed element, for building the error message
+     * @param[out] pos The parsed position, if given
+     * @param[out] dpd The parsed departPos definition
+     * @param[out] error Error message, if an error occures
+     * @return Whether the given value is a valid departPos definition
+     */
+    static bool parseDepartPosLat(const std::string& val, const std::string& element, const std::string& id,
+                                  SUMOReal& pos, DepartPosLatDefinition& dpd, std::string& error);
+
+
     /** @brief Validates a given departSpeed value
      * @param[in] val The departSpeed value to parse
      * @param[in] element The name of the type of the parsed element, for building the error message
@@ -398,6 +438,10 @@ public:
     SUMOReal departPos;
     /// @brief Information how the vehicle shall choose the departure position
     DepartPosDefinition departPosProcedure;
+    /// @brief (optional) The lateral position the vehicle shall depart from
+    SUMOReal departPosLat;
+    /// @brief Information how the vehicle shall choose the lateral departure position
+    DepartPosLatDefinition departPosLatProcedure;
     /// @brief (optional) The initial speed of the vehicle
     SUMOReal departSpeed;
     /// @brief Information how the vehicle's initial speed shall be chosen
@@ -496,6 +540,9 @@ public:
     /// @brief List of the stops the vehicle will make
     std::vector<Stop> stops;
 
+    /// @brief List of the via-edges the vehicle must visit
+    std::vector<std::string> via;
+
     /// @brief The static number of persons in the vehicle when it departs (not including boarding persons)
     unsigned int personNumber;
 
